/*-----------------------------------------------------------------------*/
/* Low level disk I/O module skeleton for FatFs     (C)ChaN, 2007        */
/*-----------------------------------------------------------------------*/
/* This is a stub disk I/O module that acts as front end of the existing */
/* disk I/O modules and attach it to FatFs module with common interface. */
/*-----------------------------------------------------------------------*/
#include "../cantus_lib_config.h"
#include "cantus.h"
#include "fatfs/diskio.h"

#ifndef CONFIG_FAT_NAND_SUPPORT
#define nand_init(...)	0
#define nand_readsects(...)	0
#define nand_updatesects(...)	0
#define nand_get_sectorcount()	0
#endif

//avoid SDCARD api linking 
#ifndef CONFIG_FAT_SDCARD_SUPPORT
#define SDC_Init(...)	0
#define SDC_Read(...)	0
#define SDC_Write(...)	0
#define SDC_GetSectorCount()	0
#endif


DSTATUS disk_initialize (
						 BYTE drv				/* Physical drive nmuber (0..) */
						 )
{
	if(drv == DRIVE_NAND )
	{
		if(nand_init())
			return RES_OK;
	}
	else if(drv == DRIVE_SDCARD)
	{
		if(SDC_Init())
			return RES_OK;
	}
	else
		debugprintf("%d driver not supported\r\n",drv);
	return RES_ERROR;
}


/*-----------------------------------------------------------------------*/
/* Return Disk Status                                                    */

DSTATUS disk_status (
					 BYTE drv		/* Physical drive nmuber (0..) */
					 )
{
	return RES_OK;
}



/*-----------------------------------------------------------------------*/
/* Read Sector(s)                                                        */

DRESULT disk_read (
				   BYTE drv,		/* Physical drive nmuber (0..) */
				   BYTE *buff,		/* Data buffer to store read data */
				   DWORD sector,	/* Sector address (LBA) */
				   BYTE count		/* Number of sectors to read (1..255) */
				   )
{
	if(drv == DRIVE_NAND )
	{
		if(nand_readsects(sector,buff,count))
			return RES_OK;
	}
	else if(drv == DRIVE_SDCARD )
	{
		if(SDC_Read(buff,sector*512,count*512))
		return RES_OK;
	}
	return RES_ERROR;
}



/*-----------------------------------------------------------------------*/
/* Write Sector(s)                                                       */

DRESULT disk_write (
					BYTE drv,			/* Physical drive nmuber (0..) */
					const BYTE *buff,	/* Data to be written */
					DWORD sector,		/* Sector address (LBA) */
					BYTE count			/* Number of sectors to write (1..255) */
					)
{
	if(drv == DRIVE_NAND)
	{
		if(nand_updatesects(sector,(void*)buff,count))
			return RES_OK;
	}
	else if(drv == DRIVE_SDCARD)
	{
		if( SDC_Write((void*)buff,sector*512,count*512) )
			return RES_OK;
	}
	return RES_ERROR;
}



/*-----------------------------------------------------------------------*/
/* Miscellaneous Functions                                               */

DRESULT disk_ioctl (
	BYTE drv,		/* Physical drive nmuber (0) */
	BYTE ctrl,		/* Control code */
	void *buff		/* Buffer to send/receive data block */
)
{
	DRESULT res =RES_PARERR ;

	switch (ctrl) {
	case CTRL_SYNC:
		if(drv==DRIVE_NAND)
#ifdef CONFIG_FAT_NAND_SUPPORT			
			nand_flushdata();
#else
			break;
#endif		
		res = RES_OK;
		break;

	case GET_SECTOR_COUNT:
		if(drv==DRIVE_NAND)
		{
			*(DWORD*)buff = nand_get_sectorcount();
		}
		else if(drv==DRIVE_SDCARD)
		{
			*(DWORD*)buff = SDC_GetSectorCount();
		}
		res = RES_OK;
		break;

	case GET_SECTOR_SIZE:
		*(WORD*)buff = 512;
		res = RES_OK;
		break;

	case GET_BLOCK_SIZE:
		*(DWORD*)buff = 1;
		res = RES_OK;
		break;
	}

	return res;
}
/*
Currnet time is returned with packed into a DWORD value. The bit field is as follows:

bit31:25 
Year from 1980 (0..127) 
bit24:21 
Month (1..12) 
bit20:16 
Day in month(1..31) 
bit15:11 
Hour (0..23) 
bit10:5 
Minute (0..59) 
bit4:0 
Second / 2 (0..29) 
*/
DWORD get_fattime (void)
{
	
	return 0;
}
