#include <string.h>
#include "cantus_lib_config.h"
#include "cantus.h"

//#define NAND_DEBUG
#ifdef NAND_DEBUG
#define NAND_DEBUGPRINTF	debugprintf
#else
#define NAND_DEBUGPRINTF(...)
#endif
// Device Code Define
#define K9F1208U	0x76EC
#define K9F1G08U	0xF1EC
#define K9F2G08U	0xDAEC
#define K9F4G08U	0xDCEC
#define K9F8G08U	0xD3EC

#define BLOCK2PAGE(BlockNum) (BlockNum * NandInfo.pageperblock)

volatile U32 g_dummy;//for dummy read

typedef struct _tagNand
{
	U16 id;
	U32 pagesize;
	U32 blocksize;
	U32 pageperblock;
	U32 blockcnt;
	U32 sparesize;
	U32 maxbadblockcnt;
	U32 rowaddrlen;
	U32 datastartblock;
}NANDTYPE;

static NANDTYPE NandInfo=
{
	.id=K9F1208U,
	.pagesize=512,
	.blocksize=512*32,
	.pageperblock=32,
	.blockcnt=4096-4,//-4(for copyback)
	.sparesize=16,
	.maxbadblockcnt=33,
	.rowaddrlen=9,
	.datastartblock=4//-4(for copyback)
};
U32 badblocktable[33];
	
	
static void WaitBusy()
{
	// Check Busy End
	while ( !(*(volatile U8*)R_NFMSTAT & 0x01) );
	while ( !(*(volatile U8*)R_NFMSTAT & 0x02) ); // for "trr" timing,see Nand-Flash data sheet
	int i;
	for( i=0;i<1;i++);
}
U16 nand_id_read(void)
{
	U16 id ;
	U16 id_dummy;

	*(volatile char *)R_NFMCMD  = NAND_COMM_RID ;  // Read 

	*(volatile char *)R_NFMADDR  = 0x00 ;  // ADDRESS 

	id = (U16)*(volatile U8*)R_NFMDATA ;
	id += ((U16)*(volatile U8*)R_NFMDATA)<<8 ;

	id_dummy = (U16)*(volatile U8*)R_NFMDATA ;
	id_dummy += ((U16)*(volatile U8*)R_NFMDATA)<<8 ;

	return id;
}


__inline__ static void setpageaddress(U32 pagenum,U32 offset)
{
	int i;
	int k;
	U8 mask;
	U8 validbit;
	*(volatile U8 *)R_NFMADDR  = offset&0xff ;  // A0-A7
	if(NandInfo.pagesize==2048)
		*(volatile U8 *)R_NFMADDR  = offset>>8 ;  

	for(i=0;i<NandInfo.rowaddrlen;i+=8)
	{
		mask=0;
		validbit = (NandInfo.rowaddrlen-i)>8?8:(NandInfo.rowaddrlen-i);
		for(k=0;k<validbit;k++)
			mask |= (1<<k);
		*(volatile U8 *)R_NFMADDR  = (U8)(pagenum  & mask) ;  // Row Address
		pagenum = pagenum>>8;
	}
}
__inline__ static void seteraseblockaddr(U32 blockaddr)
{
	int i;
	int k;
	U8 mask;
	U8 validbit;
	for(i=0;i<NandInfo.rowaddrlen;i+=8)
	{
		mask=0;
		validbit = (NandInfo.rowaddrlen-i)>8?8:(NandInfo.rowaddrlen-i);
		for(k=0;k<validbit;k++)
			mask |= (1<<k);
		*(volatile U8 *)R_NFMADDR  = (U8)(blockaddr  & mask) ;  // Row Address
		blockaddr = blockaddr>>8;
	}
}
__inline__ static void setspareaddr(U32 pagenum)
{
	if(NandInfo.pagesize==512)
		setpageaddress(pagenum,0);
	else
	{
		int i;
		int k;
		U8 mask;
		U8 validbit;
		*(volatile U8 *)R_NFMADDR  = 0 ;  
		*(volatile U8 *)R_NFMADDR  = 0x8 ;  

		for(i=0;i<NandInfo.rowaddrlen;i+=8)
		{
			mask=0;
			validbit = (NandInfo.rowaddrlen-i)>8?8:(NandInfo.rowaddrlen-i);
			for(k=0;k<validbit;k++)
				mask |= (1<<k);
			*(volatile U8 *)R_NFMADDR  = (U8)(pagenum  & mask) ;  // Row Address
			pagenum = pagenum>>8;
		}
	}
}


BOOL nand_phy_readpage(U32 pageaddr, void* buffer)
{
	U32 i;
	U8* buf = (U8*)buffer;
	g_dummy = *R_NFMSTAT; //clear busy

	*(volatile char *)R_NFMCMD  = NAND_COMM_READ1 ;  // Read 
	setpageaddress(pageaddr,0);
	if(NandInfo.pagesize == 2048)
		*(volatile char *)R_NFMCMD  = NAND_COMM_READ2 ;  // Read 
	// Check Busy End
	WaitBusy();
	if((U32)(&buf[0])&3) //not 4byte boundry
	{
		for(i=0;i<NandInfo.pagesize;)
		{
			buf[i] = *(volatile U8*)R_NFMDATA;
			buf[i+1] = *(volatile U8*)R_NFMDATA;
			buf[i+2] = *(volatile U8*)R_NFMDATA;
			buf[i+3] = *(volatile U8*)R_NFMDATA;
			i+=4;
		}
	}
	else
	{
		for(i=0;i<NandInfo.pagesize;)
		{
			*(U32*)(buf+i) = *R_NFMDATA;
			*(U32*)(buf+i+4) = *R_NFMDATA;
			*(U32*)(buf+i+8) = *R_NFMDATA;
			*(U32*)(buf+i+12) = *R_NFMDATA;
			i+=16;
		}		
	}
		
	while ( !(*(volatile U8*)R_NFMSTAT & 0x02) ); // for "trr" timing,see Nand-Flash data sheet
	return TRUE;
}
//read only 512byte for FAT
BOOL nand_phy_readsect(U32 sector,void* buffer)
{
	U8* buf = (U8*)buffer;
	if(NandInfo.pagesize==512)
		return nand_phy_readpage(sector,buf);
	else
	{
		int i;
		U32 pagenum;
		U32 offset;
		pagenum = sector/4;
		offset = (sector%4)*512;
		
		g_dummy = *R_NFMSTAT; //clear busy

		*(volatile char *)R_NFMCMD  = NAND_COMM_READ1 ;  // Read 
		setpageaddress(pagenum,offset);

		*(volatile char *)R_NFMCMD  = NAND_COMM_READ2 ;  // Read 
		// Check Busy End
		WaitBusy();
		if((U32)(&buf[0])&3) //not 4byte boundry
		{
			for(i=0;i<512;)
			{
				buf[i] = *(volatile U8*)R_NFMDATA;
				buf[i+1] = *(volatile U8*)R_NFMDATA;
				buf[i+2] = *(volatile U8*)R_NFMDATA;
				buf[i+3] = *(volatile U8*)R_NFMDATA;
				i+=4;
			}
		}
		else
		{
			for(i=0;i<512;)
			{
				*(U32*)(buf+i) = *R_NFMDATA;
				*(U32*)(buf+i+4) = *R_NFMDATA;
				*(U32*)(buf+i+8) = *R_NFMDATA;
				*(U32*)(buf+i+12) = *R_NFMDATA;
				i+=16;
			}		
		}

		while ( !(*(volatile U8*)R_NFMSTAT & 0x02) ); // for "trr" timing,see Nand-Flash data sheet
		return TRUE;
	}

}

BOOL nand_phy_readspare(U32 pageaddr,void *buffer,U32 len)
{
	int i;
	char* buf;
 	g_dummy = *R_NFMSTAT; //clear busy
	buf = (U8*)buffer;
	if(NandInfo.pagesize == 512)
	{
		*(volatile U8 *)R_NFMCMD  = NAND_COMM_SPAREREAD ;  // Read 
		setspareaddr(pageaddr);
	}
	else
	{
		*(volatile U8 *)R_NFMCMD  = NAND_COMM_READ1 ; 
		setspareaddr(pageaddr);
		*(volatile U8 *)R_NFMCMD  = NAND_COMM_READ2 ; 
	}
	
	// Check Busy End
	WaitBusy();
	if(len > NandInfo.sparesize)
		len = NandInfo.sparesize;
	for(i=0;i<len;i++)
		buf[i] = *(volatile U8*)R_NFMDATA;
	// Check Busy End
	while ( !(*(volatile U8*)R_NFMSTAT & 0x02) ); // for "trr" timing,see Nand-Flash data sheet
	return TRUE;
}

BOOL nand_phy_writepage(U32 pageaddr, void* buffer)
{
	int i;
	U8* buf = (U8*)buffer;
	g_dummy = *R_NFMSTAT; //clear busy
	if(NandInfo.pagesize == 512)
		*(volatile U8*)R_NFMCMD  = 0x0 ; 
	*(volatile U8*)R_NFMCMD  = NAND_COMM_PPRO1 ; 

	setpageaddress(pageaddr,0);
	if((U32)(&buf[0])&3)
	{
		for(i=0;i<NandInfo.pagesize;)
		{
			*(volatile U8*)R_NFMDATA = buf[i];
			*(volatile U8*)R_NFMDATA = buf[i+1];
			*(volatile U8*)R_NFMDATA = buf[i+2];
			*(volatile U8*)R_NFMDATA = buf[i+3];
			i+=4;
		}
	}
	else
	{
		U32* lbuf = (U32*)&buf[0];
		for(i=0;i<NandInfo.pagesize/4;)
		{
			*R_NFMDATA = lbuf[i];
			*R_NFMDATA = lbuf[i+1];
			*R_NFMDATA = lbuf[i+2];
			*R_NFMDATA = lbuf[i+3];
			i+=4;
		}
	}

	*(volatile U8*)R_NFMCMD = NAND_COMM_PPRO2 ; // Confirm
	WaitBusy();

	*(volatile U8*)R_NFMCMD  = NAND_STATUS ; // Command
	if(*(volatile U8*)R_NFMDATA & 1)
		return FALSE;
	return TRUE;	

}

BOOL nand_phy_writespare(U32 pageaddr,void* buffer,U32 len)
{
	int i;
	U8* buf = (U8*)buffer;
 	g_dummy = *R_NFMSTAT; //clear busy

	if(NandInfo.pagesize == 512)
	{
		*(volatile U8 *)R_NFMCMD  = NAND_COMM_SPAREREAD ;  // Read 
		*(volatile U8*)R_NFMCMD  = NAND_COMM_PPRO1 ; 
		setspareaddr(pageaddr);
	}
	else
	{
		*(volatile U8 *)R_NFMCMD  = NAND_COMM_PPRO1 ;  // Read 
		setspareaddr(pageaddr);
	}
	
	if(len > NandInfo.sparesize)
		len = NandInfo.sparesize;
	for(i=0;i<len;i++)
	{
		*(volatile U8*)R_NFMDATA = buf[i];
	}
	*(volatile U8*)R_NFMCMD = NAND_COMM_PPRO2 ; // Confirm
    // Check Busy End
    WaitBusy();
	
	*(volatile U8*)R_NFMCMD  = NAND_STATUS ; // Command
	if(*(volatile U8*)R_NFMDATA & 1)
		return FALSE;
	return TRUE;	
}

BOOL nand_phy_eraseblock(U32 blocknum)
{
	U32 pagenum = BLOCK2PAGE(blocknum);
	g_dummy = *R_NFMSTAT; //clear busy
	*(volatile char *)R_NFMCMD  = NAND_COMM_BERS1 ; 
	seteraseblockaddr(pagenum);
	*(volatile U8*)R_NFMCMD  = NAND_COMM_BERS2 ; // Command	
	// Check Busy End
	WaitBusy();

	*(volatile U8*)R_NFMCMD  = NAND_STATUS ; // Command
	if(*(volatile U8*)R_NFMDATA & 1)
		return FALSE;//bad?
	return TRUE;
}


#define BI_GOOD	0xff //not used block
#define BI_USED	0x7f //allocated block
#define BI_NOTUSED	0x0f //need for erase to check bad?
#define BI_BAD 0x00 //can't use

#ifdef NAND_FLASH_K9F8G08U
#define MAX_BLOCK_CNT 8192
#else
#define MAX_BLOCK_CNT 4096
#endif

typedef struct _tagSpare
{
	U8 LSN[3];
	U8 paritybit;
	U8 reserve; // reserve[0] is used parity
	U8 BadInfo;
	U8 Ecc[3]; 
	U8 SEcc[2];
	U8 reserve2[5];
	
}SPAREDATA; //small page


typedef struct _tagSpare2
{
	U8 BadInfo;
	U8 paritybit;// reserve is used parity
	U8 LSN[3]; // Logical Number
	U8 reserve2[3];
	U8 Ecc[3];
	U8 SEcc[2];
	U8 reserve3[3];
}SPAREDATA2;//large page

static U8 BlockInfoTable[MAX_BLOCK_CNT];
static U16 PhyBlockTable[MAX_BLOCK_CNT];

static BOOL nand_make_bbt()
{
	int i;
	int pageperblock = NandInfo.pageperblock;
	U32 logicalBlocknum;
	U8 sparedatabuf[16];
	U8 sparedatabuf2[16];
	memset(PhyBlockTable,0xff,MAX_BLOCK_CNT*2);
	for(i=NandInfo.datastartblock;i<NandInfo.blockcnt;i++)
	{
		nand_phy_readspare(i*pageperblock,sparedatabuf,16);
		nand_phy_readspare(i*pageperblock+1,sparedatabuf2,16);
			
		if(NandInfo.pagesize==512)
		{
			SPAREDATA* sparedata = (SPAREDATA*)sparedatabuf;
			SPAREDATA* sparedata2 = (SPAREDATA*)sparedatabuf2;
			if( (sparedata->BadInfo != 0xff) || (sparedata2->BadInfo != 0xff) )
			{
				BlockInfoTable[i]=BI_BAD;	
				debugprintf("%dblock is bad block\r\n",i);
				continue;
			}
			//make logical block number
			logicalBlocknum = (U32)sparedata->LSN[0] + ((U32)sparedata->LSN[1]<<8) + ((U32)sparedata->LSN[2]<<16);
			if(logicalBlocknum==0xffffff)
			{
				BlockInfoTable[i]=BI_GOOD;
			}
			else
			{
				U8 paritybit = 0;
				int k;
				//check parity bit
				for(k=0;k<24;k++)
				{
					if(logicalBlocknum & (1<<k))
					{
						if(paritybit)
							paritybit = 0;
						else
							paritybit = 1;
					}
				}
				if(sparedata->paritybit != paritybit)
				{
					BlockInfoTable[i]=BI_NOTUSED;					
				}
				else
				{
					PhyBlockTable[logicalBlocknum]=i;			
					BlockInfoTable[i]=BI_USED;
					//NAND_DEBUGPRINTF("P-%d : L-%d\r\n",i,logicalBlocknum);
				}
			}
		}
		else //large block
		{
			SPAREDATA2* sparedata = (SPAREDATA2*)sparedatabuf;
			SPAREDATA2* sparedata2 = (SPAREDATA2*)sparedatabuf2;
			if( (sparedata->BadInfo != 0xff) || (sparedata2->BadInfo != 0xff) )
			{
				BlockInfoTable[i]=BI_BAD;
				debugprintf("%d is bad block\r\n",i);
				continue;
			}
			//make logical block number
			logicalBlocknum = (U32)sparedata->LSN[0] + ((U32)sparedata->LSN[1]<<8) + ((U32)sparedata->LSN[2]<<16);
			if(logicalBlocknum==0xffffff)
			{
				BlockInfoTable[i]=BI_GOOD;
			}
			else
			{
				U8 paritybit = 0;
				int k;
				//check parity bit
				for(k=0;k<24;k++)
				{
					if(logicalBlocknum & (1<<k))
					{
						if(paritybit)
							paritybit = 0;
						else
							paritybit = 1;
					}
				}
				if(sparedata->paritybit != paritybit)
				{
					BlockInfoTable[i]=BI_NOTUSED;					
					debugprintf("%d block paritybit error\r\n",i);
				}
				else
				{
					PhyBlockTable[logicalBlocknum]=i;			
					BlockInfoTable[i]=BI_USED;
					//NAND_DEBUGPRINTF("P-%d : L-%d\r\n",i,logicalBlocknum);
				}
			}
		}
	}
	debugstring("make bad block inforamation done\r\n");
	return TRUE;
}

static int nand_get_phy_freeblock(U32 logicalblocknum)
{
	int i;
	int k=logicalblocknum;
	for(i=NandInfo.datastartblock;i<NandInfo.blockcnt;i++)
	{
		if(BlockInfoTable[k]==BI_GOOD)
		{
			if(nand_phy_eraseblock(k)==TRUE)
				return k;
		}
		else if(BlockInfoTable[i]==BI_NOTUSED)
		{
			if(nand_phy_eraseblock(k)==TRUE)
			{
				return k;
			}
			BlockInfoTable[k]=BI_BAD;
		}
		k++;
		if(k>=NandInfo.blockcnt)
			k=NandInfo.datastartblock;
	}
	return -1;
}

static BOOL nand_allocate_block(U32 phynum,U32 lognum)
{
	U8 sparebuf[16]={0,};
	if(phynum > NandInfo.blockcnt || lognum > NandInfo.blockcnt)
		return FALSE;
	
	if(NandInfo.pagesize == 512)
	{
		SPAREDATA* sparedata = (SPAREDATA*)sparebuf;
		sparedata->BadInfo=0xff;
		sparedata->LSN[0] = lognum;
		sparedata->LSN[1] = lognum>>8;
		sparedata->LSN[2] = lognum>>16;
		U8 paritybit = 0;
		int k;
		//check parity bit
		for(k=0;k<24;k++)
		{
			if(lognum & (1<<k))
			{
				if(paritybit)
					paritybit = 0;
				else
					paritybit = 1;
			}
		}
		sparedata->paritybit = paritybit;
	}
	else//2048
	{
		SPAREDATA2* sparedata = (SPAREDATA2*)sparebuf;
		sparedata->BadInfo=0xff;
		sparedata->LSN[0] = lognum;
		sparedata->LSN[1] = lognum>>8;
		sparedata->LSN[2] = lognum>>16;
		U8 paritybit = 0;
		int k;
		//check parity bit
		for(k=0;k<24;k++)
		{
			if(lognum & (1<<k))
			{
				if(paritybit)
					paritybit = 0;
				else
					paritybit = 1;
			}
		}
		sparedata->paritybit = paritybit;
	}
	U32 pagenum = phynum*NandInfo.pageperblock;
	if(nand_phy_writespare(pagenum,sparebuf,16) == FALSE)
	{
		debugprintf("pagenum:%d writespare error\r\n",pagenum);
		return FALSE;
	}
	U16 oldphynum = PhyBlockTable[lognum];
	if(oldphynum !=phynum && oldphynum <NandInfo.blockcnt)
	{
		BlockInfoTable[oldphynum]=BI_NOTUSED;
		nand_phy_eraseblock(oldphynum);
	}
	BlockInfoTable[phynum]=BI_USED;
	PhyBlockTable[lognum]=phynum;

	return TRUE;
}

U32 nand_get_phy_block(U32 lognum)
{
	U32 phyblocknum = PhyBlockTable[lognum];
	if(phyblocknum == 0xffff) //not allocate
	{
		phyblocknum = nand_get_phy_freeblock(lognum);
		if(phyblocknum ==-1)
		{
			NAND_DEBUGPRINTF("failed to get free block\r\n");
			return -1;
		}
		NAND_DEBUGPRINTF("new block allocated P-%d:L-%d\r\n",phyblocknum,lognum);
		nand_allocate_block(phyblocknum,lognum);
	}
	return phyblocknum;
}

BOOL nand_set_sparedata(U32 lognum)
{
	U32 phynum = nand_get_phy_block(lognum);
	return nand_allocate_block(phynum,lognum);
}
static U32 nandcfg=0x7777;
void nand_set_cfg(U32 cfg)
{
	nandcfg = cfg;
}
BOOL nand_init()
{
	*R_PAF3 = ((*R_PAF3) & 0x000f)|0xAAA0;//NAND ctrl
	
	//*R_NFMCFG = 0x1223;
	if(GetAHBclock()>40000000)
		*R_NFMCFG = nandcfg;
	else
		*R_NFMCFG = nandcfg|(1<<20);
		
	U16 id = nand_id_read();
	switch(id)
	{
	case K9F1208U:
		NandInfo.pagesize=512;
		NandInfo.blocksize=512*32;
		NandInfo.pageperblock=32;
		NandInfo.blockcnt=4096;
		NandInfo.sparesize=16;
		NandInfo.maxbadblockcnt=33;
		NandInfo.rowaddrlen=17;
		NandInfo.datastartblock=4;
		break;
	case K9F1G08U:
		NandInfo.pagesize=2048;
		NandInfo.pageperblock=64;
		NandInfo.blockcnt=1024;
		NandInfo.sparesize=64;
		NandInfo.maxbadblockcnt=20;
		NandInfo.rowaddrlen=16;
		NandInfo.datastartblock=1; // first block is used for copyback.
		break;
	case K9F2G08U:
		NandInfo.pagesize=2048;
		NandInfo.pageperblock=64;
		NandInfo.blockcnt=2048;
		NandInfo.sparesize=64;
		NandInfo.maxbadblockcnt=40;
		NandInfo.rowaddrlen=17;
		NandInfo.datastartblock=1;
		break;
	case K9F4G08U:
		NandInfo.pagesize=2048;
		NandInfo.pageperblock=64;
		NandInfo.blockcnt=4096;
		NandInfo.sparesize=64;
		NandInfo.maxbadblockcnt=70;
		NandInfo.rowaddrlen=18;
		NandInfo.datastartblock=1;
		break;	
	#ifdef NAND_FLASH_K9F8G08U
	case K9F8G08U:
		// UNCOMMENT! cantus_lib_config.h #58
		NandInfo.pagesize=2048;
		NandInfo.pageperblock=64;
		NandInfo.blockcnt=8192;
		NandInfo.sparesize=64;
		NandInfo.maxbadblockcnt=160;
		NandInfo.rowaddrlen=19;
		NandInfo.datastartblock=1;
		break;
	#endif
	default:
		debugprintf("nand id(%x) is not supported\r\n",id);
		return FALSE;
	}
	NandInfo.blocksize = NandInfo.pageperblock*NandInfo.pagesize;
	NandInfo.id=id;
	debugprintf("Nand Flash Memory Info:%dMbyte\r\n",NandInfo.blocksize*NandInfo.blockcnt/(1024*1024));
	return nand_make_bbt();
}

U32 nand_get_pagesize()
{
	return NandInfo.pagesize;
}

U32 nand_get_blocksize()
{
	return NandInfo.blocksize;
}
U32 nand_get_pageperblock()
{
	return NandInfo.blocksize/NandInfo.pagesize;
}
U32 nand_get_blockcnt()
{
	return NandInfo.blockcnt;
}
U32 nand_get_memsize_kbyte()
{
	return NandInfo.blocksize/1024 * (NandInfo.blockcnt-NandInfo.maxbadblockcnt);
}
	
U32 nand_get_sectorcount()
{
	return NandInfo.blocksize * NandInfo.blockcnt/512;
}


// Flash Translation Layer(FTL)

U32 nand_get_phy_page(U32 pagenum)
{
	U32 blocknum = pagenum / NandInfo.pageperblock;
	blocknum = nand_get_phy_block(blocknum);
	return (blocknum*NandInfo.pageperblock + (pagenum % NandInfo.pageperblock));
}

BOOL nand_eraseblock(U32 blocknum)
{
	U32 phyblocknum = nand_get_phy_block(blocknum);
	if(nand_phy_eraseblock(phyblocknum)==FALSE)
		return FALSE;
	nand_set_sparedata(blocknum);
	return TRUE;
}

BOOL nand_writepage(U32 pagenum,void* buf)
{
	pagenum = nand_get_phy_page(pagenum);
	return nand_phy_writepage(pagenum,buf);
}

BOOL nand_readpage(U32 pagenum,void* buf)
{
	pagenum = nand_get_phy_page(pagenum);
	return nand_phy_readpage(pagenum,buf);
}


#ifdef CONFIG_NAND_WRITE_BLOCK_CACHE
#warning Nand Write Block Buffer Mode build
typedef struct
{
	int blocknum;
	BOOL dirty; // if true, data is updated so it should be written at storage
	U8* buf;
}BLOCKBUF;
BLOCKBUF blockbuf ={-1,0,(U8*)0x40000000}; //use extern memory 128Kbyte
static void flushblockbuf()
{
	if(blockbuf.dirty==0)
		return;
	
	U32 pagesize = nand_get_pagesize();
	U32 pageperblock = nand_get_pageperblock();
	U32 startpage = blockbuf.blocknum*pageperblock;
	
	nand_eraseblock(blockbuf.blocknum);
	int i;
	for(i=0;i<pageperblock;i++)
	{
		nand_writepage(startpage+i,blockbuf.buf+(i*pagesize));
	}
	blockbuf.dirty=0;	
}

static void fillblockbuf(U32 blocknum)
{
	U32 pageperblock = nand_get_pageperblock();
	U32 startpage = blocknum*pageperblock;
	U32 pagesize = nand_get_pagesize();
	int i;
	for(i=0;i<pageperblock;i++)
	{
		nand_readpage(startpage+i,blockbuf.buf+(i*pagesize));
	}	
	blockbuf.blocknum=blocknum;
}
void nand_flushdata()
{
	flushblockbuf();
}
BOOL nand_updatesect(U32 sector,void* buffer)
{
	U8* buf = (U8*)buffer;
	U32 pagesize = nand_get_pagesize();
	U32 pagenum = sector*512/pagesize;//512 is sector size
	U32 nandblocknum = pagenum/nand_get_pageperblock();
	
	if(blockbuf.blocknum!=nandblocknum)
	{
		flushblockbuf();
		fillblockbuf(nandblocknum);
	}
	U8* pbuf = blockbuf.buf+(sector*512%nand_get_blocksize());
	memcpy(pbuf,buf,512);	
	blockbuf.dirty=1;
	return TRUE;
}
BOOL nand_updatesects(U32 startsector,void* buf,U32 cnt)
{
	int i;
	for(i=0;i<cnt;i++)
	{
		if(nand_updatesect(startsector+i,buf+(i*512))==FALSE)
			return FALSE;
	}
	return TRUE;
}
BOOL nand_readsect(U32 sector,void* buffer)
{
	U8* buf = (U8*)buffer;
	U32 pagesize = nand_get_pagesize();
	U32 pagenum = sector*512/pagesize;//512 is sector size
	U32 nandblocknum = pagenum/nand_get_pageperblock();
	
	if(blockbuf.blocknum == nandblocknum)
	{
		U32 offset = (sector*512)%nand_get_blocksize();
		memcpy(buf,blockbuf.buf+offset,512);
	}
	else
	{
		if(NandInfo.pagesize==512)
		{
			sector = nand_get_phy_page(sector);
		}
		else
		{
			pagenum = nand_get_phy_page(pagenum);
			sector = (pagenum*4)+(sector%4);
		}
		return nand_phy_readsect(sector,buf);

	}
	return TRUE;
}


#else
void nand_flushdata()
{
}

BOOL nand_updatesects(U32 sector,void* buffer,U32 sectorcnt)
{
	ALIGN4 U8 tempbuf[2048];
	U8* buf = (U8*)buffer;
	U32 pageperblock = nand_get_pageperblock();
	U32 pagesize = nand_get_pagesize();
	int blocknum;
	int pagenum;
	int sectorperpage = pagesize/512;
	U32 startblock = (sector/sectorperpage)/pageperblock;
	U32 endblock = (sector+sectorcnt)/sectorperpage/pageperblock;
	U32 startsector = sector;
	U32 endsector=sector+sectorcnt-1;
	U32 totalwritelen=0;

	for(blocknum=startblock;blocknum<endblock+1;blocknum++)
	{
		if(nand_phy_eraseblock(0)==FALSE)
		{
			debugstring("block 0 erase error\r\n");
			return FALSE;
		}
		//backup 
		U32 zeropagenum =0;
		if(pagesize==512)
		{
			for(pagenum=blocknum*pageperblock;pagenum<blocknum*pageperblock+pageperblock;pagenum++)
			{
				if( (pagenum >= startsector) 
					&&  (pagenum <= endsector) )
				{
					nand_phy_writepage(zeropagenum,buf+totalwritelen);
					totalwritelen += 512;
				}
				else
				{
					nand_readpage(pagenum,tempbuf);
					nand_phy_writepage(zeropagenum,tempbuf);
				}
				zeropagenum++;
			}
		}
		else//2048 page
		{
			U32 startpage = startsector/4;
			U32 endpage = endsector/4;
			for(pagenum=blocknum*pageperblock;pagenum<blocknum*pageperblock+pageperblock;pagenum++)
			{
				if( (pagenum >= startpage) 
					&&  (pagenum <= endpage) )
				{
					U32 sectornum=pagenum*4;
					U32 copysectorcnt;
					U32 offsetsectorinpage;
					if(sectornum <= startsector) // page |--|data|?|
					{
						offsetsectorinpage = startsector-sectornum;
						copysectorcnt = 4-offsetsectorinpage;
						if(startpage == endpage)
						{
							copysectorcnt = endsector+1 -startsector;
						}						
					}
					else//page |data|?|
					{
						offsetsectorinpage = 0;
						copysectorcnt = endsector+1-sectornum;
					}
					if(copysectorcnt>=4)
					{
						nand_phy_writepage(zeropagenum,buf+totalwritelen);
						totalwritelen += 2048;
					}
					else
					{
						nand_readpage(pagenum,tempbuf);
						memcpy(tempbuf+offsetsectorinpage*512,buf+totalwritelen,copysectorcnt*512);
						nand_phy_writepage(zeropagenum,tempbuf);
						totalwritelen += (copysectorcnt*512);
					}

					
				}
				else
				{
					nand_readpage(pagenum,tempbuf);
					nand_phy_writepage(zeropagenum,tempbuf);
				}
				zeropagenum++;
			}
		}
		
		//restore
		if(nand_eraseblock(blocknum)==FALSE)
		{
			return FALSE;
		}

		for(pagenum=0;pagenum<pageperblock;pagenum++)
		{
			nand_phy_readpage(pagenum,tempbuf);
			nand_writepage(blocknum*pageperblock+pagenum,tempbuf);
		}
	}
	return TRUE;
}
BOOL nand_readsect(U32 sector,void* buffer)
{
	
	if(NandInfo.pagesize==512)
	{
		sector = nand_get_phy_page(sector);
	}
	else
	{
		U32 pagesize = nand_get_pagesize();
		U32 sectorperpage = pagesize/512;
		U32 pagenum = sector/sectorperpage;//512 is sector size
		pagenum = nand_get_phy_page(pagenum);
		sector = (pagenum*sectorperpage)+(sector%sectorperpage);
	}
	return nand_phy_readsect(sector,buffer);
}
#endif
BOOL nand_readsects(U32 startsector,void* buf,U32 cnt)
{
	int i;
	for(i=0;i<cnt;i++)
	{
		if(nand_readsect(startsector+i,buf+(i*512))==FALSE)
			return FALSE;
	}
	return TRUE;
}
