/*

	SD CARD Driver
	
	kkh
*/


#include "cantus.h"


typedef struct 
{
	int CSD_STRUCTURE:2;
	int RESERVED:6;
	int TAAC:8;
	int TRAN_SPEED:8;
	int CCC:12;
	int READ_BL_LEN:4;
	int READ_BL_PARTIAL:1;
	int WRITE_BLK_MISALIGN:1;
	int READ_BLK_MISALIGN:1;
	int SDR_IMP:1;
	int RESERVED2:2;
	int C_SIZE:12;
	int VDD_R_CURR_MIN:3;
	int VDD_R_CURR_MAX:3;
	int VDD_W_CURR_MIN:3;
	int VDD_W_CURR_MAZ:3;
	int C_SIZE_MULT:3;
	int ERASE_BLK_EN:1;
	int SECTOR_SIZE:7;
	int WP_GRP_SIZE:7;
	int WP_GRP_ENABLE:1;
	int RESERVED3:5;
	int FILE_FORMAT_GRP:1;
	int COPY:1;
	int PERM_WRITE_PROTECT:1;
	int TMP_WRITE_PROTECT:1;
	int FILE_FORMAT:2;
	int RESERVED4:2;
	int CRC:7;
	int ALWAYS1:1;
	
}CSD_STRUCT;

enum SDC_ERROR
{
	SDC_ERR_NONE=0,
	SDC_ERR_READ_TIMEOUT,
	SDC_ERR_RESPONSE_TIMEOUT,
	SDC_ERR_WRITE_CRC,
	SDC_ERR_READ_CRC,
	SDC_ERR_RESPONSE_CRC,
};
#define SDC_NOERROR	0
#define SDC_RESPONSE_TIMEOUT 1
#define SDC_READ_TIMEOUT 2
	
static U8 g_bHCMemory = 0;
	
BOOL SDC_Command(U32 cmd,U32 arg);

//#define SDC_DEBUG_ENABLE
#ifdef SDC_DEBUG_ENABLE
	#define SDC_DEBUG debugprintf
#else
	#define SDC_DEBUG(...)
#endif


static U16 s_sdc_resposebuf[8];//max 136bit
static U32 s_sdc_lasterrorno = SDC_ERR_NONE;
static U32 s_sdc_laststat = 0;
static U32 s_BlockSize = 512;
static U32 s_MemSize = 0;


//Sets the address of the first/last write block to be erase
BOOL SDC_Erase(U32 addr,U32 endaddr)
{
	SDC_Command(0x400|32,addr); 
	SDC_Command(0x400|33,endaddr); 
	SDC_Command(0x500|38,0); 
	return TRUE;
}


//
BOOL SDC_WriteBlock(void* buf,U32 blocknum,U32 nob)
{
	register U32 i;
	register U32 stat;
	
	*R_SDCNWR = 0x10;
	*R_SDCNOB = nob;
	U32 addr;
	if(g_bHCMemory)
		addr = blocknum;
	else
		addr = blocknum*512;
	
	if(nob>1)
		SDC_Command(0x400|(1<<7)|(1<<6)|(25),addr); //multiblock
	else
		SDC_Command(0x400|(1<<7)|(1<<6)|(24),addr); //single block
	U32* databuf = (U32*)buf;
	BOOL bok = TRUE;
	for(i=0;i<nob*16;i++)
	{
		//fill half for speed
		
		*R_SDCDFA = *(U32*)databuf;	
		*R_SDCDFA = *(U32*)(databuf+1);
		*R_SDCDFA = *(U32*)(databuf+2);
		*R_SDCDFA = *(U32*)(databuf+3);
		*R_SDCDFA = *(U32*)(databuf+4);
		*R_SDCDFA = *(U32*)(databuf+5);
		*R_SDCDFA = *(U32*)(databuf+6);
		*R_SDCDFA = *(U32*)(databuf+7);
		databuf+=8;
		
		//wait for half empty
		while(1)
		{
			stat = *R_SDCSTAT;
			if((stat & (1<<12))==0) 
				break;
		}
		//check error
		if(stat & (1<<3))
		{
			bok = FALSE;
			break;
		}
	}
	if(bok)
	{
		while(1)
		{
			stat = *R_SDCSTAT;
			if(stat & (1<<9))
				break;
		}
	}
	if(nob>1)
		SDC_Command(0x50c,0);//stop commmand
	*R_SDCSTAT = 0x200;//write operation done
	return bok;
}


BOOL SDC_ReadBlock(void* buf,U32 blocknum,U32 nob)
{
	register U32 i,j;
	register U32 stat;
	*R_SDCNWR = 0x10;
	*R_SDCNOB = nob;
	U32 addr ;
	
	if(g_bHCMemory)
		addr = blocknum;
	else
		addr = blocknum*512;

	if(nob>1)
		SDC_Command(0x400|(1<<7)|(18),addr);
	else
		SDC_Command(0x400|(1<<7)|(17),addr);
	U32* databuf = (U32*)buf;
	BOOL bok = TRUE;

	for(i=0;i<nob*8;i++)
	{
		//wait for half full
		while(1)
		{
			stat = *R_SDCSTAT;
//			PRINTVAR(stat);
			if(stat & (1<<13)) //full
			{
				break;
			}
			if(stat & (1<<1)) //read time out
			{
				s_sdc_lasterrorno = SDC_ERR_READ_TIMEOUT;
				debugprintf("READ TIME OUT ERROR(0x%x block)\r\n",blocknum);
				bok = FALSE;
				break;
			}
		}
		if(bok == FALSE)
		{
			break;
		}
		
		if(stat & (1<<SDC_ERR_READ_CRC))
		{
			s_sdc_lasterrorno = SDC_ERR_READ_CRC;
			debugstring("READ CRC ERROR\r\n");
			bok = FALSE;
			break;
		}
		for(j=0;j<2;j++)
		{
			*(U32*)databuf = *R_SDCDFA ;
			*(U32*)(databuf+1) = *R_SDCDFA ;
			*(U32*)(databuf+2) = *R_SDCDFA ;
			*(U32*)(databuf+3) = *R_SDCDFA ;
			*(U32*)(databuf+4) = *R_SDCDFA ;
			*(U32*)(databuf+5) = *R_SDCDFA ;
			*(U32*)(databuf+6) = *R_SDCDFA ;
			*(U32*)(databuf+7) = *R_SDCDFA ;
			databuf+=8;
		}
			
		if(stat & (1<<8))//read operatoin completed
		{
			break;
		}
	}
	if(bok)
	{
		while(1)
		{
			stat = *R_SDCSTAT;
			if(stat & (1<<8))
				break;
			if(stat & (1<<1))
			{
				bok = FALSE;
				break;
			}
		}
	}
	if(nob >1)
	{
		SDC_Command(0x500|12,0);//stop commmand
	}
	*R_SDCSTAT = 0x100;//write operation done
	return bok;
}


BOOL SDC_Command(U32 cmd,U32 arg)
{
	U8 i; 

	*R_SDCCMDA = arg;
	*R_SDCCMDCON = cmd;
	U8 cmdnum = cmd & 0x3f;
	//debugprintf("CMD%d ,ARG:%#x \r\n", cmdnum,arg);
	while(1)
	{
		s_sdc_laststat = *R_SDCSTAT;
		if(s_sdc_laststat & 0x400)
		{
		//	PRINTVAR(s_sdc_laststat);
			break;
		}
		if(s_sdc_laststat & (1<<SDC_ERR_RESPONSE_TIMEOUT))
		{
			*R_SDCSTAT = 0x7ff; // do not clear read done / write done
			//PRINTVAR(s_sdc_laststat);
			s_sdc_lasterrorno = SDC_ERR_RESPONSE_TIMEOUT;
			SDC_DEBUG("SDC_ERR_RESPONSE_TIMEOUT\r\n");
			return FALSE;
		}
	}
	
    *R_SDCSTAT = 0x7ff; // do not clear read done / write done
	
	if(cmd & (1<<10))
	{
		
		if(s_sdc_laststat & (1<<SDC_ERR_RESPONSE_TIMEOUT))
		{
			//PRINTVAR(s_sdc_laststat);
			s_sdc_lasterrorno = SDC_ERR_RESPONSE_TIMEOUT;
			SDC_DEBUG("SDC_ERR_RESPONSE_TIMEOUT\r\n");
			return FALSE;
		}
		if(!(cmdnum == 41|| cmdnum == 1))
		{
			if( s_sdc_laststat & (1<<5) )
			{
				s_sdc_lasterrorno = SDC_ERR_RESPONSE_CRC;
				//PRINTVAR(s_sdc_lasterrorno );
				return FALSE;
			}
		}

		if(cmd & (1<<9))
		{
			SDC_DEBUG("Long Respose\r\n");
			for(i=0;i<8;i++)
				s_sdc_resposebuf[i] = *(volatile U16*)R_SDCRFA;
		}
		else 
		{
			SDC_DEBUG("Short Respose\r\n");
			for(i=0;i<3;i++)
			{
				s_sdc_resposebuf[i] = *(volatile U16*)R_SDCRFA;
				SDC_DEBUG(s_sdc_resposebuf[i]);
			}
		}
	}
    return TRUE;
}

BOOL SDC_Init()
{
	U32 i;
	g_bHCMemory = 0;
	*R_PAF4 = ((*R_PAF4) & 0xf00f)|PAF4_SDCD_CLK|PAF4_SDCD_CMD|PAF4_SDCD_D2|PAF4_SDCD_D3;
	*R_PAF5 = ((*R_PAF5) & 0xfff0)|PAF5_SDCD_D0|PAF5_SDCD_D1;
	
	*R_SDCCON = 0;//disable
	*R_SDCCON =(1<<5)|1;//1bit mode,enable
	delayms(10);
	
	// initialize
	*R_SDCCD = 0x200;
	*R_SDCRTO = 0xff ;
	*R_SDCDTO = 0xf000 ;
	*R_SDCBL = 0x200 ;
	/*
	i= 512;//512 times
	do
	{
		if(!SDC_Command(0,0)) //reset
			continue;
			
		if(SDC_Command(0x400 + 55,0))
			break;//cmd55
	}while(--i);
	
	if(i==0)
	{
		DEBUGPRINTF("TIME OUT 1 \r\n");
		return FALSE;
	}
	
		
	i= 512;//512 times
	do
	{
		if(!SDC_Command(0x400+41,0xff8000))
		{
			continue;
		}
		else
		{
			if(s_sdc_resposebuf[0] == 0x3f80)
				break;
//			else
				//PRINTVAR(s_sdc_resposebuf[0]);
		}
		SDC_Command(0x400 + 55,0);
		
	}while(--i);
	
	
	if(i==0)
	{
		DEBUGPRINTF("TIME OUT 1 \r\n");
		return FALSE;
	}
*/

	i= 512;//512 times
	do
	{
		if(!SDC_Command(0,0)) //reset
			continue;
			
		if(SDC_Command(0x400 + 8,0x1aa))
			break;//cmd55
	}while(--i);
		
	i= 1024;//512 times
	do
	{
		SDC_Command(0x400 + 55,0);//ACMDs shall be preceded with APP_CMD command (CMD55).
		if(!SDC_Command(0x500+41,0x40ff8000))
		{
			continue;
		}
		else
		{
			if(s_sdc_resposebuf[0] & 0x80)
			{
				if(s_sdc_resposebuf[0] & (1<<6))
				{
					debugstring("HCSD Memory Card\r\n");
					g_bHCMemory = 1;
				}
				else
				{
					debugstring("SD Memory Card\r\n"); 
				}
				break;
			}
		}
		
	}while(--i);
	
	if(i==0)
	{
		debugstring("TIME OUT 1 \r\n");
		return FALSE;
	}


	debugstring("Identification Mode Start\r\n");
	
	i= 512;//512 times
	do
	{
		if(SDC_Command(0x600 |2, 0) == TRUE)
		{
			break;
		}
	}while(i--);
	
	if(i==0)
	{
		debugstring("COMMAND SEND ERROR \r\n");
		return FALSE;
	}
	
	
	if(SDC_Command(0x400|3,0) == FALSE) //Get RCA 
	{

		return FALSE;
	}
	U32 rca;
	rca = ((s_sdc_resposebuf[0] & 0x00ff)<<8) | (s_sdc_resposebuf[1] >> 8);
	SDC_Command(0x600|9,rca <<16);

#ifdef SDC_DEBUG_ENABLE	
	debugstring("==== CSD ===== \r\n");
	for(i=0;i<8;i++)
	{
		debugprintf("%04X [%d:%d]\r\n",s_sdc_resposebuf[i],16*(8-i)-1,16*(8-i)-16);
		PRINTVAR(s_sdc_resposebuf[i]);
	}
#endif	
	U8 sd_ver = 0;//0:1.0, 1:2.0
	U8 TAAC,NSAC,TRAN_SPEED;
	sd_ver = s_sdc_resposebuf[0]>>14;
	TAAC = s_sdc_resposebuf[0] & 0xff;
	NSAC = s_sdc_resposebuf[1] >> 8;
	TRAN_SPEED = s_sdc_resposebuf[1] & 0xff;
	U32 maxfreq;
	switch(TRAN_SPEED & 0x7)
	{
		case 0:
			maxfreq = 1*10000;
			break;
		case 1:
			maxfreq = 1*100000;
			break;
		case 2:
			maxfreq = 10*100000;
			break;
		case 3:
			maxfreq = 100*100000;
			break;
		default:
			debugstring("unsupported operating frequency\r\n");
			maxfreq = 1*10000;
			break;
	}
	U32 freqtype[16]={0,10,12,13,15,20,25,30,35,40,45,50,55,60,70,80};
	maxfreq *= freqtype[TRAN_SPEED >> 3];	
	
	debugprintf("the mandatory maximum operating frequency of SD Memory Card : %dMhz \r\n",maxfreq/1000000);
	
	int READ_BL_LEN = s_sdc_resposebuf[2]&0xf;
	int C_SIZE;
	if(sd_ver == 0)
		C_SIZE = ( (s_sdc_resposebuf[3] & 0x03ff) << 2 ) + ((s_sdc_resposebuf[4] & 0xc000)>>14);
	else
		C_SIZE = ( (s_sdc_resposebuf[3] & 0x3f) << 16 ) + s_sdc_resposebuf[4];
	s_BlockSize = 1<<READ_BL_LEN;;
	if(sd_ver == 0)
	{
		int C_SIZE_MULT = ( (s_sdc_resposebuf[4] & 0x3) << 1 ) + ((s_sdc_resposebuf[5] & 0x8000)>>15);
		//U32 BLOCKNR = (C_SIZE+1)*(pow(2,C_SIZE_MULT+2));
		U32 BLOCKNR = (C_SIZE+1)*(1<<(C_SIZE_MULT+2));
		s_MemSize = (BLOCKNR * s_BlockSize)/(1024*1024);
	}
	else
	{
		s_MemSize = (C_SIZE+1)/2;
		debugprintf("SD Memory Size : %ldMbyte \r\n",s_MemSize);
	}
	debugprintf("SD Memory Size : %dMbyte ,Blocksize:%dbyte\r\n",s_MemSize,s_BlockSize);

	SDC_Command(0x600|10,rca <<16);

#ifdef SDC_DEBUG_ENABLE	
	debugstring("==== CID ===== \r\n");
	for(i=0;i<8;i++)
	{
		debugprintf("%04X [%d:%d]\r\n",s_sdc_resposebuf[i],16*(8-i)-1,16*(8-i)-16);
		PRINTVAR(s_sdc_resposebuf[i]);
	}
#endif

	SDC_Command(0x500|7,rca <<16);
	debugstring("SD CARD Initialize Completed\r\n");

//after initialize
	if(READ_BL_LEN != 9)//if not 512
	{
		SDC_Command((1<<10)|16,512);//set block size
		debugstring("set block size as 512byte\r\n");
	}
	SDC_Command((1<<10)|55,rca <<16);//APPcmd
	SDC_Command((1<<10)|6,2);//4bit mode
 	*R_SDCCON = (1<<5)|3; //4bit mode
	
	U32 sysclock = GetAHBclock();
	//some board or some card needs it
	if(maxfreq>24000000)
		maxfreq=23000000;
	for(i=0;i<100;i++)
	{
		if((sysclock/(2+i)) <= maxfreq)
			break;
	}
	debugprintf("SDCard operating frequency:%d \r\n",sysclock/(2+i));
	*R_SDCCD = i; 
	
	return TRUE;
}

U32 SDC_GetSectorSize()
{
	return 512;
}
U32 SDC_GetSectorCount()
{
	return ((unsigned long long)s_MemSize*1024*1024/512);
}

//return Megabyte Size
U32 SDC_MemSize()
{
	return s_MemSize;
}


#define SDC_MEMCPY(dest,src,bytesize)	memcpy((void*)dest,(void*)src,bytesize)
static BOOL SDC_Readblock_indirect(void* buf,U32 sectoraddr,U32 sectcnt)
{
	U32 tempbuf[512/4];
	U32 bufaddr=(U32)buf;
	int i;
	for(i=0;i<sectcnt;i++)
	{
		if(SDC_ReadBlock(tempbuf,(sectoraddr+i),1)==FALSE)
			return FALSE;
		SDC_MEMCPY(bufaddr,tempbuf,512);
		bufaddr += 512;
	}
	
	return TRUE;
}
static BOOL SDC_Read_all_align(void* buf, U32 addr, U32 len)
{
	unsigned int tempbuf[512/4];
	unsigned int bufaddr = (unsigned int)buf;
	int ReadSectorCnt;
	U32 remainsize;
	int	ReadStartSector = addr / 512;
	ReadSectorCnt = len/512;
	remainsize = len % 512;
	if(ReadSectorCnt)
	{
		if(SDC_ReadBlock(buf,ReadStartSector,ReadSectorCnt) ==FALSE)
		{
			return FALSE;
		}
	}
	
	if(remainsize)
	{
		if(SDC_ReadBlock(tempbuf,ReadStartSector+ReadSectorCnt,1)==FALSE)
			return FALSE;
		bufaddr += (ReadSectorCnt*512);
		memcpy((void*)bufaddr,(void*)tempbuf,remainsize );
	}
	return TRUE;
}


//Only buffer misalign
static BOOL SDC_Read_buf_misalign(void* buf, unsigned int addr, unsigned int len)
{
	
	unsigned int tempbuf[512/4];
	unsigned int bufaddr = (unsigned int)buf;
	int ReadSectorCnt;
	U32 remainsize;
	int	ReadStartSector = addr / 512;
	ReadSectorCnt = len/512;
	remainsize = len % 512;
	if(len<512)
	{
		if(SDC_ReadBlock(tempbuf,ReadStartSector,1)==FALSE)
			return FALSE;
		memcpy(buf,tempbuf,len );
	}
	else
	{
		if(SDC_Readblock_indirect(buf,ReadStartSector,ReadSectorCnt)==FALSE)
			return FALSE;
		
		if(remainsize)
		{
			
			if(SDC_ReadBlock(tempbuf,ReadStartSector+ReadSectorCnt,1)==FALSE)
				return FALSE;
			bufaddr += (ReadSectorCnt*512);
			memcpy((void*)bufaddr,(void*)tempbuf,remainsize );
		}
	}
	return TRUE;
}
//address misalign
static BOOL SDC_Read_addr_misalign(void* buf, unsigned int addr, unsigned int len)
{
	unsigned int tempbuf[512/4];
	unsigned int bufaddr = (unsigned int)buf;
	U32 offset;
	int	ReadStartSector;
	U32 copysize;
	ReadStartSector	= addr / 512;
	offset = addr%512;
	copysize=512-offset;
	if(copysize>len)
		copysize=len;
	if(SDC_ReadBlock(tempbuf,ReadStartSector,1)==FALSE)
			return FALSE;
	memcpy((void*)bufaddr,(void*)(((U32)tempbuf)+offset),copysize );
	bufaddr += copysize;
	if(len>copysize)
	{
		if(bufaddr&0x3)
		{
			if(SDC_Read_buf_misalign((void*)bufaddr,(addr+copysize),len-copysize)==FALSE)
				return FALSE;
		}
		else
		{
			if(SDC_Read_all_align((void*)bufaddr,addr+copysize,len-copysize)==FALSE)
				return FALSE;
		}
	}
	return TRUE;
	
}

//1:address, 2:buffer
BOOL SDC_Read(void* buf, unsigned int addr, unsigned int len)
{
	
	int ReadStartSector;
	int ReadSectorCnt;
	ReadStartSector = addr / 512;
	ReadSectorCnt = len/512;
	if((unsigned int)buf & 0x3)
	{
		if(addr%512)
		{
			if(SDC_Read_addr_misalign(buf,addr,len)==FALSE)
				return FALSE;
		}
		else
		{
			if(SDC_Read_buf_misalign(buf,addr,len) == FALSE)
				return FALSE;
		}
	}
	else
	{
		if(addr % 512)
		{
			if(SDC_Read_addr_misalign(buf,addr,len)==FALSE)
				return FALSE;
		}
		else
		{
			if(SDC_Read_all_align(buf,addr,len)==FALSE)
				return FALSE;
		}
	}
	return TRUE;
}


static BOOL SDC_Writeblock_indirect(void* buf, U32 sectoraddr, U32 sectcnt)
{
	U32 tempbuf[512/4];
	U32 bufaddr=(U32)buf;
	int i;
	for(i=0;i<sectcnt;i++)
	{
		SDC_MEMCPY(bufaddr,tempbuf,512);
		if(SDC_WriteBlock(tempbuf,(sectoraddr+i),1)==FALSE)
			return FALSE;
		bufaddr += 512;
	}
	return TRUE;
}
static BOOL SDC_Write_all_align(void* buf, U32 addr, U32 len)
{
	unsigned int tempbuf[512/4];
	unsigned int bufaddr = (unsigned int)buf;
	int WriteSectorCnt;
	U32 remainsize;
	int	WriteStartSector = addr / 512;
	WriteSectorCnt = len/512;
	remainsize = len % 512;
	if(WriteSectorCnt)
	{
		if(SDC_WriteBlock(buf,WriteStartSector,WriteSectorCnt) ==FALSE)
		{
			return FALSE;
		}
	}
	if(remainsize)
	{
		if(SDC_ReadBlock(tempbuf,WriteStartSector+WriteSectorCnt,1)==FALSE)
			return FALSE;
		bufaddr += (WriteSectorCnt*512);
		memcpy((void*)tempbuf,(void*)bufaddr,remainsize );
		if(SDC_WriteBlock(tempbuf,WriteStartSector+WriteSectorCnt,1)==FALSE)
			return FALSE;
	}
	return TRUE;
}
//only buf misalign
static BOOL SDC_Write_buf_misalign(void* buf, U32 addr, U32 len)
{
	unsigned int tempbuf[512/4];
	unsigned int bufaddr = (unsigned int)buf;
	int WriteSectorCnt;
	U32 remainsize;
	int	WriteStartSector = addr / 512;
	WriteSectorCnt = len/512;
	remainsize = len % 512;
	if(len<512)
	{
		if(SDC_ReadBlock(tempbuf,WriteStartSector,1)==FALSE)
			return FALSE;
		memcpy(tempbuf,buf,len );
		if(SDC_WriteBlock(tempbuf,WriteStartSector,1)==FALSE)
			return FALSE;
	}
	else
	{
		if(SDC_Writeblock_indirect(buf,WriteStartSector,WriteSectorCnt)==FALSE)
			return FALSE;
		
		if(remainsize)
		{
			
			if(SDC_ReadBlock(tempbuf,WriteStartSector+WriteSectorCnt,1)==FALSE)
				return FALSE;
			bufaddr += (WriteSectorCnt*512);
			memcpy((void*)tempbuf,(void*)bufaddr,remainsize );
			if(SDC_WriteBlock(tempbuf,WriteStartSector+WriteSectorCnt,1)==FALSE)
				return FALSE;
		}
	}
	return TRUE;
}
static BOOL SDC_Write_addr_misalign(void* buf, U32 addr, U32 len)
{
	unsigned int tempbuf[512/4];
	unsigned int bufaddr = (unsigned int)buf;
	U32 offset;
	int	ReadStartSector;
	U32 copysize;
	ReadStartSector	= addr / 512;
	offset = addr%512;
	copysize=512-offset;
	if(copysize>len)
		copysize=len;
	if(SDC_ReadBlock(tempbuf,ReadStartSector,1)==FALSE)
			return FALSE;
	memcpy((void*)(((U32)tempbuf)+offset),(void*)bufaddr,copysize );
	bufaddr += copysize;
	if(SDC_WriteBlock(tempbuf,ReadStartSector,1)==FALSE)
		return FALSE;
	if(len>copysize)
	{
		if(bufaddr&0x3)
		{
			if(SDC_Write_buf_misalign((void*)bufaddr,(addr+copysize),len-copysize)==FALSE)
				return FALSE;
		}
		else
		{
			if(SDC_Write_all_align((void*)bufaddr,addr+copysize,len-copysize)==FALSE)
				return FALSE;
		}
	}
	return TRUE;
}


BOOL SDC_Write(void* buf, unsigned int addr, unsigned int len)
{
	if((unsigned int)buf & 0x3)
	{
		if(addr%512)
		{
			if(SDC_Write_addr_misalign(buf,addr,len)==FALSE)
				return FALSE;
		}
		else
		{
			if(SDC_Write_buf_misalign(buf,addr,len)==FALSE)
				return FALSE;
		}
	}
	else
	{
		if(addr % 512)
		{
			if(SDC_Write_addr_misalign(buf,addr,len)==FALSE)
				return FALSE;
		}
		else
		{
			if(SDC_Write_all_align(buf,addr,len)==FALSE)
				return FALSE;
		}
	}
	return TRUE;
}


