
#include "cantus.h"
#include "driver/lcd_g2.h"
#include "driver/lcdutil_I2810.h"
#include "driver/font_ctrl.h"

#define UART_CH6	6

void CANfly_init()
{
	*R_PAF0 = 0xFFFF;  // GPIO 
	*R_PAF1 = 0x0FFF;  // UART7, GPIO
	*R_PAF2 = 0xFFFF;  // GPIO
	*R_PAF3 = 0xFFFF;  // GPIO
	*R_PAF4 = 0x0FFF;  // UART3, GPIO
	*R_PAF5 = 0xFFFF;  // GPIO
	*R_PAF6 = 0x030C;  // PIO6.1 -> USB-Communication mode, PIO6.4 -> USB Ctrl
	*R_P6oDIR |= (1<<4); // PIO6.4 output mode
}



#define USB_BUF_SIZE 512

typedef struct _USB_RING_BUF {
	U16 uRingBufRxCtr;     // Number of characters in the Rx ring buffer
	U8  *uRingBufRxInPtr;  // Pointer to where next character will be inserted
	U8  *uRingBufRxOutPtr; // Pointer from where next character will be extracted
	U8  uRingBufRx[USB_BUF_SIZE]; // Ring buffer character storage (Rx)
} USB_RING_BUF;

USB_RING_BUF usbbuf;

void InitRingBuf_u(USB_RING_BUF* pRing)
{
	pRing->uRingBufRxCtr    = 0;
	pRing->uRingBufRxInPtr  = &pRing->uRingBufRx[0];
	pRing->uRingBufRxOutPtr = &pRing->uRingBufRx[0];
}


#define R_USBLBOWC (volatile U32*)(0x80022c00 + 0x40)
#define R_USBHBOWC (volatile U32*)(0x80022c00 + 0x44)

U32 total_in_data_len;

void usb_isr(void)
{
	BYTE usb_data[64];
	volatile U32 len;
	volatile U32 k=0;
	USB_RING_BUF* pbuf = &usbbuf;
	BYTE ep_irq;
    BYTE usb_irq;
	
	ep_irq  = *(volatile U8*)(__USBEI_REG);	
	usb_irq = *(volatile U8*)(__USBUI_REG);
		
		
	if(ep_irq & 1)//EP0_IRQ
	{
		ep0_isr();
	}
	if (ep_irq & (1<<1))//EP1_IRQ
	{
		k=0;
		len = read_usb(usb_data, len);
		while(len) {
			if(pbuf->uRingBufRxCtr<USB_BUF_SIZE) {
				pbuf->uRingBufRxCtr++;
				*pbuf->uRingBufRxInPtr=usb_data[k++];
				pbuf->uRingBufRxInPtr++;
				if(pbuf->uRingBufRxInPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
					pbuf->uRingBufRxInPtr = &pbuf->uRingBufRx[0];
				}
				
			}
			len--;
		}
	}
	
	if(usb_irq & USB_RESET_IRQ)
	{
		usb_reset();
	}
	else if ( usb_irq & USB_RESUME_IRQ )
	{
		usb_resume();
	}
	else if ( usb_irq & USB_SUSPEND_IRQ )
	{
		usb_suspend();
	}
	*(volatile U8*)__USBEI_REG = ep_irq;   // Clear Interrupt 
	*(volatile U8*)__USBUI_REG = usb_irq;  // Clear Interrupt

}

int UsbGetCh(U8* buf)
{
	int i=0;

	USB_RING_BUF* pbuf = &usbbuf;


	if (pbuf->uRingBufRxCtr > 0) { /* No, decrement character count */
		pbuf->uRingBufRxCtr--; /* No, decrement character count */
		buf[0] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
		/* Wrap OUT pointer     */
		if (pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
			pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
		}
		i=1;
	}

	return i;
}

U8 UsbGetC()
{
	U8 ch;
	
	while(1) {
		if(UsbGetCh(&ch)) break;
	}
	return ch;
}

int UsbGetData(U8* buf, int bufmax)
{
	int i=0;

	USB_RING_BUF* pbuf = &usbbuf;

	// CRITICAL_ENTER();
	for(i=0;i<bufmax;i++) {
		if (pbuf->uRingBufRxCtr > 0) { /* No, decrement character count */
			pbuf->uRingBufRxCtr--; /* No, decrement character count */
			buf[i] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
			/* Wrap OUT pointer     */
			if (pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
				pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
			}
		}
		else {
			break;
		}	
	}
	// CRITICAL_EXIT();

	return i;
}

extern U8 Font_X_siz;
extern U8 Font_Y_siz;
U32 xp=0,yp=0;
U8 before_ch=0;
void FontDisplayLCD(char *fData, int siz)
{
	U8 ch;
	U16  txt;
	int pp;
	int i;
	U8 *str = fData;

	if(before_ch) {
		txt = (before_ch<<8) + (*str & 0xFF);
		str++;

		if(xp >= 320) {
			xp = 0;
			yp += Font_Y_siz;
			LCDdrawbox(0,yp,320,yp+Font_Y_siz+Font_Y_siz,255,255,0);
			// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
		}
		if(yp >= 240) {
			yp = 0;
			// LCDdrawbox(0,yp,320,yp+32,255,255,0);
			// LCDcls(255,255,0);
		}
		pp = DisplayFont(xp, yp, txt);
		xp += pp;
	}

	for(i=0; i<siz; i++) {
		ch = *str;
		str++;
		if(ch < 0x80) {
			txt = ch;
			if(txt == 0x0d) {
				xp = 0;
				yp += Font_Y_siz;
//				LCDdrawbox(0,yp,320,yp+Font_Y_siz+Font_Y_siz,255,255,0);
				// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
			
				if(yp >= 240) {
					yp = 0;
					// LCDdrawbox(0,yp,320,yp+32,255,255,0);
					// LCDcls(255,255,0);
				}
				continue;
			}
			else if(txt == 0x0a) {
				continue;
			}
		}
		else {
			if(i == (siz-1)) {
				before_ch = ch;
				return;
			}
			else before_ch=0;
			txt = (ch<<8) + (*str & 0xFF);
			str++;
			i++;
		}
		
		if(xp >= 320) {
			xp = 0;
			yp += Font_Y_siz;
//			LCDdrawbox(0,yp,320,yp+Font_Y_siz+Font_Y_siz,255,255,0);
			// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
		}
		if(yp >= 240) {
			yp = 0;
			// LCDdrawbox(0,yp,320,yp+32,255,255,0);
			// LCDcls(255,255,0);
		}
		pp = DisplayFont(xp, yp, txt);
		xp += pp;		
	}
}

void PrintLCD(U8 *data)
{
	int siz = strlen(data);
	int len=0;

	while(siz) {
		if(siz>64) len = 64;
		else len = siz;
		FontDisplayLCD(data, len);
		data += len;
		siz -= len;
	}
	
}

void UsbUartPutCh(U8 data)
{
	write_usb(&data, 1);	
}

void UsbUartPuts(U8 *data)
{
	int siz = strlen(data);
	int len=0;

	while(siz>0) {
		if(siz>64) len = 64;
		else len = siz;
		write_usb(data, len);
		data += len;
		siz -= len;
	}
	
}

void UartRxBufFlush(int ch)
{
	int ret;
	U8 tmp;
	do {
		ret = UartGetCh(ch, &tmp);
	} while(ret);
}

U8 UartGetC(int channel)
{
	U8 tmp;

	while(!UartGetCh(channel, &tmp));

	return tmp;
	
}

U8 UartGetCC(int channel, int *err_no)
{
	int count=0xffffff;
	U8 tmp;
	err_no[0] = 0;

	while(!UartGetCh(channel, &tmp)) {
		count--;
		if(count<=0) {
			err_no[0] = 1;
			return 0;
		}
	}

	return tmp;
	
}

int cmd_in = 0;

int EnterCmdMode(void)
{
	int i=0, j;
	U8 rdat;
	U8 temp[65];
	int cnt=0xfffff;

	UsbUartPuts("\r\n\t => Enter WiFly Command Mode!\r\n");
	for(j=0; j<5; j++) {
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "$$$");
		delayms(500);
		while(cnt>0) {
			if(UartGetCh(UART_CH6, &rdat)) {
				UartPutCh(7, rdat);
				if(rdat == 'C') {
					temp[i]=rdat;
					i++;
					goto Nnext;
				}
				else if(rdat == '$') {
					UsbUartPuts("Already came into CMD mode!\r\n");
					cmd_in = 1;
					return 1;
				}
			}
			cnt--;		
		}
		cnt = 0xfffff;
		UsbUartPuts("\r\n\t => reEnter WiFly Command Mode!\r\n");
	}
	if(j==5) {
		UsbUartPuts("\t => Fail!\r\n");
		cmd_in = 0;
		return 0;
	}

Nnext:
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0d) break;
			temp[i]=rdat;
			i++;
		}
	}

	if(strncmp(temp, "CMD", 3) == 0) {
		UsbUartPuts("\t => OK!\r\n");
		cmd_in = 1;
	}
	else {
		UsbUartPuts("\t => Fail!\r\n");
		cmd_in = 0;
	}

	UartRxBufFlush(UART_CH6);
	
	return cmd_in;
}

int GetAP_ConnectStatus(U8 *ssid)
{
	int i=0;
	char rdat;
	int ret;
	U8 tmp[256];
	
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) {
			UsbUartPuts("Fail!, Enter CMD Mode!\r\n");
			return 0;
		}
	}
	delayms(3000);
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show connection\r");

	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0a) break;
		}
	}
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0d) {
				tmp[i] = 0;
				break;
			}
			else tmp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	if(tmp[2]=='3' && tmp[3]!='3') ret=1;
	else ret=0;

	i=0;
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show net\r");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0a) break;
		}
	}
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0d) {
				tmp[i] = 0;
				break;
			}
			else tmp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	strcpy(ssid, &tmp[5]);	
	
	return ret;
}

#define CFG_CBSIZE 75
U8 console_buffer[CFG_CBSIZE];

static char * delete_char(char *buffer, char *p, int *colp, int *np, int plen)
{
	char *s;

	if(*np == 0) {
		return (p);
	}

	if(*(--p) == '\t') {			/* will retype the whole line	*/
		while(*colp > plen) {
			UsbUartPuts("\b \b");
			(*colp)--;
		}
		for(s=buffer; s<p; ++s) {
			if (*s == '\t') {
				UsbUartPuts("        "+((*colp) & 07));
				*colp += 8 - ((*colp) & 07);
			} else {
				++(*colp);
				UsbUartPutCh(*s);
			}
		}
	} else {
		UsbUartPuts("\b \b");
		(*colp)--;
	}
	(*np)--;
	return (p);
}


int readline(U8 *prompt)
{
	char *p = console_buffer;
	int	n = 0;				/* buffer index		*/
	int	plen = 0;			/* prompt length	*/
	int	col;				/* output column cnt	*/
	char c;

	/* print prompt */
	if(prompt) {
		plen = strlen(prompt);
		UsbUartPuts(prompt);
	}
	col = plen;

	for (;;) {
		c = UsbGetC();

		/*
		 * Special character handling
		 */
		switch (c) {
			case '\r':				/* Enter		*/
			case '\n':
				*p = '\0';
				UsbUartPuts("\r\n");
				return ((int)p - (int)console_buffer);

			case '\0':				/* nul			*/
				continue;

			case 0x03:				/* ^C - break		*/
				console_buffer[0] = '\0';	/* discard input */
				return (-1);

			case 0x15:				/* ^U - erase line	*/
				while (col > plen) {
					UsbUartPuts("\b \b");
					--col;
				}
				p = console_buffer;
				n = 0;
				continue;

			case 0x17:				/* ^W - erase word 	*/
				p=delete_char(console_buffer, p, &col, &n, plen);
				while ((n > 0) && (*p != ' ')) {
					p=delete_char(console_buffer, p, &col, &n, plen);
				}
				continue;

			case 0x08:				/* ^H  - backspace	*/
			case 0x7F:				/* DEL - backspace	*/
				p=delete_char(console_buffer, p, &col, &n, plen);
				continue;

			default:
				/*
				 * Must be a normal character then
				 */
				if (n < CFG_CBSIZE-2) {
					if(c == '\t') {	/* expand TABs		*/
						UsbUartPuts("        "+(col&07));
						col += 8 - (col&07);
					} else {
						++col;		/* echo input		*/
						UsbUartPutCh(c);
					}
					*p++ = c;
					++n;
				} else {			/* Buffer full		*/
					UsbUartPutCh('\a');
				}
		}
	}
}
/****************************************************************************/

 typedef struct tag_BITMAPFILEHEADER {
	U32 bfSize;	
	U32 bfReserved; 	
	U32 bfOffBits; 	
	U32 biSize; 	
	S32 biWidth; 	
	S32 biHeight; 	
	U16 biPlanes; 	
	U16 biBitCount;	
	U32 biCompression; 	
	U32 biSizeImage; 	
	S32 biXPelsPerMeter; 	
	S32 biYPelsPerMeter; 	
	U32 biClrUsed; 	
	U32 biClrImportant; 
 }BITMAPFILEHEADER;   

typedef struct tag_RGBQUAD {	
	U8 rgbBlue;	
	U8 rgbGreen;     
	U8 rgbRed;	
	U8 rgbReserved;
} RGBQUAD; 

int FtpFileDateRead(U8 *file_buf, int read_size, U32 *byte_read)
{
	int cnt=0xffffff;
	int i=0;
	
	while(i<read_size) {
		while(1) {
			if(UartGetCh(UART_CH6, &file_buf[i])) {
				cnt = 0xffffff;
				break;
			}
			else {
				cnt--;
				if(cnt<=0) return 0;
			}
		}
		i++;
	}
	byte_read[0] = i;
	return 1;

}
#if 0 //[[ molink_yyb_110704_BEGIN -- .
int FtpFileDateRead(U8 *file_buf, int read_size, U8 *end_chk)
{
	int i=0;
	end_chk[0]=0;
	
	while(i<read_size) {
		file_buf[i] = UartGetC(UART_CH6);
//							UartPutCh(7, temp[i]);
FtpFileDateCheck:
		if(file_buf[i] == '*') {
			if((file_buf[i+1] = UartGetC(UART_CH6))=='C') {
				if((file_buf[i+2] = UartGetC(UART_CH6))=='L') {
					if((file_buf[i+3] = UartGetC(UART_CH6))=='O') {
						if((file_buf[i+4] = UartGetC(UART_CH6))=='S') {
							if((file_buf[i+5] = UartGetC(UART_CH6))=='*') {
								debugprintf("---------- i = %d\r\n", i);
								end_chk[0]=1;
								return i;
							}
							else {
								debugprintf("5\r\n");
								i += 5;
								goto FtpFileDateCheck;
							}
						}
						else {
							debugprintf("4\r\n");
							i += 4;
							goto FtpFileDateCheck;
						}
					}
					else {
						debugprintf("3\r\n");
						i += 3;
						goto FtpFileDateCheck;
					}
				}
				else {
					debugprintf("2\r\n");
					i += 2;
					goto FtpFileDateCheck;
				}
			}
			else {
				debugprintf("1, i=%d\r\n", i);
				i++;
				goto FtpFileDateCheck;
			}
		}
		i++;
	}
	return i; // file size count

}
#endif //]] molink_yyb_110704_END -- .

void ExitFtpTransfer()
{
	UartPuts(6, "$$$");
	UsbUartPuts("\t => Enter WiFly Command Mode!\r\n");

	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "close\r");
	UsbUartPuts("\r\n\t => [close] Command!\r\n");
	while(1) {
		if(UartGetC(UART_CH6) == '<') break;
	}
}

void SetFtpParameters()
{
	RGBQUAD color_palette[64];
	ALIGN4 BITMAPFILEHEADER bmpfh;	
	ALIGN4 U8 linebuf[320*3];
	U8 *file_buf;
	U32 w,h;	
	int end_x, end_y;
	int x, y;
	U32 ibpl; //line pitch
	U16 tmp;
	volatile U8* frame=(volatile U8*)(LCD_BASE_ADDR|LCD_SEL_CTL_REG);
					
	int len;
	int i, j, k;
	int ret=0;
	U8 rdat;

	U8 temp[75];
	U8 tmp_buf[8][25];
	
	while(1)
	{
		UsbUartPuts("\r\n");
		UsbUartPuts("================================================\r\n");
		UsbUartPuts("   <<<< Sub Menu - Set FTP Parameters >>>>      \r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   01. Set Ftp Address                   -[A]\r\n");
		UsbUartPuts("   02. Set Ftp Port Number               -[B]\r\n");
		UsbUartPuts("   03. Set Ftp Access Directory          -[C]\r\n");
		UsbUartPuts("   04. Set Ftp Update Default File Name  -[D]\r\n");
		UsbUartPuts("   05. Set Ftp Access User Name          -[E]\r\n");
		UsbUartPuts("   06. Set Ftp Access User Password      -[F]\r\n");
		UsbUartPuts("   07. Set Ftp Access Mode               -[G]\r\n");
		UsbUartPuts("   08. Set Ftp Access Timeout            -[H]\r\n");
		UsbUartPuts("   09. Get Ftp Information               -[I]\r\n");
		UsbUartPuts("   10. Ftp File Upload                   -[J]\r\n");
		UsbUartPuts("   11. Ftp File Download                 -[K]\r\n");
		UsbUartPuts("   12. Ftp BMP File Download & Display   -[L]\r\n");
		UsbUartPuts("   13. Ftp TXT File Download & Display   -[M]\r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   14. Return Previous Menu              -[Z]\r\n");
		UsbUartPuts("================================================\r\n");

		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110624_BEGIN --  set ftp addr ???
				strcpy(temp, "Set FTP Addr[current : ");
				j = 4;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[0][j] ==':') break;
					temp[i] = tmp_buf[0][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current address[");
					tmp_buf[0][j]=0;
					UsbUartPuts(&tmp_buf[0][4]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp addr ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Address)\r\n");
						PrintLCD("-> Success, Set FTP Address!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110624_END --  set ftp addr ???
				break;

			case 'b' : case 'B' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp remote ??? (ftp port number setting)
				strcpy(temp, "Set FTP Port Number[current : ");
				j=0;
				while(1) {
					if(tmp_buf[0][j] == ':') break;
					j++;
				}
				j++;
				k = j;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[0][j] ==0x0d) break;
					temp[i] = tmp_buf[0][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current port number[");
					tmp_buf[0][j]=0;
					UsbUartPuts(&tmp_buf[0][k]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp remote ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Port Number)\r\n");
						PrintLCD("-> Success, Set FTP Port Number!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp remote ??? (ftp port number setting)
				break;

			case 'c' : case 'C' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp dir ??? (ftp access default directory setting)
				strcpy(temp, "Set FTP Access Directory[current : ");
				j = 4;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[4][j] == 0x0d) break;
					temp[i] = tmp_buf[4][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current directory[");
					tmp_buf[4][j]=0;
					UsbUartPuts(&tmp_buf[4][4]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp dir ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Access Directory)\r\n");
						PrintLCD("-> Success, Set FTP Access Directory!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp dir ??? (ftp access default directory setting)
				break;

			case 'd' : case 'D' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp filename ??? (ftp update default file setting)
				strcpy(temp, "Set FTP Update Default File Name[current : ");
				j = 5;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[1][j] == 0x0d) break;
					temp[i] = tmp_buf[1][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current file name[");
					tmp_buf[1][j]=0;
					UsbUartPuts(&tmp_buf[1][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp filename ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Update Default File)\r\n");
						PrintLCD("-> Success, Set FTP Update Default File!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp filename ??? (ftp update default file setting)
				break;

			case 'e' : case 'E' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp user ??? (ftp access user setting)
				strcpy(temp, "Set FTP Access User Name[current : ");
				j = 5;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[2][j] == 0x0d) break;
					temp[i] = tmp_buf[2][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current user name[");
					tmp_buf[2][j]=0;
					UsbUartPuts(&tmp_buf[2][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp user ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Access User Name)\r\n");
						PrintLCD("-> Success, Set FTP Access User Name!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp user ??? (ftp access user setting)
				break;

			case 'f' : case 'F' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp pass ??? (ftp access user password setting)
				strcpy(temp, "Set FTP Access User Password[current : ");
				j = 5;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[3][j] == 0x0d) break;
					temp[i] = tmp_buf[3][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current user password[");
					tmp_buf[3][j]=0;
					UsbUartPuts(&tmp_buf[3][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp pass ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Access User Password)\r\n");
						PrintLCD("-> Success, Set FTP Access User Password!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp pass ??? (ftp access user password setting)
				break;

			case 'g' : case 'G' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp mode ??? (ftp access mode setting)
				strcpy(temp, "Set FTP Access Mode[current : ");
				j = 9;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[6][j] == 0x0d) break;
					temp[i] = tmp_buf[6][j];
					i++;
					j++;
				}
				if(temp[i-1] == '0') {
					strcpy(&temp[i], "(passive)");
					i += 9;
				}
				else if(temp[i-1] == '1') {
					strcpy(&temp[i], "(active)");
					i += 8;
				}
				else {
					debugstring("ftp mode setting error!\r\n");
					break;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current ftp mode[");
					tmp_buf[6][j]=0;
					UsbUartPuts(&tmp_buf[6][9]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp mode ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Access Mode)\r\n");
						PrintLCD("-> Success, Set FTP Access Mode!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp mode ??? (ftp access mode setting)
				break;

			case 'h' : case 'H' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get ftp\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
				//[[ molink_yyb_110627_BEGIN -- set ftp time ??? (ftp timeout setting)
				strcpy(temp, "Set FTP Access Timeout[current : ");
				j = 8;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[5][j] == 0x0d) break;
					temp[i] = tmp_buf[5][j];
					i++;
					j++;
				}
				tmp_buf[5][j]=0;
				k = atoi(&tmp_buf[5][8]);
				sprintf(&temp[i], "(%3d/8 = %02d sec )", k, k/8);
				i += 16;
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current timeout[");
					tmp_buf[5][j]=0;
					UsbUartPuts(&tmp_buf[5][8]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					strcpy(temp, "set ftp timer ");
					i = strlen(temp);
					j = strlen(console_buffer);
					memcpy(&temp[i], console_buffer, j);
					temp[i+j] = '\r';
					temp[i+j+1] = 0;
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Ftp Access Timeout)\r\n");
						PrintLCD("-> Success, Set FTP Access Timeout!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				//]] molink_yyb_110627_END -- set ftp time ??? (ftp timeout setting)
				break;

			case 'i' : case 'I':
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get ftp\r");
				UsbUartPuts("\r\n\t => [get ftp] Command!\r\n");
				i = 0;
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;

			case 'j' : case 'J' :
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				strcpy(temp, "FTP Upload File Name?(default name: test.bin) : ");
				i=strlen(temp);
				temp[i] = 0;
				
				len = readline(temp);

				if(len<0) {
					UsbUartPuts("\r\n");
					UsbUartPuts("\t => Cancel File Upload\r\n");
					break;
				}
				else if(len==0) {
					strcpy(console_buffer, "test.bin");
				}

				strcpy(temp, "ftp put ");
				i = strlen(temp);
				j = strlen(console_buffer);
				memcpy(&temp[i], console_buffer, j);
				temp[i+j] = '\r';
				temp[i+j+1] = 0;
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, temp);
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat==0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat==0x0a) break;
					}
				}
				for(i=0; i<6; i++) {
					while(!UartGetCh(UART_CH6, &rdat));
					temp[i]=rdat;
				}
				if(strncmp(temp, "*OPEN*", 6)==0) {
					delayms(500);
					rdat = 0x61;
					for(i=0; i<50; i++) {
						UartPutCh(UART_CH6, rdat);
					}
					for(i=0; i<6; i++) {
						temp[i]=UartGetC(UART_CH6);
					}
					if(strncmp(temp, "*CLOS*", 6)==0) {
						UsbUartPuts("\t => Ftp Upload Complete!\r\n");
						PrintLCD("-> Success, Ftp Upload!\r\n");
						break;
					}
					else {
						temp[i]=0;
						UsbUartPuts(temp);
						UsbUartPuts("\r\n\t => Ftp Upload Fail!\r\n");
					}
					
				}
				else {
					UartRxBufFlush(UART_CH6);
					temp[4]=0;
					UsbUartPuts("\t => ");
					UsbUartPuts(temp);
					UsbUartPuts("Error!\r\n");
					UsbUartPuts("\t => Ftp Upload Fail!\r\n");
					break;
				}
				break;

			case 'k' : case 'K' :
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				strcpy(temp, "FTP Download File Name?(default name: test.bin) : ");
				i=strlen(temp);
				temp[i] = 0;
				
				len = readline(temp);

				if(len<0) {
					UsbUartPuts("\t => Cancel File Download\r\n");
					break;
				}
				else if(len==0) {
					strcpy(console_buffer, "test.bin");
				}

				strcpy(temp, "ftp get ");
				i = strlen(temp);
				j = strlen(console_buffer);
				memcpy(&temp[i], console_buffer, j);
				temp[i+j] = '\r';
				temp[i+j+1] = 0;

				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, temp);
				while(1) {
					if((rdat=UartGetC(UART_CH6)) == 0x0a) break;
				}
				while(1) {
					if((rdat=UartGetC(UART_CH6)) == 0x0a) break;
				}
				for(i=0; i<6; i++) {
					temp[i]=UartGetC(UART_CH6);
				}
				ret = 0;
				if(strncmp(temp, "*OPEN*", 6)==0) {
					for(i=0; i<3; i++) {
						temp[i] = UartGetC(UART_CH6);
					}
					if(strncmp(temp, "550", 3)==0) {
						while(1) {
							temp[i] = UartGetC(UART_CH6);
							if(temp[i] == 0x0a) break;
							i++;
						}
						i++;
						for(k=0; k<7; k++) temp[i+k] = UartGetC(UART_CH6);
						if(strncmp(&temp[i], "FTP ERR", 7)==0) {
							UsbUartPuts("\t => Ftp Download Fail!\r\n");
							UsbUartPuts("\t => ");
							temp[i] = 0;
							UsbUartPuts(temp);
							break;
						}
						i += 7;
					}
					else {
						if(temp[0] == '*') {
							if(temp[1] == 'C') {
								if(temp[2] == 'L') {
									if((temp[i] = UartGetC(UART_CH6))=='O') {
										i++;
										if((temp[i] = UartGetC(UART_CH6))=='S') {
											i++;
											if((temp[i] = UartGetC(UART_CH6))=='*') {
												i=0;
												ret=0;
												goto FtpGetEnd;
											}
											else {
												goto FtpGetDataCheck;
											}
										}
										else {
											goto FtpGetDataCheck;
										}
									}
									else {
										goto FtpGetDataCheck;
									}
								}
								else {
									goto FtpGetJmp0;
								}
							}
						}

						if(temp[1] == '*') {
							if(temp[2] == 'C') {
								if((temp[i] = UartGetC(UART_CH6)) == 'L') {
									i++;
									if((temp[i] = UartGetC(UART_CH6))=='O') {
										i++;
										if((temp[i] = UartGetC(UART_CH6))=='S') {
											i++;
											if((temp[i] = UartGetC(UART_CH6))=='*') {
												i=1;
												ret=1;
												goto FtpGetEnd;
											}
											else {
												goto FtpGetDataCheck;
											}
										}
										else {
											goto FtpGetDataCheck;
										}
									}
									else {
										goto FtpGetDataCheck;
									}
								}
								else {
									goto FtpGetDataCheck;
								}
							}
							else {
								goto FtpGetJmp0;
							}
						}
FtpGetJmp0:
						if(temp[2] == '*') {
							if((temp[i] = UartGetC(UART_CH6)) == 'C') {
								i++;
								if((temp[i] = UartGetC(UART_CH6)) == 'L') {
									i++;
									if((temp[i] = UartGetC(UART_CH6))=='O') {
										i++;
										if((temp[i] = UartGetC(UART_CH6))=='S') {
											i++;
											if((temp[i] = UartGetC(UART_CH6))=='*') {
												i=2;
												ret=2;
												goto FtpGetEnd;
											}
											else goto FtpGetDataCheck;
										}
										else goto FtpGetDataCheck;
									}
									else goto FtpGetDataCheck;
								}
								else goto FtpGetDataCheck;
							}
							else goto FtpGetDataCheck;
						}
						

					}

					while(1) {
						while(i<64) {
							temp[i] = UartGetC(UART_CH6);
FtpGetDataCheck:
							if(temp[i] == '*') {
								if((temp[i+1] = UartGetC(UART_CH6))=='C') {
									if((temp[i+2] = UartGetC(UART_CH6))=='L') {
										if((temp[i+3] = UartGetC(UART_CH6))=='O') {
											if((temp[i+4] = UartGetC(UART_CH6))=='S') {
												if((temp[i+5] = UartGetC(UART_CH6))=='*') {
													ret += i;
													goto FtpGetEnd;
												}
												else {
													i += 5;
													goto FtpGetDataCheck;
												}
											}
											else {
												i += 4;
												goto FtpGetDataCheck;
											}
										}
										else {
											i += 3;
											goto FtpGetDataCheck;
										}
									}
									else {
										i += 2;
										goto FtpGetDataCheck;
									}
								}
								else {
									i++;
									goto FtpGetDataCheck;
								}

								
							}
							i++;
						}
						ret += i; // file size count
//						UsbUartPutCh('.');
//						debugprintf("\r\ni=%d\r\n", i);
//						memcpy(file_buf, temp, i); // FTP  ٿε   ó : comment by yyb[110628]
						i=0;
					}
FtpGetEnd:
//				memcpy(file_buf, temp, i); // FTP  ٿε   ó : comment by yyb[110628]
//					debugprintf("\r\n\r\n--------> FTP Get File Size = %d\r\n", ret);
					UsbUartPuts("\t => ");
					UsbUartPuts(console_buffer);
					UsbUartPuts("[");
					sprintf(temp, "%d byte] Ftp File Download Complete!\r\n", ret);
					UsbUartPuts(temp);
					PrintLCD("-> Success, Ftp File Download!\r\n");
					break;
					
				}
				else {
					while(1) {
						temp[i] = UartGetC(UART_CH6);
						if(temp[i] == 0x0a) break;
						i++;
					}					
					UartRxBufFlush(UART_CH6);
					temp[i]=0;
					UsbUartPuts("\t => ");
					UsbUartPuts(temp);
					UsbUartPuts("\r\n");
					UsbUartPuts("\t => Ftp Download Fail!\r\n");
					break;
				}
				break;

			case 'l' : case 'L' :
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
FtpGetBmpJmp:
				strcpy(temp, "FTP Download File(bmp) Name?(default name: CanFlyTest.bmp) : ");
				i=strlen(temp);
				temp[i] = 0;
				
				len = readline(temp);

				if(len<0) {
					UsbUartPuts("\t => Cancel File Download\r\n");
					break;
				}
				else if(len==0) {
					strcpy(console_buffer, "CanFlyTest.bmp");
				}
				else {
					if(strncmp(&console_buffer[strlen(console_buffer)-4], ".bmp", 4)!=0) {
						UsbUartPuts("Error! - Enter only bmp file!\r\n\r\n");
						goto FtpGetBmpJmp;
					}
				}

				strcpy(temp, "ftp get ");
				i = strlen(temp);
				j = strlen(console_buffer);
				memcpy(&temp[i], console_buffer, j);
				temp[i+j] = '\r';
				temp[i+j+1] = 0;

				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, temp);
				while(1) {
					if((rdat=UartGetC(UART_CH6)) == 0x0a) break;
				}
				while(1) {
					if((rdat=UartGetC(UART_CH6)) == 0x0a) break;
				}
				for(i=0; i<6; i++) {
					temp[i]=UartGetC(UART_CH6);
				}
				ret = 0;
				if(strncmp(temp, "*OPEN*", 6)==0) {
					for(i=0; i<3; i++) {
						temp[i] = UartGetC(UART_CH6);
					}

					if(strncmp(temp, "550", 3)==0) {
						while(1) {
							temp[i] = UartGetC(UART_CH6);
							if(temp[i] == 0x0a) break;
							i++;
						}
						i++;
						for(k=0; k<7; k++) temp[i+k] = UartGetC(UART_CH6);
						if(strncmp(&temp[i], "FTP ERR", 7)==0) {
							UsbUartPuts("\t => Ftp Download Fail!\r\n");
							UsbUartPuts("\t => ");
							temp[i] = 0;
							UsbUartPuts(temp);
						}
						else {
							UsbUartPuts("\t => Invalid BMP-ID!\r\n");
							ExitFtpTransfer();							
						}
						break;						
					}

					file_buf = (U8 *)&bmpfh;
					
					if((temp[1] != 0x4d) || (temp[0] != 0x42)) {
						UsbUartPuts("\t => Invalid BMP-ID!\r\n");
						ExitFtpTransfer();
						break;
					}
					file_buf[0] = temp[2];
					len=3;

					ret = FtpFileDateRead(&file_buf[1], sizeof(BITMAPFILEHEADER)-1, &k);
					if(ret<=0 || k != (sizeof(BITMAPFILEHEADER)-1)) {
						UsbUartPuts("\t => BMP File Read Error!\r\n");
						ExitFtpTransfer();
						break;
					}
					len+=k;

					if( bmpfh.biBitCount == 4 )	{
						UsbUartPuts("Not support 4bit bmp\r\n");
						ExitFtpTransfer();
						break;
					}
					w = bmpfh.biWidth;	
					h = bmpfh.biHeight;	
//					end_x = w>240?240:w;	
//					end_y = h>320?320:h;
					end_x = w>320?320:w;	
					end_y = h>240?240:h;

					LCDcls(0,0,0);	// back ground block

					if(bmpfh.biBitCount==8) {
						file_buf = (U8 *)color_palette;
						ret = FtpFileDateRead(file_buf, 1024, &k);
						if(ret<=0 || k != 1024) {
							UsbUartPuts("\t => BMP File Read Error!\r\n");
							ExitFtpTransfer();
							break;
						}
						len+=k;

						for(i=0;i<256;i++) {	
							color_palette[i].rgbRed=((color_palette[i].rgbRed&0xf8)|(color_palette[i].rgbGreen&0xfc)>>5);
							color_palette[i].rgbGreen=(((color_palette[i].rgbGreen&0xfc)>>2&0x07)<<5|(color_palette[i].rgbBlue&0xf8)>>3);
						}

						ibpl = ((w*1) + 3) & 0xfffffffc;		
		
						if(ibpl>320*3) {
							UsbUartPuts("\t => Image width is too big!\r\n");
							ExitFtpTransfer();
							break;
						}		
							
						ret = FtpFileDateRead(file_buf, bmpfh.bfOffBits-len, &k);
						len+=k;

						for(y=h; y>0; y--) {
							ret = FtpFileDateRead(linebuf, ibpl, &k);
							len += k;
							if(ret<=0 || k!=ibpl) break;
							if(y>end_y) continue;
							for(i=0,x=0;x<end_x;x++) 	{
								LCDRamSet(240-y,x,240-y,x);
								*frame=(U8)color_palette[linebuf[i]].rgbRed;
								*frame=(U8)color_palette[linebuf[i]].rgbGreen;
								i++;
							}
						}

						
					}
					else if(bmpfh.biBitCount==16) {
						ibpl = ((w*2) + 3) & 0xfffffffc;
						if(ibpl>320*3) {
							UsbUartPuts("Image width is too big!\r\n");
							ExitFtpTransfer();
							break;
						}

						file_buf = (U8 *)color_palette;
						ret = FtpFileDateRead(file_buf, bmpfh.bfOffBits-len, &k);
						len+=k;

						if(bmpfh.biCompression==0) {	// 555
							for(y=h;y>0;y--) {
								ret = FtpFileDateRead(linebuf, ibpl, &k);
								len+=k;
								if(ret<=0 || k!=ibpl) break;
								if(y>end_y) continue;
								for(i=0,x=0;x<end_x;x++) 	{
									LCDRamSet(240-y,x,240-y,x);
									tmp=(linebuf[i+1]<<8|linebuf[i])<<1;
									
									*frame=tmp>>8;
									*frame=(linebuf[i]&0x1f)|(tmp&0xc0);

									i+=2;
								}
							}
						}
						else {	
							for(y=h;y>0;y--) {
								ret = FtpFileDateRead(linebuf, ibpl, &k);
								len+=k;
								if(ret<=0 || k!=ibpl) break;
								if(y>end_y) continue;
								for(i=0,x=0;x<end_x;x++) 	{
									LCDRamSet(240-y,x,240-y,x);
									*frame = linebuf[i+1];
									*frame = linebuf[i];
									i+=2;
								}
							}
						}
					}
					else if(bmpfh.biBitCount==24) {
						ibpl = ((w*3) + 3) & 0xfffffffc;
						if(ibpl>320*3) {
							UsbUartPuts("Image width is too big!\r\n");
							ExitFtpTransfer();
							break;
						}
						file_buf = (U8 *)color_palette;
						ret = FtpFileDateRead(file_buf, bmpfh.bfOffBits-len, &k);
						len+=k;

						for(y=h;y>0;y--) {
							ret = FtpFileDateRead(linebuf, ibpl, &k);
							len+=k;
							if(ret<=0 || k!=ibpl) break;
							if(y>end_y) continue;
							for(i=0,x=0;x<end_x;x++) 	{
								LCDRamSet(240-y,x,240-y,x);
								*frame=(linebuf[i+2]&0xf8)|(linebuf[i+1]&0xfc)>>5;
								*frame=((linebuf[i+1]&0x1c)<<3) | linebuf[i]>>3;
								i+=3;
							}
						}
					}

					while(1) {
						i=0;
						while(i<64) {
							temp[i] = UartGetC(UART_CH6);
FtpGetBmpDataCheck:
							if(temp[i] == '*') {
								if((temp[i+1] = UartGetC(UART_CH6))=='C') {
									if((temp[i+2] = UartGetC(UART_CH6))=='L') {
										if((temp[i+3] = UartGetC(UART_CH6))=='O') {
											if((temp[i+4] = UartGetC(UART_CH6))=='S') {
												if((temp[i+5] = UartGetC(UART_CH6))=='*') {
													ret += i;
													goto FtpGetBmpEnd;
												}
												else {
													i += 5;
													goto FtpGetBmpDataCheck;
												}
											}
											else {
												i += 4;
												goto FtpGetBmpDataCheck;
											}
										}
										else {
											i += 3;
											goto FtpGetBmpDataCheck;
										}
									}
									else {
										i += 2;
										goto FtpGetBmpDataCheck;
									}
								}
								else {
									i++;
									goto FtpGetBmpDataCheck;
								}
							}
							i++;
						}
						len += i; // file size count
//						UsbUartPutCh('.');
//						debugprintf("\r\ni=%d\r\n", i);
						// memcpy(file_buf, temp, i); // FTP  ٿε   ó : comment by yyb[110628]
					}
FtpGetBmpEnd:
//				memcpy(file_buf, temp, i); // FTP  ٿε   ó : comment by yyb[110628]
//					debugprintf("\r\n\r\n--------> FTP Get File Size = %d\r\n", ret);
					UsbUartPuts("\t => ");
					UsbUartPuts(console_buffer);
					UsbUartPuts("[");
					sprintf(temp, "%d byte] Ftp File Download Complete!\r\n", len);
					UsbUartPuts(temp);
					break;
					
				}
				else {
					while(1) {
						temp[i] = UartGetC(UART_CH6);
						if(temp[i] == 0x0a) break;
						i++;
					}					
					UartRxBufFlush(UART_CH6);
					temp[i]=0;
					UsbUartPuts("\t => ");
					UsbUartPuts(temp);
					UsbUartPuts("\r\n");
					UsbUartPuts("\t => Ftp Download Fail!\r\n");
					break;
				}
				break;

			case 'm' : case 'M' :
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
FtpGetTxtJmp:
				strcpy(temp, "FTP Download File(text) Name?(default name: CanFlyTest.txt) : ");
				i=strlen(temp);
				temp[i] = 0;
				
				len = readline(temp);

				if(len<0) {
					UsbUartPuts("\t => Cancel File Download\r\n");
					break;
				}
				else if(len==0) {
					strcpy(console_buffer, "CanFlyTest.txt");
				}
				else {
					if(strncmp(&console_buffer[strlen(console_buffer)-4], ".txt", 4)!=0) {
						UsbUartPuts("Error! - Enter only txt file!\r\n\r\n");
						goto FtpGetTxtJmp;
					}
				}

				strcpy(temp, "ftp get ");
				i = strlen(temp);
				j = strlen(console_buffer);
				memcpy(&temp[i], console_buffer, j);
				temp[i+j] = '\r';
				temp[i+j+1] = 0;

				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, temp);
				while(1) {
					if((rdat=UartGetC(UART_CH6)) == 0x0a) break;
				}
				while(1) {
					if((rdat=UartGetC(UART_CH6)) == 0x0a) break;
				}
				for(i=0; i<6; i++) {
					temp[i]=UartGetC(UART_CH6);
				}
				ret = 0;
				if(strncmp(temp, "*OPEN*", 6)==0) {
					for(i=0; i<3; i++) {
						temp[i] = UartGetC(UART_CH6);
					}

					if(strncmp(temp, "550", 3)==0) {
						while(1) {
							temp[i] = UartGetC(UART_CH6);
							if(temp[i] == 0x0a) break;
							i++;
						}
						i++;
						for(k=0; k<7; k++) temp[i+k] = UartGetC(UART_CH6);
						if(strncmp(&temp[i], "FTP ERR", 7)==0) {
							UsbUartPuts("\t => Ftp Download Fail!\r\n");
							UsbUartPuts("\t => ");
							temp[i] = 0;
							UsbUartPuts(temp);
							break;
						}
						i += 7;						
					}
					else {
						if(temp[0] == '*') {
							if(temp[1] == 'C') {
								if(temp[2] == 'L') {
									if((temp[i] = UartGetC(UART_CH6))=='O') {
										i++;
										if((temp[i] = UartGetC(UART_CH6))=='S') {
											i++;
											if((temp[i] = UartGetC(UART_CH6))=='*') {
												i=0;
												ret=0;
												goto FtpGetTxtEnd;
											}
											else {
												goto FtpGetTxtDataCheck;
											}
										}
										else {
											goto FtpGetTxtDataCheck;
										}
									}
									else {
										goto FtpGetTxtDataCheck;
									}
								}
								else {
									goto FtpGetTxtJmp0;
								}
							}
						}

						if(temp[1] == '*') {
							if(temp[2] == 'C') {
								if((temp[i] = UartGetC(UART_CH6)) == 'L') {
									i++;
									if((temp[i] = UartGetC(UART_CH6))=='O') {
										i++;
										if((temp[i] = UartGetC(UART_CH6))=='S') {
											i++;
											if((temp[i] = UartGetC(UART_CH6))=='*') {
												i=1;
												ret=1;
												goto FtpGetTxtEnd;
											}
											else {
												goto FtpGetTxtDataCheck;
											}
										}
										else {
											goto FtpGetTxtDataCheck;
										}
									}
									else {
										goto FtpGetTxtDataCheck;
									}
								}
								else {
									goto FtpGetTxtDataCheck;
								}
							}
							else {
								goto FtpGetTxtJmp0;
							}
						}
FtpGetTxtJmp0:
						if(temp[2] == '*') {
							if((temp[i] = UartGetC(UART_CH6)) == 'C') {
								i++;
								if((temp[i] = UartGetC(UART_CH6)) == 'L') {
									i++;
									if((temp[i] = UartGetC(UART_CH6))=='O') {
										i++;
										if((temp[i] = UartGetC(UART_CH6))=='S') {
											i++;
											if((temp[i] = UartGetC(UART_CH6))=='*') {
												i=2;
												ret=2;
												goto FtpGetTxtEnd;
											}
											else goto FtpGetTxtDataCheck;
										}
										else goto FtpGetTxtDataCheck;
									}
									else goto FtpGetTxtDataCheck;
								}
								else goto FtpGetTxtDataCheck;
							}
							else goto FtpGetTxtDataCheck;
						}
						

					}

					while(1) {
						while(i<64) {
							temp[i] = UartGetC(UART_CH6);
FtpGetTxtDataCheck:
							if(temp[i] == '*') {
								if((temp[i+1] = UartGetC(UART_CH6))=='C') {
									if((temp[i+2] = UartGetC(UART_CH6))=='L') {
										if((temp[i+3] = UartGetC(UART_CH6))=='O') {
											if((temp[i+4] = UartGetC(UART_CH6))=='S') {
												if((temp[i+5] = UartGetC(UART_CH6))=='*') {
													ret += i;
													goto FtpGetTxtEnd;
												}
												else {
													i += 5;
													goto FtpGetTxtDataCheck;
												}
											}
											else {
												i += 4;
												goto FtpGetTxtDataCheck;
											}
										}
										else {
											i += 3;
											goto FtpGetTxtDataCheck;
										}
									}
									else {
										i += 2;
										goto FtpGetTxtDataCheck;
									}
								}
								else {
									i++;
									goto FtpGetTxtDataCheck;
								}

								
							}
							i++;
						}
						ret += i; // file size count
//						UsbUartPutCh('.');
//						debugprintf("\r\ni=%d\r\n", i);
//						memcpy(file_buf, temp, i); // FTP  ٿε   ó : comment by yyb[110628]
						temp[i]=0;
						PrintLCD(temp);
						i=0;
					}
FtpGetTxtEnd:
					temp[i]=0;
					PrintLCD(temp);
//				memcpy(file_buf, temp, i); // FTP  ٿε   ó : comment by yyb[110628]
//					debugprintf("\r\n\r\n--------> FTP Get File Size = %d\r\n", ret);
					UsbUartPuts("\t => ");
					UsbUartPuts(console_buffer);
					UsbUartPuts("[");
					sprintf(temp, "%d byte] Ftp File Download Complete!\r\n", ret);
					UsbUartPuts(temp);
				}
				else {
					while(1) {
						temp[i] = UartGetC(UART_CH6);
						if(temp[i] == 0x0a) break;
						i++;
					}					
					UartRxBufFlush(UART_CH6);
					temp[i]=0;
					UsbUartPuts("\t => ");
					UsbUartPuts(temp);
					UsbUartPuts("\r\n");
					UsbUartPuts("\t => Ftp Download Fail!\r\n");
					break;
				}
				break;
				
			case 'z' : case 'Z' :
				return;

			default:
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}

	}
}

char Num_UartBaud(int baud)
{
	char ret = -1;
	
	switch(baud) {
		case 2400 : ret='0'; break;
		case 4800 : ret='1'; break;
		case 9600 : ret='2'; break;
		case 19200 : ret='3'; break;
		case 38400 : ret='4'; break;
		case 57600 : ret='5'; break;
		case 115200 : ret='6'; break;
		case 230400 : ret='7'; break;
		case 460800 : ret='8'; break;
		case 921600 : ret='9'; break;
		default : ret = -1; break;
	}
	
	return ret;
}

int String_UartBaud(char num)
{
	int ret = -1;
	
	switch(num) {
		case '0' : ret=2400; break;
		case '1' : ret=4800; break;
		case '2' : ret=9600; break;
		case '3' : ret=19200; break;
		case '4' : ret=38400; break;
		case '5' : ret=57600; break;
		case '6' : ret=115200; break;
		case '7' : ret=230400; break;
		case '8' : ret=460800; break;
		case '9' : ret=921600; break;
		default : ret = -1; break;
	}
	
	return ret;
}

void SetUartParameters()
{
	int len;
	int i, j, k;
	int ret=0;
	char rdat;

	U8 temp[75];
	U8 tmp_buf[8][25];
	
	while(1)
	{
		UsbUartPuts("\r\n");
		UsbUartPuts("================================================\r\n");
		UsbUartPuts("   <<<< Sub Menu - Set UART Parameters >>>>     \r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   01. Set UART Baudrate - Standard      -[a]\r\n");
		UsbUartPuts("   02. Set UART Baudrate - Non-Standard  -[b]\r\n");
		UsbUartPuts("   03. Set UART Baudrate - Instant       -[c]\r\n");
		UsbUartPuts("   04. Set UART Flow Control             -[d]\r\n");
		UsbUartPuts("   05. Set UART Mode                     -[e]\r\n");
		UsbUartPuts("   06. Set UART TX Enable/Disable        -[f]\r\n");
		UsbUartPuts("   07. Get UART Information              -[g]\r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   08. Return Previous Menu              -[z]\r\n");
		UsbUartPuts("================================================\r\n");

		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get uart\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
UartBaudJmp0:
				UsbUartPuts("\r\n");
				UsbUartPuts(" 0:2400,  1:4800,   2:9600,   3:19200,  4:38400,  \r\n");
				UsbUartPuts(" 5:57600, 6:115200, 7:230400, 8:460800, 9:921600  \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Select Number[current : ");
				j = 9;
				i = strlen(temp);
				k = atoi(&tmp_buf[0][j]);
				while(1) {
					if(tmp_buf[0][j] == 0) break;
					temp[i] = tmp_buf[0][j];
					i++;
					j++;
				}
				
				if((rdat = Num_UartBaud(k)) < 0) {
					strcpy(&temp[i], "(Non-Standard");
					i += 13;
				}
				else {
					strcpy(&temp[i], "(No. ");
					i += 5;
					temp[i++] = rdat;
				}
				
				temp[i++] = ')';
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current baudrate[");
					UsbUartPuts(&tmp_buf[0][9]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					if(strlen(console_buffer)>1) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto UartBaudJmp0;
					}
					else {
						if((k = String_UartBaud(console_buffer[0]))<0) {
							UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
							goto UartBaudJmp0;
						}
					}
					sprintf(temp, "set uart baud %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set UART Baudrate)\r\n");
						PrintLCD("-> Success, Set UART Baudrate!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'b' : case 'B' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get uart\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
UartBaudJmp1:
				strcpy(temp, "Enter Baudrate[current : ");
				j = 9;
				i = strlen(temp);
				while(1) {
					if(tmp_buf[0][j] == 0) break;
					temp[i] = tmp_buf[0][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current baudrate[");
					UsbUartPuts(&tmp_buf[0][9]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = atoi(console_buffer);
					if(k<2400 || k>1100000) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto UartBaudJmp1;
					}

					sprintf(temp, "set uart raw %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set UART Baudrate)\r\n");
						PrintLCD("-> Success, Set UART Baudrate!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'c' : case 'C' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get uart\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
UartBaudJmp2:
				UsbUartPuts("\r\n");
				UsbUartPuts(" 0:2400,  1:4800,   2:9600,   3:19200,  4:38400,  \r\n");
				UsbUartPuts(" 5:57600, 6:115200                                \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Select Number[current : ");
				j = 9;
				i = strlen(temp);
				k = atoi(&tmp_buf[0][j]);
				while(1) {
					if(tmp_buf[0][j] == 0) break;
					temp[i] = tmp_buf[0][j];
					i++;
					j++;
				}
				if((rdat = Num_UartBaud(k)) < 0) {
					strcpy(&temp[i], "(Non-Standard");
					i += 13;
				}
				else {
					strcpy(&temp[i], "(No. ");
					i += 5;
					temp[i++] = rdat;
				}
				
				temp[i++] = ')';
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current baudrate[");
					UsbUartPuts(&tmp_buf[0][9]);
					UsbUartPuts("] - not immediately change!\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					if(strlen(console_buffer)>1) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto UartBaudJmp2;
					}
					else {
						k = String_UartBaud(console_buffer[0]);
						if(k<0 || k>115200) {
							UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
							goto UartBaudJmp2;
						}
					}
					sprintf(temp, "set uart instant %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					delayms(1);
					UartRxBufFlush(UART_CH6);

					UartConfig(UART_CH6,k,DATABITS_8,STOPBITS_1,UART_PARNONE);

					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set UART Baudrate - Instant)\r\n");
						PrintLCD("-> Success, Set UART Baudrate - Instant!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'd' : case 'D' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get uart\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
UartBaudJmp3:
				UsbUartPuts("\r\n");
				UsbUartPuts(" 0:Disable,  1:Enable \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Select UART Flow Control[current : ");
				j = 7;
				i = strlen(temp);
				while(1) {
					if(tmp_buf[1][j] == 0) break;
					temp[i] = tmp_buf[1][j];
					i++;
					j++;
				}
				if(tmp_buf[1][7] == '0') {
					strcpy(&temp[i],"(Disabled)] - ");
				}
				else if(tmp_buf[1][7] == '1') {
					strcpy(&temp[i],"(Enabled)] - ");
				}
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current flow control[");
					UsbUartPuts(&tmp_buf[1][7]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = atoi(console_buffer);
					if(k!=0 && k!=1) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto UartBaudJmp3;
					}

					sprintf(temp, "set uart flow %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set UART Flow Control)\r\n");
						PrintLCD("-> Success, Set UART Flow Control!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'e' : case 'E' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get uart\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
UartBaudJmp4:
				UsbUartPuts("\r\n");
				UsbUartPuts(" Bit  Function                                            \r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("  0   NOECHO-disable echo of RX data while in command mode\r\n");
				UsbUartPuts("  1   DATA TRIGGER makes connection on RX data            \r\n");
				UsbUartPuts("  2   Reserved for future use                             \r\n");
				UsbUartPuts("  3   Enable Sleep on RX BREAK signal                     \r\n");
				UsbUartPuts("  4   UART RX data buffer - flushed                       \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Set UART Mode Register[current : ");
				j = 5;
				i = strlen(temp);
				while(1) {
					if(tmp_buf[2][j] == 0) break;
					temp[i] = tmp_buf[2][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current mode[");
					UsbUartPuts(&tmp_buf[2][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					char *pt=NULL;
					k = strtoul(console_buffer, &pt, 16);
					if(k<0 || k>0x1f) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto UartBaudJmp4;
					}

					sprintf(temp, "set uart mode 0x%x\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set UART Mode Register)\r\n");
						PrintLCD("-> Success, Set UART Mode Register!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'f' : case 'F' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
#if 0 //[[ molink_yyb_110630_BEGIN -- .
				UartPuts(UART_CH6, "get uart\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
#endif //]] molink_yyb_110630_END -- .
UartBaudJmp5:
				UsbUartPuts("\r\n");
				UsbUartPuts(" 0:Disable,  1:Enable \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Select UART TX Control - ");
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current TX control\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = atoi(console_buffer);
					if(k!=0 && k!=1) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto UartBaudJmp5;
					}

					sprintf(temp, "set uart tx %d\r", k);
					if(k==0) {
						UsbUartPuts("TX cannot be disabled!\r\n");
						UsbUartPuts("This function cannot use except for special cases!\r\n");
						break;
					}
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							UartPutCh(7, rdat);
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							UartPutCh(7, rdat);
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set UART TX Control");
						if(k) UsbUartPuts(" - Enabled)\r\n");
						else  UsbUartPuts(" - Disabled)\r\n");
						PrintLCD("-> Success, Set UART TX Control!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'g' : case 'G':
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get uart\r");
				UsbUartPuts("\r\n\t => [get uart] Command!\r\n");
				i = 0;
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;
				
			case 'z' : case 'Z' :
				return;

			default:
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}

	}
}

void WiFly_UartBaudSetting()
{
	int k;
	int len;
	U8 temp[75];
	
	while(1) {
		UsbUartPuts("\r\n");
		UsbUartPuts("          < Select WiFly(RN-171) Current UART BaudRate >\r\n");
		UsbUartPuts("\r\n");
		UsbUartPuts(" 0:2400, 1:4800, 2:9600, 3:19200, 4:38400, 5:57600, 6:115200\r\n");
		UsbUartPuts("\r\n");
		strcpy(temp, "Select Number - ");

		len = readline(temp);
				
		if(len<=0) {
			UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
		}
		else {
			if(strlen(console_buffer)>1) {
				UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
			}
			else {
				k = String_UartBaud(console_buffer[0]);
				if(k<0 || k>115200) {
					UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
				}
				else {
					UartConfig(UART_CH6,k,DATABITS_8,STOPBITS_1,UART_PARNONE);
					UsbUartPuts("\r\n ==> WiFly(RN-171)'s uart baudrate is set to ");
					sprintf(temp, "%d\r\n", k);
					UsbUartPuts(temp);
					break;
				}
			}
		}
	}
}

void SetWlanParameters()
{
	int len;
	int i, j, k;
	int ret=0;
	char rdat;
	char *pt=NULL;

	U8 temp[75];
	U8 tmp_buf[11][25];
	
	while(1)
	{
		UsbUartPuts("\r\n");
		UsbUartPuts("================================================\r\n");
		UsbUartPuts("   <<<< Sub Menu - Set WLAN Parameters >>>>     \r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   01. Set WLAN Authentication           -[A]\r\n");
		UsbUartPuts("   02. Set WLAN Connecting Fixed Channel -[B]\r\n");
		UsbUartPuts("   03. Set WLAN Joining Way              -[C]\r\n");
		UsbUartPuts("   04. Set WLAN PassKey Hide             -[D]\r\n");
		UsbUartPuts("   05. Set WLAN WEP Password             -[E]\r\n");
		UsbUartPuts("   06. Set WLAN WPA Password             -[F]\r\n");
		UsbUartPuts("   07. Set WLAN Link Monitor             -[G]\r\n");
		UsbUartPuts("   08. Set WLAN Using Channel Mask       -[H]\r\n");
		UsbUartPuts("   09. Set WLAN WEP Key Num              -[I]\r\n");
		UsbUartPuts("   10. Set WLAN Rate                     -[J]\r\n");
		UsbUartPuts("   11. Set WLAN SSID                     -[K]\r\n");
		UsbUartPuts("   12. Set WLAN TX Power                 -[L]\r\n");
		UsbUartPuts("   13. Set WLAN Buffer Window Size       -[M]\r\n");
		UsbUartPuts("   14. Get WLAN Information              -[N]\r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   15. Return Previous Menu              -[Z]\r\n");
		UsbUartPuts("================================================\r\n");

		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
WlanAuthJmp0:
				UsbUartPuts("\r\n");
				UsbUartPuts("ValueAuthentication Mode          \r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("  0  Open                         \r\n");
				UsbUartPuts("  1  WEP-128                      \r\n");
				UsbUartPuts("  2  WPA1                         \r\n");
				UsbUartPuts("  3  Mixed WPA1 & WPA2-PSK        \r\n");
				UsbUartPuts("  4  WPA2-PSK                     \r\n");
				UsbUartPuts("  5  Not Used                     \r\n");
				UsbUartPuts("  6  Adhoc, Join any Adhoc network\r\n");
				UsbUartPuts("  7  Not Used                     \r\n");
				UsbUartPuts("  8  WEP-64                       \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Set Authentication Mode[current : ");
				
				j = 5;
				i = strlen(temp);
				while(1) {
					if(tmp_buf[4][j] == 0) break;
					temp[i] = tmp_buf[4][j];
					i++;
					j++;
				}
				
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current authentication[");
					UsbUartPuts(&tmp_buf[4][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					if(strlen(console_buffer)>1) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto WlanAuthJmp0;
					}
					else {
						k = atoi(console_buffer);
						if(k>8) {
							UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
							goto WlanAuthJmp0;
						}
					}
					sprintf(temp, "set wlan auth %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Authentication)\r\n");
						PrintLCD("-> Success, Set WLAN Authentication!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'b' : case 'B' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
WlanAuthJmp1:
				strcpy(temp, "Set WLAN Fixed Channel[current : ");
				
				j = 5;
				i = strlen(temp);
				while(1) {
					if(tmp_buf[1][j] == 0) break;
					temp[i] = tmp_buf[1][j];
					i++;
					j++;
				}
				k = atoi(&tmp_buf[1][5]);
				if(k==0) {
					strcpy(&temp[i], "(not fixed)");
					i += 11;
				}
				
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current channel[");
					UsbUartPuts(&tmp_buf[1][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					if(strlen(console_buffer)>2) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto WlanAuthJmp1;
					}
					else {
						k = atoi(console_buffer);
						if(k>13) {
							UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
							goto WlanAuthJmp1;
						}
					}
					sprintf(temp, "set wlan channel %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Fixed Channel)\r\n");
						PrintLCD("-> Success, Set WLAN Fixed Channel!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}				
				break;

			case 'c' : case 'C' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
WlanAuthJmp2:
				UsbUartPuts("\r\n");
				UsbUartPuts("Value Policy                                              \r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("  0   Manual, don't try to join automatically             \r\n");
				UsbUartPuts("  1   Auto, try to join(stored SSID, Passkey, Channel)    \r\n");
				UsbUartPuts("  2   Join ANY AP with matching the stored authentication \r\n");
				UsbUartPuts("  3   Reserved - Not Used                                 \r\n");
				UsbUartPuts("  4   Create an Adhoc network                             \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Set the way of Joining AP[current : ");
				
				j = 5;
				i = strlen(temp);
				while(1) {
					if(tmp_buf[3][j] == 0) break;
					temp[i] = tmp_buf[3][j];
					i++;
					j++;
				}
				
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current joining way[");
					UsbUartPuts(&tmp_buf[3][5]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					if(strlen(console_buffer)>1) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto WlanAuthJmp2;
					}
					else {
						k = atoi(console_buffer);
						if(k>4) {
							UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
							goto WlanAuthJmp2;
						}
					}
					sprintf(temp, "set wlan join %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Join)\r\n");
						PrintLCD("-> Success, Set WLAN Join!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}				
				break;

			case 'd' : case 'D' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
WlanAuthJmp3:
				UsbUartPuts("\r\n");
				UsbUartPuts(" 0:Unhide,  1:Hide  \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Set WLAN PassKey Hide[current PassKey : ");
				if(tmp_buf[8][0]=='P') {
					i=8;
					j=11;
				}
				else {
					i=9;
					j=4;
				}
				if(tmp_buf[i][j]=='*' && tmp_buf[i][j+1]=='*') k=0xfff;
				else k=0xaaa;
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current hide type\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					if(strlen(console_buffer)>1) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto WlanAuthJmp3;
					}
					else {
						i = atoi(console_buffer);
						if(i!=0 && i!=1) {
							UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
							goto WlanAuthJmp3;
						}
					}
					
					sprintf(temp, "set wlan hide %d\r", i);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					j=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[j]=rdat;
							j++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN PassKey Hide)\r\n");
						PrintLCD("-> Success, Set WLAN PassKey Hide!\r\n");

						if(i==0) {
							if(k==0xfff) {
								UsbUartPuts("--> To unhide the passkey,\r\n");
								UsbUartPuts("    re-enter the key or passphrase\r\n");
								UsbUartPuts("    using the set wlan key or set wlan passphrase command\r\n");
							}
						}
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}				
				break;

			case 'e' : case 'E' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				if(tmp_buf[8][0]=='P') {
					UsbUartPuts("! The current authentication mode isn't WEP\r\n");
					UsbUartPuts("! The current key input is applied only WEP Auth Mode\r\n");
					i=8;
					j=11;					
				}
				else {
					i=9;
					j=4;
				}
				strcpy(temp, "Set WLAN WEP Password[current Password : ");
				
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					if(j==11) UsbUartPuts("\t => Keep the current password\r\n");
					else {
						UsbUartPuts("\t => Keep the current password[");
						UsbUartPuts(&tmp_buf[i][j]);
						UsbUartPuts("]\r\n");
					}
				}
				else if(len<0) {
					break;
				}
				else {
					sprintf(temp, "set wlan key %s\r", console_buffer);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN WEP Password)\r\n");
						PrintLCD("-> Success, Set WLAN WEP Password!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'f' : case 'F' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				if(tmp_buf[8][0]!='P') {
					UsbUartPuts("! The current authentication mode isn't WPA\r\n");
					UsbUartPuts("! The current pass phrase input is applied only WPA Auth Mode\r\n");
					i=9;
					j=4;
				}
				else {
					i=8;
					j=11;
				}
				strcpy(temp, "Set WLAN WPA Password[current PassPhrase : ");
				
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					if(j==4) UsbUartPuts("\t => Keep the current password\r\n");
					else {
						UsbUartPuts("\t => Keep the current phrase[");
						UsbUartPuts(&tmp_buf[i][j]);
						UsbUartPuts("]\r\n");
					}
				}
				else if(len<0) {
					break;
				}
				else {
					for(i=0; i<len; i++) {
						if(console_buffer[i]==' ') console_buffer[i]='$';
					}
					
					sprintf(temp, "set wlan phrase %s\r", console_buffer);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN WPA PassPhrase)\r\n");
						PrintLCD("-> Success, Set WLAN WPA PassPhrase!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'g' : case 'G' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
WlanAuthJmp4:
				UsbUartPuts("\r\n ! Recommended value is 5\r\n");
				strcpy(temp, "Set WLAN Link Monitor[current value : ");
				i=7;
				j=8;
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);

				k=atoi(&tmp_buf[i][j]);
				if(k==0) {
					strcpy(&temp[ret], "(default)");
					ret = strlen(temp);
				}
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current value[");
					UsbUartPuts(&tmp_buf[i][j]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
						goto WlanAuthJmp4;
					}
					
					sprintf(temp, "set wlan linkmon %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Link Monitor)\r\n");
						PrintLCD("-> Success, Set WLAN Link Monitor!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'h' : case 'H' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				i=5;
				j=5;
WlanAuthJmp5:
				ret = strtoul(&tmp_buf[i][j], &pt, 16);
				UsbUartPuts("\r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("Channel 13121110090807060504030201\r\n");
				UsbUartPuts("Masked  ");
				for(k=12; k>=0; k--) {
					if(ret & (1<<k)) UsbUartPuts(" 1");
					else UsbUartPuts(" 0");
				}
				UsbUartPuts("\r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Set WLAN Channel Mask[current value : ");

				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);

				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current channel mask[");
					UsbUartPuts(&tmp_buf[i][j]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 16);
					if(*pt!=0) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp5;
					}
					
					sprintf(temp, "set wlan mask 0x%x\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Channel Mask)\r\n");
						UsbUartPuts("\t => Masked Channel ");
						for(i=12; i>=0; i--) {
							if(k & (1<<i)) {
								sprintf(temp, "%d ", i+1);
								UsbUartPuts(temp);
							}
						}
						UsbUartPuts("\r\n");
						PrintLCD("-> Success, Set WLAN Channel Mask!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'i' : case 'I' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				i=8;
				j=7;
WlanAuthJmp6:
				if(tmp_buf[i][0] != 'K') {
					UsbUartPuts("\r\n");
					UsbUartPuts("! The current authentication mode isn't WEP\r\n");
					UsbUartPuts("! The current WEP key input is applied only WEP Auth Mode\r\n");
					strcpy(temp, "Set WLAN WEP Key Number[enter number (1~4)");
				}
				else {
					UsbUartPuts("\r\n");
					strcpy(temp, "Set WLAN WEP Key Number[current key : ");
					ret = strlen(temp);
					strcpy(&temp[ret], &tmp_buf[i][j]);
				}
				ret = strlen(temp);

				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					if(tmp_buf[i][0] != 'K') {
						UsbUartPuts("\t => Keep the current WEP key\r\n");
					}
					else {
						UsbUartPuts("\t => Keep the current WEP key[");
						UsbUartPuts(&tmp_buf[i][j]);
						UsbUartPuts("]\r\n");
					}
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp6;
					}
					if(k<1 || k>4) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp6;
					}
					
					sprintf(temp, "set wlan num %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN WEP Key Number)\r\n");
						PrintLCD("-> Success, Set WLAN WEP Key Number!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'j' : case 'J' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
WlanAuthJmp7:
				UsbUartPuts("\r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("Value Wireless Data Rate    \r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("  0              1 Mbit/sec \r\n");
				UsbUartPuts("  1              2 Mbit/sec \r\n");
				UsbUartPuts("  2            5.5 Mbit/sec \r\n");
				UsbUartPuts("  3             11 Mbit/sec \r\n");
				UsbUartPuts(" 4~7                Invalid \r\n");
				UsbUartPuts("  8              6 Mbit/sec \r\n");
				UsbUartPuts("  9              9 Mbit/sec \r\n");
				UsbUartPuts(" 10             12 Mbit/sec \r\n");
				UsbUartPuts(" 11             18 Mbit/sec \r\n");
				UsbUartPuts(" 12   (default) 24 Mbit/sec \r\n");
				UsbUartPuts(" 13             36 Mbit/sec \r\n");
				UsbUartPuts(" 14             48 Mbit/sec \r\n");
				UsbUartPuts(" 15             54 Mbit/sec \r\n");
				UsbUartPuts("\r\n");
				strcpy(temp, "Set the wireless data rate(Select Value)[current : ");

				i=6;
				j=5;
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);				
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current wlan rate[");
					UsbUartPuts(&tmp_buf[i][j]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp7;
					}
					if((k>3 && k<8) || k>15) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp7;
					}
					
					sprintf(temp, "set wlan rate %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Rate)\r\n");
						PrintLCD("-> Success, Set WLAN Rate!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}				
				break;

			case 'k' : case 'K' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				i=0;
				j=5;
				strcpy(temp, "Set WLAN SSID[current SSID : ");
				
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
WlanAuthJmp8:
				UsbUartPuts("\r\n");
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current ssid[");
					UsbUartPuts(&tmp_buf[i][j]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else if(len>32) {
					UsbUartPuts("Characters in SSID too long. Only 1~32 chars. Please Re-enter!\r\n");
					goto WlanAuthJmp8;
				}
				else {
					for(i=0; i<len; i++) {
						if(console_buffer[i]==' ') console_buffer[i]='$';
					}
					
					sprintf(temp, "set wlan ssid %s\r", console_buffer);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN SSID)\r\n");
						PrintLCD("-> Success, Set WLAN SSID!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'l' : case 'L' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get wlan\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
WlanAuthJmp9:
				UsbUartPuts("\r\nTx Power Setting Range = 0~12\r\n");
				strcpy(temp, "Set WLAN Tx Power[current : ");

				if(tmp_buf[8][0] == 'P') i=9;
				else i=10;
				j=8;
				ret = strlen(temp);
				strcpy(&temp[ret], &tmp_buf[i][j]);
				ret = strlen(temp);

				k=atoi(&tmp_buf[i][j]);
				if(k==0) {
					strcpy(&temp[ret], "(12dBm)");
				}
				else {
					sprintf(&temp[ret], "(%ddBm)", k);
				}
				ret = strlen(temp);
				
				temp[ret++] = ']';
				temp[ret++] = ' ';
				temp[ret++] = '-';
				temp[ret++] = ' ';
				temp[ret] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current tx power[");
					UsbUartPuts(&tmp_buf[i][j]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp9;
					}
					if(k>12) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp9;
					}
					
					sprintf(temp, "set wlan tx %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Tx Power)\r\n");
						PrintLCD("-> Success, Set WLAN Tx Power!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}				
				break;

			case 'm' : case 'M' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				
WlanAuthJmp10:
				UsbUartPuts("\r\nIP maximum buffer window size => default is 1460 bytes.\r\n");
				strcpy(temp, "Set WLAN Buffer Window Size - ");

				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current window size\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto WlanAuthJmp10;
					}
					
					sprintf(temp, "set wlan window %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set WLAN Window Size)\r\n");
						PrintLCD("-> Success, Set WLAN Window Size!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}				
				break;

			case 'n' : case 'N' :
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get wlan\r");
				UsbUartPuts("\r\n\t => [get wlan] Command!\r\n");
				i = 0;
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;
				
			case 'z' : case 'Z' :
				return;

			default:
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}

	}
}

U8 Scan_Aps[13][25];
U8 Aps_Num=0;
void Scan_APs()
{
	int ver;
	int i,j,k,m,n,p;
	int ret;
	U8 rdat;
	U8 tmp[200];
	U8 temp[30];
	char *pt=NULL;
	
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) return;
	}
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "get sys\r");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
		}
	}
	i=0;
	while(1) {
		rdat = UartGetC(UART_CH6);
		UartPutCh(7, rdat);
		tmp[i++] = rdat;
		if(rdat==0x0a) {			
			if(strncmp(tmp, "PrintLvl=", 9)==0) {
				k = strtoul(&tmp[9], &pt, 16);
				debugprintf("\r\nk=0x%x\r\n", k);
				break;
			}
			i=0;
		}
	}
	UartRxBufFlush(UART_CH6);

	if(k!=0x4000) {
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "set sys printlvl 0x4000\r");
		while(1) {
			if(UartGetCh(UART_CH6, &rdat)) {
				UartPutCh(7, rdat);
				if(rdat == 0x0a) break;
			}
		}
		UartRxBufFlush(UART_CH6);
	}

	debugprintf("\r\n________ scan start~\r\n");
	
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "scan\r");

	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
		}
	}
	i=0;
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
			tmp[i] = rdat;
			i++;
		}
	}
	tmp[5]=0;
	ver=atoi(&tmp[3]);
	
	i=0;
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
			tmp[i] = rdat;
			i++;
		}
		
	}
	tmp[12]=0;
	j=atoi(&tmp[11]);
	Aps_Num=j;
//	debugprintf("SCAN:Found=%d\r\n", j);

	UsbUartPuts("\r\n");
	UsbUartPuts("NumSSID               ChRSSI(dBm)Security Mode  CapabilitiesMAC Address       \r\n");
	UsbUartPuts("\r\n");
	
	for(i=0; i<j; i++) {
		sprintf(temp, "%02d ", i+1);
		UsbUartPuts(temp);
		m=0;
		while(1) {
			rdat = UartGetC(UART_CH6);
			UartPutCh(7, rdat);
			tmp[m++] = rdat;
			if(rdat==0x0a) {
				tmp[m-2] = 0;
				if(ver>23) p=42;
				else p=36;
				sprintf(Scan_Aps[i], "%s", &tmp[p]);
				UsbUartPuts(Scan_Aps[i]);
				m=strlen(&tmp[p]);
				for(n=m; n<19; n++) UsbUartPutCh(' ');
				UsbUartPuts("");

				tmp[5]=0;
				sprintf(temp, "%s", &tmp[3]);
				UsbUartPuts(temp);

				tmp[9]=0;
				sprintf(temp, "   %s   ", &tmp[6]);
				UsbUartPuts(temp);

				tmp[12]=0;
				n=strtoul(&tmp[10], &pt, 10);
				switch(n) {
					case 0: 	UsbUartPuts("Open           "); break;
					case 1: UsbUartPuts("WEP            "); break;
					case 2: UsbUartPuts("WPA-PSK        "); break;
					case 3: UsbUartPuts("WPA-Mixed      "); break;
					case 4: UsbUartPuts("WPA2-PSK       "); break;
					case 6: UsbUartPuts("WPA-Enterprise "); break;
					case 8: UsbUartPuts("WPA2-Enterprise"); break;
					default: break;
				}

				tmp[17]=0;
				n=strtoul(&tmp[13], &pt, 10);
				sprintf(temp, "    %04d    ", n);
				UsbUartPuts(temp);

				tmp[p-1]=0;
				if(ver>23) p=24;
				else p=18;
				sprintf(temp, "%s \r\n", &tmp[p]);
				UsbUartPuts(temp);
				break;
			}
		}
	}
	UsbUartPuts("\r\n");

	UartRxBufFlush(UART_CH6);
	sprintf(tmp, "set sys printlvl 0x%x\r", k);
	UartPuts(UART_CH6, tmp);
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
		}
	}
	UartRxBufFlush(UART_CH6);
}

void Join_AP()
{
	int i,j,k;
	int ret;
	U8 tmp[200];
	char *pt;
	U8 rdat;
	int err;
	
	i=0; j=0;
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) return;
	}
	if(Aps_Num==0) {
		UsbUartPuts("Error!\r\n");
		UsbUartPuts("First of all, Excute Scan Command\r\n\r\n");
		return;
	}

JoinAPJmp:
	UsbUartPuts("\r\n");
	UsbUartPuts("NumSSID                 \r\n");
	UsbUartPuts("\r\n");

	for(i=0; i<Aps_Num; i++) {
		sprintf(tmp, "%02d ", i+1);
		UsbUartPuts(tmp);
		k=strlen(Scan_Aps[i]);
		UsbUartPuts(Scan_Aps[i]);
		for(j=k; j<21; j++) UsbUartPutCh(' ');
		UsbUartPuts("\r\n");
	}
	UsbUartPuts("\r\n");
	strcpy(tmp, "Select Join AP Number : ");
				
	k = readline(tmp);
	
	if(k<=0) {
		return;
	}
	else {
		k = strtoul(console_buffer, &pt, 10);
		if(*pt!=0) {
			UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
			goto JoinAPJmp;
		}
		if(k>=Aps_Num) {
			UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
			goto JoinAPJmp;
		}

		sprintf(tmp, "join %s\r", Scan_Aps[k-1]);
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, tmp);
		i=0; j=0; ret=0;
		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			if(err) {
				ret++;
				if(ret>0) {
					sprintf(tmp, "\t==> * %s joining fail!\r\n", Scan_Aps[k-1]);
					UsbUartPuts(tmp);
					return;
				}
				continue;
			}
			UartPutCh(7,rdat);
			tmp[i++] = rdat;
			if(rdat==0x0a) {
				pt=strstr(tmp, "Associated!");
				if(pt>0) break;
				i=0;
				j++;
			}
			if(j>6) {
				sprintf(tmp, "\t==> %s joining fail!\r\n", Scan_Aps[k-1]);
				UsbUartPuts(tmp);
				return;
			}
		}
		
		i=0; j=0; ret=0;
		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			if(err) {
				ret++;
				if(ret>0) {
					sprintf(tmp, "\t==> * %s joining fail!\r\n", Scan_Aps[k-1]);
					UsbUartPuts(tmp);
					return;
				}
				continue;
			}
			UartPutCh(7,rdat);
			tmp[i++] = rdat;
			if(rdat==0x0a) {
				pt=strstr(tmp, "GW=");
				if(pt>0) break;
				i=0;
				j++;
			}
			if(j>10) {
				sprintf(tmp, "\t==> %s joining fail!\r\n", Scan_Aps[k-1]);
				UsbUartPuts(tmp);
				return;
			}
		}		
		UartRxBufFlush(UART_CH6);

		sprintf(tmp, "\t==> OK!(%s joining success)\r\n", Scan_Aps[k-1]);
		UsbUartPuts(tmp);
		PrintLCD("-> Success, Join AP!\r\n");
	}
}

void Status_Command()
{
	U8 rdat;
	U8 temp[50];
	int len;
	int k;
	char *pt;
	
	while(1)
	{
		UsbUartPuts("\r\n");
		UsbUartPuts("================================================\r\n");
		UsbUartPuts("   <<<< Sub Menu - Status Command >>>>     \r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   01. show battery                -[A]\r\n");
		UsbUartPuts("   02. show connection             -[B]\r\n");
		UsbUartPuts("   03. show io                     -[C]\r\n");
		UsbUartPuts("   04. show net                    -[D]\r\n");
		UsbUartPuts("   05. show rssi                   -[E]\r\n");
		UsbUartPuts("   06. show stats                  -[F]\r\n");
		UsbUartPuts("   07. show time                   -[G]\r\n");
		UsbUartPuts("   08. show q <sensor no.>         -[H]\r\n");
		UsbUartPuts("   09. show q 0x1<sensor mask>     -[I]\r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   10. Return Previous Menu        -[Z]\r\n");
		UsbUartPuts("================================================\r\n");

		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show battery\r");
				UsbUartPuts("\r\n\t => [show battery] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'b' : case 'B' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show connection\r");
				UsbUartPuts("\r\n\t => [show connection] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'c' : case 'C' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show io\r");
				UsbUartPuts("\r\n\t => [show io] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'd' : case 'D' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show net\r");
				UsbUartPuts("\r\n\t => [show net] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'e' : case 'E' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show rssi\r");
				UsbUartPuts("\r\n\t => [show rssi] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'f' : case 'F' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show stats\r");
				UsbUartPuts("\r\n\t => [show stats] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'g' : case 'G' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show time\r");
				UsbUartPuts("\r\n\t => [show time] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 'h' : case 'H' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}

SensorJmp0:
				UsbUartPuts("\r\nSensor Number Range = 0 ~ 7\r\n");
				strcpy(temp, "For showing value of sensor, Select Number : ");
			
				len = readline(temp);
				
				if(len<=0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto SensorJmp0;
					}
					if(k>7) {
						UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
						goto SensorJmp0;
					}
					
					sprintf(temp, "show q %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					UsbUartPuts("\r\n\t => [show q <sensor no.>] Command!\r\n");
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat=='<') break;
							UsbUartPutCh(rdat);
						}
					}
					UartRxBufFlush(UART_CH6);
				}
				break;

			case 'i' : case 'I' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}

SensorJmp1:
				UsbUartPuts("\r\nSensor Mask Range = 0x1 ~ 0xFF\r\n");
				strcpy(temp, "For showing value of sensors, Enter Mask : ");
			
				len = readline(temp);
				
				if(len<=0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 16);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto SensorJmp1;
					}
					if(k==0x0 || k>0xff) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto SensorJmp1;
					}
					
					sprintf(temp, "show q 0x1%02x\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					UsbUartPuts("\r\n\t => [show q 0x1<sensor mask>] Command!\r\n");
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat=='<') break;
							UsbUartPutCh(rdat);
						}
					}
					UartRxBufFlush(UART_CH6);
				}
				break;

			case 'z' : case 'Z' :
				return;

			default :
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}
	}
}

void SetSensorParameters()
{
	U8 rdat;
	U8 temp[50];
	int len;
	int i,j,k;
	int ret;
	char *pt;
	U8 tmp_buf[3][25];
	
	while(1)
	{
		UsbUartPuts("\r\n");
		UsbUartPuts("================================================\r\n");
		UsbUartPuts("   <<<< Sub Menu - Sensor Parameters >>>>     \r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   01. Set Using Sensor Mask          -[A]\r\n");
		UsbUartPuts("   02. Set Using Sensor Power         -[B]\r\n");
		UsbUartPuts("   03. Get Sensor                     -[C]\r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("   04. Return Previous Menu           -[Z]\r\n");
		UsbUartPuts("================================================\r\n");

		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}

SensorParamJmp0:
				UsbUartPuts("\r\nSensor Mask Range = 0x1 ~ 0xFF\r\n");
				strcpy(temp, "For using sensors, Enter Mask : ");
			
				len = readline(temp);
				
				if(len<=0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 16);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto SensorParamJmp0;
					}
					if(k==0x0 || k>0xff) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto SensorParamJmp0;
					}
					
					sprintf(temp, "set q sensor 0x%02x\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Sensor Mask)\r\n");
						PrintLCD("-> Success, Set Sensor Mask!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'b' : case 'B' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get q\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);
				
SensorParamJmp1:
				UsbUartPuts("\r\n");
				UsbUartPuts("Value Sensor pin voltage                \r\n");
				UsbUartPuts("\r\n");
				UsbUartPuts("  0   Turn off the sensor power         \r\n");
				UsbUartPuts("  1   Ground the sensor pin             \r\n");
				UsbUartPuts("  2   1.2V internal regulated reference \r\n");
				UsbUartPuts("  3   VBATT input pin                   \r\n");
				UsbUartPuts("  4   3.3V output of on board regulator \r\n");
				UsbUartPuts("\r\n");
				sprintf(temp, "Set using the sensor power[current : %s] - ", &tmp_buf[1][9]);

				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current sensor power[");
					UsbUartPuts(&tmp_buf[1][9]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 16);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto SensorParamJmp1;
					}
					if(k>0x40) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto SensorParamJmp1;
					}

					sprintf(temp, "set q power 0x%02x\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Sensor Power)\r\n");
						PrintLCD("-> Success, Set Sensor Power!\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}	
				break;

			case 'c' : case 'C' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get q\r");
				UsbUartPuts("\r\n\t => [get q] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);
				break;

			case 'z' : case 'Z' :
				return;

			default :
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}
	}
}

void PrintCommandWiFly()
{
	UsbUartPuts("\r\n");
	UsbUartPuts("====================================================================\r\n");
	UsbUartPuts("                 <<<< WiFly Command Selection >>>>               \r\n");
	UsbUartPuts("--------------------------------------------------------------------\r\n");
	UsbUartPuts("   1. Enter Command Mode       -[A]   |   6. Get Everything    -[J] \r\n");
	UsbUartPuts("   2. Set FTP Parameters       -[B]   |   7. Get FTP           -[K] \r\n");
	UsbUartPuts("   3. Set UART Parameters      -[C]   |   8. Get UART          -[L] \r\n");
	UsbUartPuts("   4. Set WLAN Parameters      -[D]   |   9. Get WLAN          -[M] \r\n");
	UsbUartPuts("   5. Set Sensor Parameters    -[E]   |  10. Get Sensor        -[N] \r\n");
	UsbUartPuts("--------------------------------------------------------------------\r\n");
	UsbUartPuts("   11. Status Command          -[O]         \r\n");
	UsbUartPuts("   12. Scan Command            -[P]         \r\n");
	UsbUartPuts("   13. Join Command            -[Q]         \r\n");
	UsbUartPuts("   14. Time Command            -[R]         \r\n");
	UsbUartPuts("   15. Save Command            -[S]         \r\n");
	UsbUartPuts("   16. Close Command           -[X]         \r\n");
	UsbUartPuts("   17. Command Prompt          -[Z]         \r\n");
	UsbUartPuts("====================================================================\r\n");
	
}

int main()
{	
	U8 tmp[100];
	USB_RING_BUF* pubuf;
	U8 rdat;
	
	pubuf = &usbbuf;
	
	InitRingBuf_u(pubuf);
	
	CANfly_init();
	
	*(volatile U16*)0x80000408 = 0xFFF0;//BANK2 8Bit
	*R_PAF0 = 0xAAAA; // SRAM interface address, data 
	*R_PAF1 = 0x00FF; // P1.0, P1.1, P1.2, P1.3, UART6, UART7
	*R_PAF2 = 0xEFAF; // nCS2, nWE, nRE, P2.7, P2.5, P2.4, P2.1, P2.0
	*R_PAF3 = 0xFFF3; // EIRQ1, P3.0, P3.2~7
	*R_PAF4 = 0xAFFF; // TWI, P4.0~5
	// *R_PAF5 = 0xBFFF; // A18, P5.0~6
	*R_PAF5 = 0xFFFF; // P5.0~7
	*R_PAF6 = 0x0300;//PIO6.4
	
	*R_P5oDIR = (1<<7); //P5.7 Output
	*R_P5oHIGH = (1<<7); // LED2 OFF
	
	*R_P6oDIR = (1<<4);//PIO6.4 output
	*R_P6oLOW = (1<<4);//PIO6.4 high , off
	
	*R_P1oDIR = 0x3; //PIO1.0,1.1 output // for LCD_nReset(PIO1.0) and LCD BackLight(PIO1.1)
	*R_P1oLOW= (1<<1);// LCD BackLight OFF

	// *R_P1oHIGH = 1; // LCD_nRESET High
	// delayms(1);

	*R_P1oLOW = (1<<3);// RN-171 Reset
	
	InitInterrupt();
	UartConfig(7,115200,DATABITS_8,STOPBITS_1,UART_PARNONE);

	setinterrupt(INTNUM_USB,usb_isr);
	EnableInterrupt(INTNUM_USB,TRUE);

	//USB On, not USB ctrl
	*R_P6oHIGH = (1<<4);//PIO6.4 high , on
	delayms(10);
	
	usb_init();
	
	delayms(1000);
	delayms(1000);
	
	*R_PAF5 |= (1<<15);
	*R_PAF5 &= ~(1<<14); // A18

	delayms(1000);
	WiFly_UartBaudSetting();
	
	*R_P1oLOW = 1; // LCD_nRESET Low
	delayms(500);
	*R_P1oHIGH = 1; // LCD_nRESET High
	
	LCDInit();
	
	LCDcls(0,0,0);	// clear white
	delayms(500);
	
//	LCDcls(255,255,0);	// back ground yellow
	LCDcls(0,0,0);	// back ground block
	
	SetFontColor(0, 0, 255, 135, 255, 60);
	SetFontSize(Font16, Font16);

	xp=0;
	yp=0;
	
	*R_P1oHIGH = (1<<3);// RN-171 Reset Release
	delayms(1000);
	if(GetAP_ConnectStatus(tmp)) {
		UsbUartPuts(" ==> Connected to AP(SSID - ");
		UsbUartPuts(tmp);
		UsbUartPuts(")\r\n\r\n");
	}
	else {
		UsbUartPuts(" ==> Not connected to AP\r\n\r\n");
	}
	
	while(1)
	{
		PrintCommandWiFly();
		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				if(cmd_in) {
					UsbUartPuts("Already came into CMD mode!\r\n");
					break;
				}
				EnterCmdMode();
				break;
			
			case 'b' : case 'B' :
				SetFtpParameters();
				break;
				
			case 'c' : case 'C' :
				SetUartParameters();
				break;

			case 'd' : case 'D' :
				SetWlanParameters();
				break;

			case 'e' : case 'E' :
				SetSensorParameters();
				break;
				
			case 'j' : case 'J' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get everything\r");
				UsbUartPuts("\r\n\t => [get everything] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;
				
			case 'k' : case 'K' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get ftp\r");
				UsbUartPuts("\r\n\t => [get ftp] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;

			case 'l' : case 'L' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get uart\r");
				UsbUartPuts("\r\n\t => [get uart] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;

			case 'm' : case 'M' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get wlan\r");
				UsbUartPuts("\r\n\t => [get wlan] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;

			case 'n' : case 'N' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get q\r");
				UsbUartPuts("\r\n\t => [get q] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);
				break;

			case 'o' : case 'O' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				Status_Command();
				break;

			case 'p' : case 'P' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				Scan_APs();
				break;

			case 'q' : case 'Q' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				Join_AP();
				break;

			case 'r' : case 'R' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "time\r");
				UsbUartPuts("\r\n\t => [time] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);
				break;

			case 's' : case 'S' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "save\r");
				UsbUartPuts("\r\n\t => [save] Command!\r\n");
				UsbUartPuts("\r\n\t => OK!\r\n");
				UartRxBufFlush(UART_CH6);
				break;

			case 'x' : case 'X' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "close\r");
				UsbUartPuts("\r\n\t => [close] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				break;

			case 'z' : case 'Z' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				while(1) {
					if(UsbGetCh(&rdat)) {
						if(rdat == 0x03) break;
						UartPutCh(UART_CH6, rdat);
					}
					if(UartGetCh(UART_CH6, &rdat)) {
						UsbUartPutCh(rdat);
					}
				}
				break;
			
			default :
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}
		
	}
	return 0;
}
