
#include "cantus.h"
#include "driver/lcd_g2.h"
#include "driver/lcdutil_I2810.h"
#include "driver/font_ctrl.h"

#define UART_CH6	6

void CANfly_init()
{
	*R_PAF0 = 0xFFFF;  // GPIO 
	*R_PAF1 = 0x0FFF;  // UART7, GPIO
	*R_PAF2 = 0xFFFF;  // GPIO
	*R_PAF3 = 0xFFFF;  // GPIO
	*R_PAF4 = 0x0FFF;  // UART3, GPIO
	*R_PAF5 = 0xFFFF;  // GPIO
	*R_PAF6 = 0x030C;  // PIO6.1 -> USB-Communication mode, PIO6.4 -> USB Ctrl
	*R_P6oDIR |= (1<<4); // PIO6.4 output mode
}



#define USB_BUF_SIZE 512

typedef struct _USB_RING_BUF {
	U16 uRingBufRxCtr;     // Number of characters in the Rx ring buffer
	U8  *uRingBufRxInPtr;  // Pointer to where next character will be inserted
	U8  *uRingBufRxOutPtr; // Pointer from where next character will be extracted
	U8  uRingBufRx[USB_BUF_SIZE]; // Ring buffer character storage (Rx)
} USB_RING_BUF;

USB_RING_BUF usbbuf;

void InitRingBuf_u(USB_RING_BUF* pRing)
{
	pRing->uRingBufRxCtr    = 0;
	pRing->uRingBufRxInPtr  = &pRing->uRingBufRx[0];
	pRing->uRingBufRxOutPtr = &pRing->uRingBufRx[0];
}


#define R_USBLBOWC (volatile U32*)(0x80022c00 + 0x40)
#define R_USBHBOWC (volatile U32*)(0x80022c00 + 0x44)

U32 total_in_data_len;

void usb_isr(void)
{
	BYTE usb_data[64];
	volatile U32 len;
	volatile U32 k=0;
	USB_RING_BUF* pbuf = &usbbuf;
	BYTE ep_irq;
    BYTE usb_irq;
	
	ep_irq  = *(volatile U8*)(__USBEI_REG);	
	usb_irq = *(volatile U8*)(__USBUI_REG);
		
		
	if(ep_irq & 1)//EP0_IRQ
	{
		ep0_isr();
	}
	if (ep_irq & (1<<1))//EP1_IRQ
	{
		k=0;
		len = read_usb(usb_data, len);
		while(len) {
			if(pbuf->uRingBufRxCtr<USB_BUF_SIZE) {
				pbuf->uRingBufRxCtr++;
				*pbuf->uRingBufRxInPtr=usb_data[k++];
				pbuf->uRingBufRxInPtr++;
				if(pbuf->uRingBufRxInPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
					pbuf->uRingBufRxInPtr = &pbuf->uRingBufRx[0];
				}
				
			}
			len--;
		}
	}
	
	if(usb_irq & USB_RESET_IRQ)
	{
		usb_reset();
	}
	else if ( usb_irq & USB_RESUME_IRQ )
	{
		usb_resume();
	}
	else if ( usb_irq & USB_SUSPEND_IRQ )
	{
		usb_suspend();
	}
	*(volatile U8*)__USBEI_REG = ep_irq;   // Clear Interrupt 
	*(volatile U8*)__USBUI_REG = usb_irq;  // Clear Interrupt

}

int UsbGetCh(U8* buf)
{
	int i=0;

	USB_RING_BUF* pbuf = &usbbuf;


	if (pbuf->uRingBufRxCtr > 0) { /* No, decrement character count */
		pbuf->uRingBufRxCtr--; /* No, decrement character count */
		buf[0] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
		/* Wrap OUT pointer     */
		if (pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
			pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
		}
		i=1;
	}

	return i;
}

U8 UsbGetC()
{
	U8 ch;
	
	while(1) {
		if(UsbGetCh(&ch)) break;
	}
	return ch;
}

int UsbGetData(U8* buf, int bufmax)
{
	int i=0;

	USB_RING_BUF* pbuf = &usbbuf;

	// CRITICAL_ENTER();
	for(i=0;i<bufmax;i++) {
		if (pbuf->uRingBufRxCtr > 0) { /* No, decrement character count */
			pbuf->uRingBufRxCtr--; /* No, decrement character count */
			buf[i] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
			/* Wrap OUT pointer     */
			if (pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
				pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
			}
		}
		else {
			break;
		}	
	}
	// CRITICAL_EXIT();

	return i;
}

extern U8 Font_X_siz;
extern U8 Font_Y_siz;
U32 xp=0,yp=0;
U8 before_ch=0;
void FontDisplayLCD(char *fData, int siz)
{
	U8 ch;
	U16  txt;
	int pp;
	int i;
	U8 *str = fData;

	if(before_ch) {
		txt = (before_ch<<8) + (*str & 0xFF);
		str++;

		if(xp >= 310) {
			xp = 0;
			yp += Font_Y_siz;
//			LCDdrawbox(0,yp,320,yp+Font_Y_siz,0,0,0);
			// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
		}
		if(yp >= 240) {
			yp = 0;
			// LCDdrawbox(0,yp,320,yp+32,255,255,0);
			// LCDcls(255,255,0);
		}
		pp = DisplayFont(xp, yp, txt);
		xp += pp;
	}

	for(i=0; i<siz; i++) {
		ch = *str;
		str++;
		if(ch < 0x80) {
			txt = ch;
			if(txt == 0x0d) {
				xp = 0;
				yp += Font_Y_siz;
//				LCDdrawbox(0,yp,320,yp+Font_Y_siz,0,0,0);
				// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
			
				if(yp >= 240) {
					yp = 0;
					// LCDdrawbox(0,yp,320,yp+32,255,255,0);
					// LCDcls(255,255,0);
				}
				continue;
			}
			else if(txt == 0x0a) {
				continue;
			}
		}
		else {
			if(i == (siz-1)) {
				before_ch = ch;
				return;
			}
			else before_ch=0;
			txt = (ch<<8) + (*str & 0xFF);
			str++;
			i++;
		}
		
		if(xp >= 310) {
			xp = 0;
			yp += Font_Y_siz;
//			LCDdrawbox(0,yp,320,yp+Font_Y_siz,0,0,0);
			// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
		}
		if(yp >= 240) {
			yp = 0;
			// LCDdrawbox(0,yp,320,yp+32,255,255,0);
			// LCDcls(255,255,0);
		}
		pp = DisplayFont(xp, yp, txt);
		xp += pp;		
	}
}

void PrintLCD(U8 *data)
{
	int siz = strlen(data);
	int len=0;

	while(siz) {
		if(siz>64) len = 64;
		else len = siz;
		FontDisplayLCD(data, len);
		data += len;
		siz -= len;
	}
	
}

void UsbUartPutCh(U8 data)
{
	write_usb(&data, 1);	
}

void UsbUartPuts(U8 *data)
{
	int siz = strlen(data);
	int len=0;

	while(siz>0) {
		if(siz>64) len = 64;
		else len = siz;
		write_usb(data, len);
		data += len;
		siz -= len;
	}
	
}

void UartRxBufFlush(int ch)
{
	int ret;
	U8 tmp;
	do {
		ret = UartGetCh(ch, &tmp);
	} while(ret);
}

U8 UartGetC(int channel)
{
	int count=0xffffff;
	U8 tmp;

	while(!UartGetCh(channel, &tmp)) {
		count--;
		if(count<=0) {
			return 0;
		}
	}

	return tmp;
}

U8 UartGetCC(int channel, int *err_no)
{
	int count=0xffffff;
	U8 tmp;
	err_no[0] = 0;

	while(!UartGetCh(channel, &tmp)) {
		count--;
		if(count<=0) {
			err_no[0] = 1;
			return 0;
		}
	}

	return tmp;
	
}

int cmd_in = 0;

int EnterCmdMode(void)
{
	int i=0, j;
	U8 rdat;
	U8 temp[65];
	int cnt=0xfffff;

	UsbUartPuts("\r\n\t => Enter WiFly Command Mode!\r\n");
	for(j=0; j<5; j++) {
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "$$$");
		delayms(500);
		while(cnt>0) {
			if(UartGetCh(UART_CH6, &rdat)) {
				UartPutCh(7, rdat);
				if(rdat == 'C') {
					temp[i]=rdat;
					i++;
					goto Nnext;
				}
				else if(rdat == '$') {
					UsbUartPuts("Already came into CMD mode!\r\n");
					cmd_in = 1;
					return 1;
				}
			}
			cnt--;		
		}
		cnt = 0xfffff;
		UsbUartPuts("\r\n\t => reEnter WiFly Command Mode!\r\n");
	}
	if(j==5) {
		UsbUartPuts("\t => Fail!\r\n");
		cmd_in = 0;
		return 0;
	}

Nnext:
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0d) break;
			temp[i]=rdat;
			i++;
		}
	}

	// UartPuts(7, usb_data_out);
	if(strncmp(temp, "CMD", 3) == 0) {
		UsbUartPuts("\t => OK!\r\n");
		cmd_in = 1;
	}
	else {
		UsbUartPuts("\t => Fail!\r\n");
		cmd_in = 0;
	}

	UartRxBufFlush(UART_CH6);
	
	return cmd_in;
}

int GetAP_ConnectStatus(U8 *ssid)
{
	int i=0;
	char rdat;
	int ret;
	U8 tmp[256];
	
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) {
			UsbUartPuts("Fail!, Enter CMD Mode!\r\n");
			return 0;
		}
	}
	delayms(3000);
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show connection\r");

	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7,rdat);
			if(rdat == 0x0a) break;
		}
	}
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7,rdat);
			if(rdat == 0x0d) {
				tmp[i] = 0;
				break;
			}
			else tmp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	if(tmp[2]=='3' && tmp[3]!='3') ret=1;
	else ret=0;

	i=0;
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show net\r");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0a) break;
		}
	}
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0d) {
				tmp[i] = 0;
				break;
			}
			else tmp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	strcpy(ssid, &tmp[5]);	
	
	return ret;
}

#define CFG_CBSIZE 75
U8 console_buffer[CFG_CBSIZE];

static char * delete_char(char *buffer, char *p, int *colp, int *np, int plen)
{
	char *s;

	if(*np == 0) {
		return (p);
	}

	if(*(--p) == '\t') {			/* will retype the whole line	*/
		while(*colp > plen) {
			UsbUartPuts("\b \b");
			(*colp)--;
		}
		for(s=buffer; s<p; ++s) {
			if (*s == '\t') {
				UsbUartPuts("        "+((*colp) & 07));
				*colp += 8 - ((*colp) & 07);
			} else {
				++(*colp);
				UsbUartPutCh(*s);
			}
		}
	} else {
		UsbUartPuts("\b \b");
		(*colp)--;
	}
	(*np)--;
	return (p);
}


int readline(U8 *prompt)
{
	char *p = console_buffer;
	int	n = 0;				/* buffer index		*/
	int	plen = 0;			/* prompt length	*/
	int	col;				/* output column cnt	*/
	char c;

	/* print prompt */
	if(prompt) {
		plen = strlen(prompt);
		UsbUartPuts(prompt);
	}
	col = plen;

	for (;;) {
		c = UsbGetC();

		/*
		 * Special character handling
		 */
		switch (c) {
			case '\r':				/* Enter		*/
			case '\n':
				*p = '\0';
				UsbUartPuts("\r\n");
				return ((int)p - (int)console_buffer);

			case '\0':				/* nul			*/
				continue;

			case 0x03:				/* ^C - break		*/
				console_buffer[0] = '\0';	/* discard input */
				return (-1);

			case 0x15:				/* ^U - erase line	*/
				while (col > plen) {
					UsbUartPuts("\b \b");
					--col;
				}
				p = console_buffer;
				n = 0;
				continue;

			case 0x17:				/* ^W - erase word 	*/
				p=delete_char(console_buffer, p, &col, &n, plen);
				while ((n > 0) && (*p != ' ')) {
					p=delete_char(console_buffer, p, &col, &n, plen);
				}
				continue;

			case 0x08:				/* ^H  - backspace	*/
			case 0x7F:				/* DEL - backspace	*/
				p=delete_char(console_buffer, p, &col, &n, plen);
				continue;

			default:
				/*
				 * Must be a normal character then
				 */
				if (n < CFG_CBSIZE-2) {
					if(c == '\t') {	/* expand TABs		*/
						UsbUartPuts("        "+(col&07));
						col += 8 - (col&07);
					} else {
						++col;		/* echo input		*/
						UsbUartPutCh(c);
					}
					*p++ = c;
					++n;
				} else {			/* Buffer full		*/
					UsbUartPutCh('\a');
				}
		}
	}
}
/****************************************************************************/

 typedef struct tag_BITMAPFILEHEADER {
	U32 bfSize;	
	U32 bfReserved; 	
	U32 bfOffBits; 	
	U32 biSize; 	
	S32 biWidth; 	
	S32 biHeight; 	
	U16 biPlanes; 	
	U16 biBitCount;	
	U32 biCompression; 	
	U32 biSizeImage; 	
	S32 biXPelsPerMeter; 	
	S32 biYPelsPerMeter; 	
	U32 biClrUsed; 	
	U32 biClrImportant; 
 }BITMAPFILEHEADER;   

typedef struct tag_RGBQUAD {	
	U8 rgbBlue;	
	U8 rgbGreen;     
	U8 rgbRed;	
	U8 rgbReserved;
} RGBQUAD; 


extern U8 weather_cloudy[];
extern U8 weather_rain[];
extern U8 weather_chance_of_storm[];
extern U8 weather_dust[];
extern U8 weather_fog[];
extern U8 weather_mostly_cloudy[];
extern U8 weather_mostly_sunny[];
extern U8 weather_partly_cloudy[];
extern U8 weather_snow[];
extern U8 weather_sunny[];
extern U8 weather_thunderstorm[];

void run_weather_demo()
{
	float kmh;
	int i,k;
	int err;
	U8 rdat;
	U8 temp[100];

	U8 tmp_city[10];
	U8 tmp_time[50];
	U8 tmp_cond[30];
	U8 tmp_temp[4];
	U8 tmp_humi[4];
	U8 tmp_icon[50];
	U8 tmp_wind[30];
	U8 tmp_uint[5];
	U8 tmp_week[5];
	U8 week[7][3] = {"", "", "ȭ", "", "", "", ""};

	RGBQUAD color_palette[64];
	ALIGN4 BITMAPFILEHEADER bmpfh;	
	ALIGN4 U8 linebuf[320*3];
	U8 *file_buf;
	U32 w,h;	
	int end_x, end_y;
	int x=0, y=0;
	U32 ibpl; //line pitch
	U16 tmp;
	volatile U8* frame=(volatile U8*)(LCD_BASE_ADDR|LCD_SEL_CTL_REG);

	U8 *icon_data=NULL;
	char *pt=NULL;
	int year;
	int month;
	int date;
	int hour;
	int min;
	int sec;

	U8 tp[3];
	int r_hour;
	int r_min;
	int r_min_b;
	int first_chk=0;

	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show connection\r");

	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7,rdat);
			if(rdat == 0x0a) break;
		}
	}
	i=0;
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7,rdat);
			if(rdat == 0x0d) {
				temp[i] = 0;
				break;
			}
			else temp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	if(temp[2]!='3' || temp[3]=='3') {
		UsbUartPuts("Not associated with AP!\r\n");
		UsbUartPuts("First of all, Join AP!\r\n");
		return;
	}
	
	
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "time\r");
	while(1) {
		if((rdat=UartGetCC(UART_CH6, &err)) == 0x0a) {
			UartPutCh(7, rdat);
			break;
		}
		UartPutCh(7, rdat);
	}
	delayms(2000);
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show t\r");
	while(1) {
		if((rdat=UartGetCC(UART_CH6, &err)) == 0x0a) {
			UartPutCh(7, rdat);
			break;
		}
		UartPutCh(7, rdat);
	}
	i=0;
	while(1) {
		rdat=UartGetCC(UART_CH6, &err);
		temp[i]=rdat;
		UartPutCh(7, rdat);
		if(rdat == 0x0a) {
			memcpy(tp, &temp[5], 2);
			tp[2]=0;
			r_hour = strtoul(tp, &pt, 10);
			debugprintf("r_hour=%d\r\n", r_hour);
			
			memcpy(tp, &temp[8], 2);
			tp[2]=0;
			r_min = strtoul(tp, &pt, 10);
			debugprintf("r_min=%d\r\n", r_min);
			break;
		}
		i++;
	}
	UartRxBufFlush(UART_CH6);

	SetFontSize(16, 16);
	xp=205;
	yp=10;

	sprintf(temp, "(%02d:%02d)", r_hour, r_min);
	PrintLCD(temp);

	r_min_b = -1;

	while(1) {
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "show t\r");
		while(1) {
			if((rdat=UartGetCC(UART_CH6, &err)) == 0x0a) {
				break;
			}
		}
		i=0;
		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			temp[i]=rdat;
			UartPutCh(7, rdat);
			if(rdat == 0x0a) {
				memcpy(tp, &temp[5], 2);
				tp[2]=0;
				r_hour = strtoul(tp, &pt, 10);
				
				
				memcpy(tp, &temp[8], 2);
				tp[2]=0;
				r_min = strtoul(tp, &pt, 10);
				break;
			}
			i++;
		}
		UartRxBufFlush(UART_CH6);

		if(r_min == r_min_b) {
			delayms(1000);
			continue;
		}

		r_min_b=r_min;

		SetFontSize(24, 24);
		xp=205;
		yp=10;

		sprintf(temp, "(%02d:%02d)", r_hour, r_min);
		PrintLCD(temp);

		if((r_min%5)  && first_chk != 0) continue;
		first_chk = 1;

		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "open\r");
		while(1) {
			if((rdat=UartGetCC(UART_CH6, &err)) == 0x0a) {
				UartPutCh(7, rdat);
				break;
			}
			UartPutCh(7, rdat);
		}

reParsing1:
		rdat=UartGetCC(UART_CH6, &err);
		UartPutCh(7, rdat);
		while(1) {
			if(rdat == '*') {
				if((rdat=UartGetCC(UART_CH6, &err)) == 'O') {
					UartPutCh(7, rdat);
					if((rdat=UartGetCC(UART_CH6, &err)) == 'P') {
						UartPutCh(7, rdat);
						if((rdat=UartGetCC(UART_CH6, &err)) == 'E') {
							UartPutCh(7, rdat);
							if((rdat=UartGetCC(UART_CH6, &err)) == 'N') {
								UartPutCh(7, rdat);
								if((rdat=UartGetCC(UART_CH6, &err)) == '*') {
									UartPutCh(7, rdat);
									break;
								}
							}
						}
					}
				}
			} else goto reParsing1;
		}
				

		debugstring("\r\nRX START!!!\r\n");
		delayms(500);
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "\r");
		i=0;
		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			temp[i]=rdat;
			UartPutCh(7, rdat);
			if(rdat == 0x0a || rdat == '>') {	
				if(strncmp(temp, "Date: ", 6)==0) {
					memcpy(tmp_week, &temp[6], 3);
					tmp_week[3]=0;
				}
				else if(strncmp(temp, "<postal_code data=\"", 19)==0) {
					memcpy(tmp_city, &temp[19], i-21);
					tmp_city[i-21]=0;
				}
				else if(strncmp(temp, "<current_date_time data=\"", 25)==0) {
					memcpy(tmp_time, &temp[25], i-27);
					tmp_time[i-27]=0;
				}
				else if(strncmp(temp, "<condition data=\"", 17)==0) {
					memcpy(tmp_cond, &temp[17], i-19);
					tmp_cond[i-19]=0;
				}
				else if(strncmp(temp, "<temp_c data=\"", 14)==0) {
					memcpy(tmp_temp, &temp[14], i-16);
					tmp_temp[i-16]=0;
				}
				else if(strncmp(temp, "<humidity data=\"Humidity: ", 26)==0) {
					memcpy(tmp_humi, &temp[26], i-28);
					tmp_humi[i-28]=0;
				}
				else if(strncmp(temp, "<icon data=\"", 12)==0) {
					memcpy(tmp_icon, &temp[12], i-14);
					tmp_icon[i-14]=0;
				}
				else if(strncmp(temp, "<wind_condition data=\"Wind: ", 28)==0) {
					memcpy(tmp_wind, &temp[28], i-30);
					tmp_wind[i-30]=0;
				}
				else if(strncmp(temp, "<unit_system data=\"", 19)==0) {
					memcpy(tmp_uint, &temp[19], 4);
					tmp_uint[4]=0;
				}
				else if(strncmp(temp, "</current_conditions", 20)==0) {
					break;
				}
				i=0;
			}
			else if(rdat == '*') {
				if((rdat=UartGetCC(UART_CH6, &err)) == 'C') {
					temp[i++] = rdat;
					UartPutCh(7, rdat);
					if((rdat=UartGetCC(UART_CH6, &err)) == 'L') {
						temp[i++] = rdat;
						UartPutCh(7, rdat);
						if((rdat=UartGetCC(UART_CH6, &err)) == 'O') {
							temp[i++] = rdat;
							UartPutCh(7, rdat);
							if((rdat=UartGetCC(UART_CH6, &err)) == 'S') {
								temp[i++] = rdat;
								UartPutCh(7, rdat);
								if((rdat=UartGetCC(UART_CH6, &err)) == '*') {
									UartPutCh(7, rdat);
									goto NextProcess2;
								}
							}
						}
					}
				}
			}
			else {
				i++;
			}
		}

		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			UartPutCh(7, rdat);
			if(rdat == '*') {
				if((rdat=UartGetCC(UART_CH6, &err)) == 'C') {
					UartPutCh(7, rdat);
					if((rdat=UartGetCC(UART_CH6, &err)) == 'L') {
						UartPutCh(7, rdat);
						if((rdat=UartGetCC(UART_CH6, &err)) == 'O') {
							UartPutCh(7, rdat);
							if((rdat=UartGetCC(UART_CH6, &err)) == 'S') {
								UartPutCh(7, rdat);
								if((rdat=UartGetCC(UART_CH6, &err)) == '*') {
									UartPutCh(7, rdat);
									goto NextProcess2;
								}
							}
						}
					}
				}
			}
		}
		
NextProcess2:

		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "$$$");
		delayms(500);
		UartRxBufFlush(UART_CH6);

		i=strlen(tmp_time);
		k = strtoul(&tmp_time[i-4], &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}
		k = 9-k;

		memcpy(temp, &tmp_time[i-8], 2); temp[2]=0;
		sec = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}

		memcpy(temp, &tmp_time[i-11], 2); temp[2]=0;
		min = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}

		memcpy(temp, &tmp_time[i-14], 2); temp[2]=0;
		hour = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}

		memcpy(temp, &tmp_time[i-17], 2); temp[2]=0;
		date = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}

		memcpy(temp, &tmp_time[i-17], 2); temp[2]=0;
		date = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}
		memcpy(temp, &tmp_time[i-20], 2); temp[2]=0;
		month = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}
		memcpy(temp, &tmp_time[i-25], 4); temp[4]=0;
		year = strtoul(temp, &pt, 10);
		if(*pt!=0) {
			debugstring("Invalid Number!\r\n");
		}

		x=0;
		i=hour+k;
		if(i>=24) {
			hour = i-24;
			date += 1;
			x=1;
		}
		else hour = i;
		
		if(date>30) {
			if(month==4 || month==6 || month==9 || month==6 || month==11) {
				month += 1;
				date = 1;
			}			
		}
		else if(date>28) {
			if(month==2 && year==2012) {
				month = 3;
				date = 1;
			}
		}
		if(strncmp(tmp_week, "Sun", 3)==0) y=0;
		else if(strncmp(tmp_week, "Mon", 3)==0) y=1;
		else if(strncmp(tmp_week, "Tue", 3)==0) y=2;
		else if(strncmp(tmp_week, "Wed", 3)==0) y=3;
		else if(strncmp(tmp_week, "Thu", 3)==0) y=4;
		else if(strncmp(tmp_week, "Fri", 3)==0) y=5;
		else if(strncmp(tmp_week, "Sat", 3)==0) y=6;
		
		pt = week[y];
		if(x) {
			if(y+1 == 7) y=0;
			else y += 1;
			pt = week[y];
		}


		SetFontSize(24, 24);
		SetFontColor(0, 0, 255, 135, 255, 60);
		xp=10;
		yp=10;
		debugprintf("%04d %02d %02d(%s) %02d %02d %02d \r\n", year, month, date, pt, hour, min, sec);
		sprintf(temp, "%d-%d-%d[%s]", year, month, date, pt);
		PrintLCD(temp);
		
		xp=10; yp=40;
		SetFontSize(24, 24);
		if(strncmp(tmp_city, "seoul", 5)==0) {
			PrintLCD(":  ");
		}

		xp=10; yp=70;
		SetFontSize(16, 16);
		sprintf(temp, "Update: %02d %02d %02d ", hour, min, sec);
		PrintLCD(temp);
		
		LCDdrawbox(10,96,320,112,0,0,0);
		SetFontColor(0, 0, 255, 135, 255, 60);
		xp=10; yp=96;

		if(strncmp(tmp_cond, "Cloudy", 6)==0) {
			PrintLCD(": 帲");
			icon_data = weather_cloudy;
		}
		else if(strncmp(tmp_cond, "Showers", 6)==0) {
			PrintLCD(": ҳ");
			icon_data = weather_rain;			
		}
		else if(strncmp(tmp_cond, "Rain", 6)==0) {
			PrintLCD(": ");
			icon_data = weather_rain;			
		}
		else if(strncmp(tmp_cond, "Mostly Cloudy", 6)==0) {
			PrintLCD(": ü 帲");
			icon_data = weather_mostly_cloudy;			
		}
		else if(strncmp(tmp_cond, "Partly Cloudy", 6)==0) {
			PrintLCD(": κ 帲");
			icon_data = weather_partly_cloudy;			
		}
		else if(strncmp(tmp_cond, "Scattered Thunderstorms", 6)==0) {
			PrintLCD(":  ");
			icon_data = weather_chance_of_storm;			
		}
		else if(strncmp(tmp_cond, "Isolated Thunderstorms", 6)==0) {
			PrintLCD(":  ");
			icon_data = weather_chance_of_storm;			
		}
		else if(strncmp(tmp_cond, "Sunny", 6)==0) {
			PrintLCD(": ");
			icon_data = weather_sunny;			
		}
		
		LCDdrawbox(120,126,320,190,0,0,0);
		sprintf(temp, "µ: %s", tmp_temp);
		xp=120;
		yp=126;
		PrintLCD(temp);
		sprintf(temp, ": %s ", tmp_humi);
		xp=120;
		yp=142;
		PrintLCD(temp);

		if(tmp_wind[0] == 'E') strcpy(temp, "");
		else if(tmp_wind[0] == 'W') strcpy(temp, "");
		else if(tmp_wind[0] == 'S') strcpy(temp, "");
		else if(tmp_wind[0] == 'N') strcpy(temp, "");
		if(tmp_wind[1] != ' ') {
			if(tmp_wind[1] == 'E') strcpy(&temp[2], "");
			else if(tmp_wind[1] == 'W') strcpy(&temp[2], "");
			else if(tmp_wind[1] == 'S') strcpy(&temp[2], "");
			else if(tmp_wind[1] == 'N') strcpy(&temp[2], "");
		}

		i=strlen(temp);
		strcpy(&temp[i], "ǳ");
		k = i/2;
		k += 4;
		
		pt = NULL;
		i = strtoul(&tmp_wind[k], &pt, 10);
		if(strncmp(pt, " mph", 4)==0) {
			kmh = i*1.6;
		}
		else kmh = i;

		sprintf(linebuf, "ǳ: %s", temp);
		xp=120;
		yp=158;
		PrintLCD(linebuf);
		
		sprintf(linebuf, "ǳ: %.2f km/h", kmh);
		xp=120;
		yp=174;
		PrintLCD(linebuf);
		
		xp=10;
		yp=112;
		file_buf = (U8 *)&bmpfh;
		memcpy(file_buf, &icon_data[2], sizeof(BITMAPFILEHEADER));
					
		if( bmpfh.biBitCount == 4 )	{
			debugstring("Not support 4bit bmp\r\n");
		}
		w = bmpfh.biWidth;	
		h = bmpfh.biHeight;

		end_x = w>320?320:w;	
		end_y = h>240?240:h;

//		LCDdrawbox(xp,yp,320,yp+end_y,0,0,0);

		if(bmpfh.biBitCount==8) {
			memcpy((U8 *)color_palette, &icon_data[sizeof(BITMAPFILEHEADER)+2], 1024);

			for(i=0;i<256;i++) {	
				color_palette[i].rgbRed=((color_palette[i].rgbRed&0xf8)|(color_palette[i].rgbGreen&0xfc)>>5);
				color_palette[i].rgbGreen=(((color_palette[i].rgbGreen&0xfc)>>2&0x07)<<5|(color_palette[i].rgbBlue&0xf8)>>3);
			}

			ibpl = ((w*1) + 3) & 0xfffffffc;		

			if(ibpl>320*3) {
				debugstring("\t => Image width is too big!\r\n");
			}
			
			for(k=1,y=end_y;y>0;y--) {
				memcpy(linebuf, &icon_data[bmpfh.bfOffBits+(y-1)*ibpl], ibpl);

				for(i=0,x=0;x<end_x;x++) 	{
					LCDRamSet(240-(k+yp),x+xp,240-(k+yp),x+xp);
					*frame=(U8)color_palette[linebuf[i]].rgbRed;
					*frame=(U8)color_palette[linebuf[i]].rgbGreen;
					i++;
				}
				k++;
			}
		}
		else if(bmpfh.biBitCount==16) {
			ibpl = ((w*2) + 3) & 0xfffffffc;
			if(ibpl>320*3) {
				debugstring("Image width is too big!\r\n");
			}

			if(bmpfh.biCompression==0) {	// 555
				for(k=1,y=end_y;y>0;y--) {
					memcpy(linebuf, &icon_data[bmpfh.bfOffBits+(y-1)*ibpl], ibpl);
					for(i=0,x=0;x<end_x;x++) 	{
						LCDRamSet(240-(k+yp),x+xp,240-(k+yp),x+xp);
						tmp=(linebuf[i+1]<<8|linebuf[i])<<1;
						
						*frame=tmp>>8;
						*frame=(linebuf[i]&0x1f)|(tmp&0xc0);

						i+=2;
					}
					k++;
				}
			}
			else {
				for(k=1,y=end_y;y>0;y--) {
					memcpy(linebuf, &icon_data[bmpfh.bfOffBits+(y-1)*ibpl], ibpl);
					for(i=0,x=0;x<end_x;x++) 	{
						LCDRamSet(240-(k+yp),x+xp,240-(k+yp),x+xp);
						*frame = linebuf[i+1];
						*frame = linebuf[i];
						i+=2;
					}
					k++;
				}
			}
		}
		else if(bmpfh.biBitCount==24) {
			ibpl = ((w*3) + 3) & 0xfffffffc;
			if(ibpl>320*3) {
				debugstring("Image width is too big!\r\n");
			}

			for(k=1,y=end_y;y>0;y--) {
				memcpy(linebuf, &icon_data[bmpfh.bfOffBits+(y-1)*ibpl], ibpl);
				for(i=0,x=0;x<end_x;x++) 	{
					LCDRamSet(240-(k+yp),x+xp,240-(k+yp),x+xp);
					*frame=(linebuf[i+2]&0xf8)|(linebuf[i+1]&0xfc)>>5;
					*frame=((linebuf[i+1]&0x1c)<<3) | linebuf[i]>>3;
					i+=3;
				}
				k++;
			}
		}

	}
}


int String_UartBaud(char num)
{
	int ret = -1;
	
	switch(num) {
		case '0' : ret=2400; break;
		case '1' : ret=4800; break;
		case '2' : ret=9600; break;
		case '3' : ret=19200; break;
		case '4' : ret=38400; break;
		case '5' : ret=57600; break;
		case '6' : ret=115200; break;
		case '7' : ret=230400; break;
		case '8' : ret=460800; break;
		case '9' : ret=921600; break;
		default : ret = -1; break;
	}
	
	return ret;
}

void WiFly_UartBaudSetting()
{
	int k;
	int len;
	U8 temp[75];
	
	while(1) {
		UsbUartPuts("\r\n");
		UsbUartPuts("         < Select WiFly(RN-171) Current UART BaudRate >\r\n");
		UsbUartPuts("\r\n");
		UsbUartPuts(" 0:2400, 1:4800, 2:9600, 3:19200, 4:38400, 5:57600, 6:115200\r\n");
		UsbUartPuts("\r\n");
		strcpy(temp, "Select Number - ");

		len = readline(temp);
				
		if(len<=0) {
			UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
		}
		else {
			if(strlen(console_buffer)>1) {
				UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
			}
			else {
				k = String_UartBaud(console_buffer[0]);
				if(k<0 || k>115200) {
					UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
				}
				else {
					UartConfig(UART_CH6,k,DATABITS_8,STOPBITS_1,UART_PARNONE);
					UsbUartPuts("\r\n ==> WiFly(RN-171)'s uart baudrate is set to ");
					sprintf(temp, "%d\r\n", k);
					UsbUartPuts(temp);
					break;
				}
			}
		}
	}
}


void SetTimeServerParameters()
{
	U8 rdat;
	U8 temp[50];
	int len;
	int i,j,k;
	int ret=0;
	char *pt;
	U8 tmp_buf[4][25];
	
	while(1)
	{
		UsbUartPuts("\r\n");
		UsbUartPuts("================================================\r\n");
		UsbUartPuts(" <<<< Sub Menu - Time Server Parameters >>>>     \r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("     01. Set Time Address           -[A]\r\n");
		UsbUartPuts("     02. Set Time Port              -[B]\r\n");
		UsbUartPuts("     03. Set Time Enable            -[C]\r\n");
		UsbUartPuts("     04. Set Time Zone              -[D]\r\n");
		UsbUartPuts("     05. Get Time                   -[E]\r\n");
		UsbUartPuts("------------------------------------------------\r\n");
		UsbUartPuts("     06. Show Time                  -[S]\r\n");
		UsbUartPuts("     07. Time Command               -[T]\r\n");
		UsbUartPuts("     08. Return Previous Menu       -[Z]\r\n");
		UsbUartPuts("================================================\r\n");

		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get time\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);

				strcpy(temp, "Set Time Server Addr[current sNTP server : ");
				j = 8;
				i=strlen(temp);
				while(1) {
					if(tmp_buf[1][j] ==':') break;
					temp[i] = tmp_buf[1][j];
					i++;
					j++;
				}
				temp[i++] = ']';
				temp[i++] = ' ';
				temp[i++] = '-';
				temp[i++] = ' ';
				temp[i] = 0;
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current address[");
					UsbUartPuts(&tmp_buf[1][8]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					sprintf(temp, "set time address %s\r", console_buffer);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Time Server Address)\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'b' : case 'B' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get time\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);

TimeJmp0:				
				UsbUartPuts("\r\n");
				j=0;
				while(1) {
					if(tmp_buf[1][j] == ':') break;
					j++;
				}
				j++;
				sprintf(temp, "Set Time Port Number[current : %s] - ", &tmp_buf[1][j]);
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current port number[");
					UsbUartPuts(&tmp_buf[1][j]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto TimeJmp0;
					}
					
					sprintf(temp, "set time port %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Time Port)\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			case 'c' : case 'C' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get time\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);

TimeJmp1:
				UsbUartPuts("\r\nTime Enable Setting\r\n");
				UsbUartPuts("    ==> 0 = disable\r\n");
				UsbUartPuts("    ==> 1 = get time only once on power up\r\n");
				UsbUartPuts("    ==> value>1 = get time continuously every <value> minutes\r\n");
				strcpy(temp, "Enter value for time enable[current : ");

				if(tmp_buf[0][8] == '0') {
					strcpy(temp, "Enter value for time enable[current : 0(disabled)] - ");
				}
				else {
					k=strtoul(&tmp_buf[0][8], &pt, 10);
					sprintf(temp, "Enter value for time enable[current : %d] - ", k);
				}
			
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current value[");
					UsbUartPuts(&tmp_buf[0][8]);
					UsbUartPuts("]\r\n");
				}
				else if(len<0) {
					break;
				}
				else {
					k = strtoul(console_buffer, &pt, 10);
					if(*pt!=0) {
						UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
						goto TimeJmp1;
					}
					
					sprintf(temp, "set time enable %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Time Enable)\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}
				}
				break;

			case 'd' : case 'D' :
				i=0; j=0;
				if(!cmd_in) {
					ret = EnterCmdMode();
					if(!ret) return;
				}
				UartPuts(UART_CH6, "get time\r");

				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat == 0x0a) break;
					}
				}
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						tmp_buf[j][i] = rdat;
						i++;
						if(rdat == 0x0a) {
							tmp_buf[j][i-2] = 0;
							j++;
							i=0;
						}							
					}
				}				
				UartRxBufFlush(UART_CH6);

TimeJmp2:	
				i=2;
				j=5;

				k=strtoul(&tmp_buf[i][j], &pt, 10);
				if(k==12 || k==0) {
					ret=0;
					sprintf(temp, "Set Time Zone [current : GMT 0] - ");
				}
				else if(k>12) {
					ret=24-k;
					sprintf(temp, "Set Time Zone [current : GMT +%d] - ", ret);
				}
				else if(k<12) {
					ret=k;
					sprintf(temp, "Set Time Zone [current : GMT -%d] - ", ret);
				}
				
				len = readline(temp);
				
				if(len==0) {
					UsbUartPuts("\t => Keep the current time zone[");
					if(k==12 || k==0) {
						sprintf(temp, "\t => Keep the current time zone[GMT 0]");
					}
					else if(k>12) {
						sprintf(temp, "Set Time Zone [current : GMT +%d]", ret);
					}
					else if(k<12) {
						sprintf(temp, "Set Time Zone [current : GMT -%d]", ret);
					}
					UsbUartPuts(temp);
				}
				else if(len<0) {
					break;
				}
				else {
					if(console_buffer[0] == '+') {
						k = strtoul(&console_buffer[1], &pt, 10);
						if(*pt!=0) {
							UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
							goto TimeJmp2;
						}
						if(k==12) k = 0;
						else if(k>0 && k<12) k=24-k;
						else {
							UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
							goto TimeJmp2;
						}
					}
					else if(console_buffer[0] == '-') {
						k = strtoul(&console_buffer[1], &pt, 10);
						if(*pt!=0) {
							UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
							goto TimeJmp2;
						}
						if(k==12) k = 0;
						else if(k>12) {
							UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
							goto TimeJmp2;
						}
					}
					else {
						k = strtoul(console_buffer, &pt, 10);
						if(*pt!=0) {
							UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
							goto TimeJmp2;
						}
						if(k==12) k = 0;
						else if(k>0 && k<12) k=24-k;
						else {
							UsbUartPuts("Invalid Value is entered! Please Re-enter!\r\n");
							goto TimeJmp2;
						}
					}	
					
					sprintf(temp, "set time zone %d\r", k);
					UartRxBufFlush(UART_CH6);
					UartPuts(UART_CH6, temp);
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
						}
					}
					i=0;
					while(1) {
						if(UartGetCh(UART_CH6, &rdat)) {
							if(rdat==0x0a) break;
							temp[i]=rdat;
							i++;
						}
					}
					UartRxBufFlush(UART_CH6);
					if(strncmp(temp, "AOK", 3)==0) {
						UsbUartPuts("\t => OK! (Set Time Zone)\r\n");
					}
					else {
						UsbUartPuts("\t => Fail!\r\n");
					}							
				}
				break;

			

			case 'e' : case 'E' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get time\r");
				UsbUartPuts("\r\n\t => [get time] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);
				break;

			case 's' : case 'S' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "show t t\r");
				UsbUartPuts("\r\n\t => [show time] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);			
				break;

			case 't' : case 'T' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "time\r");
				UsbUartPuts("\r\n\t => [time] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);
				break;

			case 'z' : case 'Z' :
				return;

			default :
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}
	}
}

U8 Scan_Aps[13][25];
U8 Aps_Num=0;
void Scan_APs()
{
	int ver;
	int i,j,k,m,n,p;
	int ret;
	U8 rdat;
	U8 tmp[200];
	U8 temp[30];
	char *pt=NULL;
	
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) return;
	}
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "get sys\r");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
		}
	}
	i=0;
	while(1) {
		rdat = UartGetC(UART_CH6);
		UartPutCh(7, rdat);
		tmp[i++] = rdat;
		if(rdat==0x0a) {			
			if(strncmp(tmp, "PrintLvl=", 9)==0) {
				k = strtoul(&tmp[9], &pt, 16);
				debugprintf("\r\nk=0x%x\r\n", k);
				break;
			}
			i=0;
		}
	}
	UartRxBufFlush(UART_CH6);

	if(k!=0x4000) {
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "set sys printlvl 0x4000\r");
		while(1) {
			if(UartGetCh(UART_CH6, &rdat)) {
				UartPutCh(7, rdat);
				if(rdat == 0x0a) break;
			}
		}
		UartRxBufFlush(UART_CH6);
	}



	debugprintf("\r\n________ scan start~\r\n");
	
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "scan\r");

	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
		}
	}
	i=0;
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
			tmp[i] = rdat;
			i++;
		}
	}
	tmp[5]=0;
	ver=atoi(&tmp[3]);
	
	i=0;
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
			tmp[i] = rdat;
			i++;
		}
		
	}
	tmp[12]=0;
	j=atoi(&tmp[11]);
	Aps_Num=j;
//	debugprintf("SCAN:Found=%d\r\n", j);

	UsbUartPuts("\r\n");
	UsbUartPuts("NumSSID               ChRSSI(dBm)Security Mode  CapabilitiesMAC Address       \r\n");
	UsbUartPuts("\r\n");
	
	for(i=0; i<j; i++) {
		sprintf(temp, "%02d ", i+1);
		UsbUartPuts(temp);
		m=0;
		while(1) {
			rdat = UartGetC(UART_CH6);
			UartPutCh(7, rdat);
			tmp[m++] = rdat;
			if(rdat==0x0a) {
				tmp[m-2] = 0;
				if(ver>23) p=42;
				else p=36;
				sprintf(Scan_Aps[i], "%s", &tmp[p]);
				UsbUartPuts(Scan_Aps[i]);
				m=strlen(&tmp[p]);
				for(n=m; n<19; n++) UsbUartPutCh(' ');
				UsbUartPuts("");

				tmp[5]=0;
				sprintf(temp, "%s", &tmp[3]);
				UsbUartPuts(temp);

				tmp[9]=0;
				sprintf(temp, "   %s   ", &tmp[6]);
				UsbUartPuts(temp);

				tmp[12]=0;
				n=strtoul(&tmp[10], &pt, 10);
				switch(n) {
					case 0: 	UsbUartPuts("Open           "); break;
					case 1: UsbUartPuts("WEP            "); break;
					case 2: UsbUartPuts("WPA-PSK        "); break;
					case 3: UsbUartPuts("WPA-Mixed      "); break;
					case 4: UsbUartPuts("WPA2-PSK       "); break;
					case 6: UsbUartPuts("WPA-Enterprise "); break;
					case 8: UsbUartPuts("WPA2-Enterprise"); break;
					default: break;
				}

				tmp[17]=0;
				n=strtoul(&tmp[13], &pt, 10);
				sprintf(temp, "    %04d    ", n);
				UsbUartPuts(temp);

				tmp[p-1]=0;
				if(ver>23) p=24;
				else p=18;
				sprintf(temp, "%s \r\n", &tmp[p]);
				UsbUartPuts(temp);
				break;
			}
		}
	}
	UsbUartPuts("\r\n");

	UartRxBufFlush(UART_CH6);
	sprintf(tmp, "set sys printlvl 0x%x\r", k);
	UartPuts(UART_CH6, tmp);
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0a) break;
		}
	}
	UartRxBufFlush(UART_CH6);
}

void Join_AP()
{
	int i,j,k;
	int ret;
	U8 tmp[200];
	char *pt;
	U8 rdat;
	int err;
	
	i=0; j=0;
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) return;
	}
	if(Aps_Num==0) {
		UsbUartPuts("Error!\r\n");
		UsbUartPuts("First of all, Excute Scan Command\r\n\r\n");
		return;
	}

JoinAPJmp:
	UsbUartPuts("\r\n");
	UsbUartPuts("NumSSID                 \r\n");
	UsbUartPuts("\r\n");

	for(i=0; i<Aps_Num; i++) {
		sprintf(tmp, "%02d ", i+1);
		UsbUartPuts(tmp);
		k=strlen(Scan_Aps[i]);
		UsbUartPuts(Scan_Aps[i]);
		for(j=k; j<21; j++) UsbUartPutCh(' ');
		UsbUartPuts("\r\n");
	}
	UsbUartPuts("\r\n");
	strcpy(tmp, "Select Join AP Number : ");
				
	k = readline(tmp);
	
	if(k<=0) {
		return;
	}
	else {
		k = strtoul(console_buffer, &pt, 10);
		if(*pt!=0) {
			UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
			goto JoinAPJmp;
		}
		if(k>=Aps_Num) {
			UsbUartPuts("Invalid Number is entered! Please Re-enter!\r\n");
			goto JoinAPJmp;
		}

		sprintf(tmp, "join %s\r", Scan_Aps[k-1]);
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, tmp);
		i=0; j=0; ret=0;
		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			if(err) {
				ret++;
				if(ret>0) {
					sprintf(tmp, "\t==> * %s joining fail!\r\n", Scan_Aps[k-1]);
					UsbUartPuts(tmp);
					return;
				}
				continue;
			}
			UartPutCh(7,rdat);
			tmp[i++] = rdat;
			if(rdat==0x0a) {
				pt=strstr(tmp, "Associated!");
				if(pt>0) break;
				i=0;
				j++;
			}
			if(j>6) {
				sprintf(tmp, "\t==> %s joining fail!\r\n", Scan_Aps[k-1]);
				UsbUartPuts(tmp);
				return;
			}
		}
		
		i=0; j=0; ret=0;
		while(1) {
			rdat=UartGetCC(UART_CH6, &err);
			if(err) {
				ret++;
				if(ret>0) {
					sprintf(tmp, "\t==> * %s joining fail!\r\n", Scan_Aps[k-1]);
					UsbUartPuts(tmp);
					return;
				}
				continue;
			}
			UartPutCh(7,rdat);
			tmp[i++] = rdat;
			if(rdat==0x0a) {
				pt=strstr(tmp, "GW=");
				if(pt>0) break;
				i=0;
				j++;
			}
			if(j>10) {
				sprintf(tmp, "\t==> %s joining fail!\r\n", Scan_Aps[k-1]);
				UsbUartPuts(tmp);
				return;
			}
		}		
		UartRxBufFlush(UART_CH6);

		sprintf(tmp, "\t==> OK!(%s joining success)\r\n", Scan_Aps[k-1]);
		UsbUartPuts(tmp);
		PrintLCD("-> Success, Join AP!\r\n");
	}
}

void Init_Weather_Info()
{
	U8 rdat;
	
	if(!cmd_in) {
		if(!EnterCmdMode()) return;
	}
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "set dns name www.google.com\r");
	UsbUartPuts("\t => Set DNS Site!\r\n");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat=='<') break;
			UartPutCh(7, rdat);
//			UsbUartPutCh(rdat);
		}
	}
	UartRxBufFlush(UART_CH6);

	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "set com remote GET$/ig/api?weather=seoul&hl=en\r");
	UsbUartPuts("\t => Set Com Remote!\r\n");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat=='<') break;
			UartPutCh(7, rdat);
//			UsbUartPutCh(rdat);
		}
	}
	UartRxBufFlush(UART_CH6);

	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "set ip proto 18\r");
	UsbUartPuts("\t => Set IP Proto!\r\n");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat=='<') break;
			UartPutCh(7, rdat);
//			UsbUartPutCh(rdat);
		}
	}
	UartRxBufFlush(UART_CH6);

	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "set ip host 0\r");
	UsbUartPuts("\t => Set IP Host!\r\n");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat=='<') break;
			UartPutCh(7, rdat);
//			UsbUartPutCh(rdat);
		}
	}
	UartRxBufFlush(UART_CH6);

	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "set ip remote 80\r");
	UsbUartPuts("\t => Set IP Remote!\r\n");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat=='<') break;
			UartPutCh(7, rdat);
//			UsbUartPutCh(rdat);
		}
	}
	UartRxBufFlush(UART_CH6);
	
}

void PrintCommandWiFly()
{
	UsbUartPuts("\r\n");
	UsbUartPuts("=========================================\r\n");
	UsbUartPuts("    <<<< WiFly Command Selection >>>>               \r\n");
	UsbUartPuts("-----------------------------------------\r\n");
	UsbUartPuts("   1. Weather & Time Display     -[A] \r\n");
	UsbUartPuts("   2. Set Time Parameters        -[B] \r\n");
	UsbUartPuts("   3. Get Time                   -[C] \r\n");
	UsbUartPuts("-----------------------------------------\r\n");
	UsbUartPuts("   4. Scan Command               -[P] \r\n");
	UsbUartPuts("   5. Join Command               -[Q] \r\n");
	UsbUartPuts("   6. Save Command               -[S] \r\n");
	UsbUartPuts("   7. Command Prompt             -[Z] \r\n");
	UsbUartPuts("=========================================\r\n");
	
}

int main()
{	
	U8 tmp[100];
	USB_RING_BUF* pubuf;
	U8 rdat;
	
	pubuf = &usbbuf;
	
	InitRingBuf_u(pubuf);
	
	CANfly_init();
	
	*(volatile U16*)0x80000408 = 0xFFF0;//BANK2 8Bit
	*R_PAF0 = 0xAAAA; // SRAM interface address, data 
	*R_PAF1 = 0x00FF; // P1.0, P1.1, P1.2, P1.3, UART6, UART7
	*R_PAF2 = 0xEFAF; // nCS2, nWE, nRE, P2.7, P2.5, P2.4, P2.1, P2.0
	*R_PAF3 = 0xFFF3; // EIRQ1, P3.0, P3.2~7
	*R_PAF4 = 0xAFFF; // TWI, P4.0~5
	// *R_PAF5 = 0xBFFF; // A18, P5.0~6
	*R_PAF5 = 0xFFFF; // P5.0~7
	*R_PAF6 = 0x0300;//PIO6.4
	
	*R_P5oDIR = (1<<7); //P5.7 Output
	*R_P5oHIGH = (1<<7); // LED2 OFF
	
	*R_P6oDIR = (1<<4);//PIO6.4 output
	*R_P6oLOW = (1<<4);//PIO6.4 high , off
	
	*R_P1oDIR = 0x3; //PIO1.0,1.1 output // for LCD_nReset(PIO1.0) and LCD BackLight(PIO1.1)
	*R_P1oLOW= (1<<1);// LCD BackLight OFF

	// *R_P1oHIGH = 1; // LCD_nRESET High
	// delayms(1);

	*R_P1oLOW = (1<<3);// RN-171 Reset
	
	InitInterrupt();
	UartConfig(7,115200,DATABITS_8,STOPBITS_1,UART_PARNONE);

	setinterrupt(INTNUM_USB,usb_isr);
	EnableInterrupt(INTNUM_USB,TRUE);

	//USB On, not USB ctrl
	*R_P6oHIGH = (1<<4);//PIO6.4 high , on
	delayms(10);
	
	usb_init();
	
	delayms(1000);
	delayms(1000);
	
	*R_PAF5 |= (1<<15);
	*R_PAF5 &= ~(1<<14); // A18

	delayms(1000);
	WiFly_UartBaudSetting();
	
	*R_P1oLOW = 1; // LCD_nRESET Low
	delayms(500);
	*R_P1oHIGH = 1; // LCD_nRESET High
	
	LCDInit();
	
	LCDcls(0,0,0);	// clear white
	delayms(500);
	
	SetFontColor(0, 0, 255, 135, 255, 60);
	SetFontSize(Font16, Font16);

	xp=0;
	yp=0;

	*R_P1oHIGH = (1<<3);// RN-171 Reset Release
	delayms(1000);
	if(GetAP_ConnectStatus(tmp)) {
		UsbUartPuts(" ==> Connected to AP(SSID - ");
		UsbUartPuts(tmp);
		UsbUartPuts(")\r\n\r\n");
	}
	else {
		UsbUartPuts(" ==> Not connected to AP\r\n\r\n");
	}

	Init_Weather_Info();

	while(1)
	{
		PrintCommandWiFly();
		UsbUartPuts("\r\nSelect Item : ");

		UartRxBufFlush(UART_CH6);
		rdat = UsbGetC();
		UsbUartPutCh(rdat);
		UsbUartPuts("\r\n");

		switch(rdat) {
			case 'a' : case 'A' :
				run_weather_demo();
				break;
			
			case 'b' : case 'B' :
				SetTimeServerParameters();
				break;

			case 'c' : case 'C' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "get time\r");
				UsbUartPuts("\r\n\t => [get time] Command!\r\n");
				while(1) {
					if(UartGetCh(UART_CH6, &rdat)) {
						if(rdat=='<') break;
						UsbUartPutCh(rdat);
					}
				}
				UartRxBufFlush(UART_CH6);
				break;

			case 'p' : case 'P' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				Scan_APs();
				break;

			case 'q' : case 'Q' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				Join_AP();
				break;

			case 's' : case 'S' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				UartRxBufFlush(UART_CH6);
				UartPuts(UART_CH6, "save\r");
				UsbUartPuts("\r\n\t => [save] Command!\r\n");
				UsbUartPuts("\r\n\t => OK!\r\n");
				UartRxBufFlush(UART_CH6);
				break;

			case 'z' : case 'Z' :
				if(!cmd_in) {
					if(!EnterCmdMode()) break;
				}
				while(1) {
					if(UsbGetCh(&rdat)) {
						if(rdat == 0x03) break;
						UartPutCh(UART_CH6, rdat);
					}
					if(UartGetCh(UART_CH6, &rdat)) {
						UsbUartPutCh(rdat);
					}
				}
				break;
				
			default :
				UsbUartPuts("\r\nInvalid Command!\r\n");
				break;
		}
		
	}
	return 0;
}
