
#include "cantus.h"
#include "driver/lcd_g2.h"
#include "driver/lcdutil_I2810.h"
#include "driver/font_ctrl.h"

#define UART_CH6	6

void CANfly_init()
{
	*R_PAF0 = 0xFFFF;  // GPIO 
	*R_PAF1 = 0x0FFF;  // UART7, GPIO
	*R_PAF2 = 0xFFFF;  // GPIO
	*R_PAF3 = 0xFFFF;  // GPIO
	*R_PAF4 = 0x0FFF;  // UART3, GPIO
	*R_PAF5 = 0xFFFF;  // GPIO
	*R_PAF6 = 0x030C;  // PIO6.1 -> USB-Communication mode, PIO6.4 -> USB Ctrl
	*R_P6oDIR |= (1<<4); // PIO6.4 output mode
}



#define USB_BUF_SIZE 512

typedef struct _USB_RING_BUF {
	U16 uRingBufRxCtr;     // Number of characters in the Rx ring buffer
	U8  *uRingBufRxInPtr;  // Pointer to where next character will be inserted
	U8  *uRingBufRxOutPtr; // Pointer from where next character will be extracted
	U8  uRingBufRx[USB_BUF_SIZE]; // Ring buffer character storage (Rx)
} USB_RING_BUF;

USB_RING_BUF usbbuf;

void InitRingBuf_u(USB_RING_BUF* pRing)
{
	pRing->uRingBufRxCtr    = 0;
	pRing->uRingBufRxInPtr  = &pRing->uRingBufRx[0];
	pRing->uRingBufRxOutPtr = &pRing->uRingBufRx[0];
}


#define R_USBLBOWC (volatile U32*)(0x80022c00 + 0x40)
#define R_USBHBOWC (volatile U32*)(0x80022c00 + 0x44)

U32 total_in_data_len;

void usb_isr(void)
{
	BYTE usb_data[64];
	volatile U32 len;
	volatile U32 k=0;
	USB_RING_BUF* pbuf = &usbbuf;
	BYTE ep_irq ;
    BYTE usb_irq ;
	// int i=0;
	// int re=-1;
	
	ep_irq  = *(volatile U8*)(__USBEI_REG);	
	usb_irq = *(volatile U8*)(__USBUI_REG);
		
		
	if(ep_irq & 1)//EP0_IRQ
	{
		// debugstring("1");
		ep0_isr();
		// debugstring("2");
	}
	if (ep_irq & (1<<1))//EP1_IRQ
	{
		// PRINTVAR(*R_USBLBOWC);
		// PRINTVAR(*R_USBHBOWC);
		// total_in_data_len = (*R_USBHBOWC & 0xFF)<<8 | (*R_USBLBOWC & 0xFF);		
		
		/*
		if(current_flag != end_flag)
		{
			xfer_len = read_usb(usb_data_buffer, 64);
		    current_flag = (current_flag + xfer_len);
		}
		else
		{
		    data overflow
		}
		
		*/
		k=0;
		len = read_usb(usb_data, len);
		while(len) {
			if(pbuf->uRingBufRxCtr<USB_BUF_SIZE) {
				pbuf->uRingBufRxCtr++;
				*pbuf->uRingBufRxInPtr=usb_data[k++];
				pbuf->uRingBufRxInPtr++;
				if(pbuf->uRingBufRxInPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
					pbuf->uRingBufRxInPtr = &pbuf->uRingBufRx[0];
				}
				
			}
			len--;
		}
	}
	
	if(usb_irq & USB_RESET_IRQ)
	{
		usb_reset();
	}
	else if ( usb_irq & USB_RESUME_IRQ )
	{
		usb_resume();
	}
	else if ( usb_irq & USB_SUSPEND_IRQ )
	{
		usb_suspend();
	}
	*(volatile U8*)__USBEI_REG = ep_irq;   // Clear Interrupt 
	*(volatile U8*)__USBUI_REG = usb_irq;  // Clear Interrupt

}

int UsbGetCh(U8* buf)
{
	int i=0;

	USB_RING_BUF* pbuf = &usbbuf;


	if (pbuf->uRingBufRxCtr > 0) { /* No, decrement character count */
		pbuf->uRingBufRxCtr--; /* No, decrement character count */
		buf[0] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
		/* Wrap OUT pointer     */
		if (pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
			pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
		}
		i=1;
	}

	return i;
}

U8 UsbGetC()
{
	U8 ch;
	
	while(1) {
		if(UsbGetCh(&ch)) break;
	}
	return ch;
}

int UsbGetData(U8* buf, int bufmax)
{
	int i=0;

	USB_RING_BUF* pbuf = &usbbuf;

	// CRITICAL_ENTER();
	for(i=0;i<bufmax;i++) {
		if (pbuf->uRingBufRxCtr > 0) { /* No, decrement character count */
			pbuf->uRingBufRxCtr--; /* No, decrement character count */
			buf[i] = *pbuf->uRingBufRxOutPtr++; /* Get character from buffer */
			/* Wrap OUT pointer     */
			if (pbuf->uRingBufRxOutPtr == &pbuf->uRingBufRx[USB_BUF_SIZE]) {
				pbuf->uRingBufRxOutPtr = &pbuf->uRingBufRx[0];
			}
		}
		else {
			break;
		}	
	}
	// CRITICAL_EXIT();

	return i;
}

extern U8 Font_X_siz;
extern U8 Font_Y_siz;
U32 xp=0,yp=0;
void FontDisplayLCD(char *fData, int siz)
{
	U8 ch;
	U16  txt;
	int pp;
	int i;
	U8 *str = fData;

	for(i=0; i<siz; i++) {
		ch = *str;
		str++;
		if(ch < 0x80) {
			txt = ch;
			if(txt == 0x0d) {
				xp = 0;
				yp += Font_Y_siz;
				LCDdrawbox(0,yp,320,yp+Font_Y_siz+Font_Y_siz,255,255,0);
				// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
			
				if(yp >= 240) {
					yp = 0;
					// LCDdrawbox(0,yp,320,yp+32,255,255,0);
					// LCDcls(255,255,0);
				}
				continue;
			}
			else if(txt == 0x0a) {
				continue;
			}
		}
		else {
			txt = (ch<<8) + (*str & 0xFF);
			str++;
		}
		
		if(xp >= 320) {
			xp = 0;
			yp += Font_Y_siz;
			LCDdrawbox(0,yp,320,yp+Font_Y_siz+Font_Y_siz,255,255,0);
			// if(yp+32>=240) LCDdrawbox(0,0,320,16,255,255,0);
		}
		if(yp >= 240) {
			yp = 0;
			// LCDdrawbox(0,yp,320,yp+32,255,255,0);
			// LCDcls(255,255,0);
		}
		pp = DisplayFont(xp, yp, txt);
		xp += pp;		
		
	}
}

void UartRxBufFlush(int ch)
{
	int ret;
	U8 tmp;
	do {
		ret = UartGetCh(ch, &tmp);
	} while(ret);
}

void UsbUartPutCh(U8 data)
{
	write_usb(&data, 1);	
}

void UsbUartPuts(U8 *data)
{
	int siz = strlen(data);
	int len=0;

	while(siz>0) {
		if(siz>64) len = 64;
		else len = siz;
		write_usb(data, len);
		data += len;
		siz -= len;
	}
	
}
#define CFG_CBSIZE 75
U8 console_buffer[CFG_CBSIZE];

static char * delete_char(char *buffer, char *p, int *colp, int *np, int plen)
{
	char *s;

	if(*np == 0) {
		return (p);
	}

	if(*(--p) == '\t') {			/* will retype the whole line	*/
		while(*colp > plen) {
			UsbUartPuts("\b \b");
			(*colp)--;
		}
		for(s=buffer; s<p; ++s) {
			if (*s == '\t') {
				UsbUartPuts("        "+((*colp) & 07));
				*colp += 8 - ((*colp) & 07);
			} else {
				++(*colp);
				UsbUartPutCh(*s);
			}
		}
	} else {
		UsbUartPuts("\b \b");
		(*colp)--;
	}
	(*np)--;
	return (p);
}


int readline(U8 *prompt)
{
	char *p = console_buffer;
	int	n = 0;				/* buffer index		*/
	int	plen = 0;			/* prompt length	*/
	int	col;				/* output column cnt	*/
	char c;

	/* print prompt */
	if(prompt) {
		plen = strlen(prompt);
		UsbUartPuts(prompt);
	}
	col = plen;

	for (;;) {
		c = UsbGetC();

		/*
		 * Special character handling
		 */
		switch (c) {
			case '\r':				/* Enter		*/
			case '\n':
				*p = '\0';
				UsbUartPuts("\r\n");
				return ((int)p - (int)console_buffer);

			case '\0':				/* nul			*/
				continue;

			case 0x03:				/* ^C - break		*/
				console_buffer[0] = '\0';	/* discard input */
				return (-1);

			case 0x15:				/* ^U - erase line	*/
				while (col > plen) {
					UsbUartPuts("\b \b");
					--col;
				}
				p = console_buffer;
				n = 0;
				continue;

			case 0x17:				/* ^W - erase word 	*/
				p=delete_char(console_buffer, p, &col, &n, plen);
				while ((n > 0) && (*p != ' ')) {
					p=delete_char(console_buffer, p, &col, &n, plen);
				}
				continue;

			case 0x08:				/* ^H  - backspace	*/
			case 0x7F:				/* DEL - backspace	*/
				p=delete_char(console_buffer, p, &col, &n, plen);
				continue;

			default:
				/*
				 * Must be a normal character then
				 */
				if (n < CFG_CBSIZE-2) {
					if(c == '\t') {	/* expand TABs		*/
						UsbUartPuts("        "+(col&07));
						col += 8 - (col&07);
					} else {
						++col;		/* echo input		*/
						UsbUartPutCh(c);
					}
					*p++ = c;
					++n;
				} else {			/* Buffer full		*/
					UsbUartPutCh('\a');
				}
		}
	}
}

int String_UartBaud(char num)
{
	int ret = -1;
	
	switch(num) {
		case '0' : ret=2400; break;
		case '1' : ret=4800; break;
		case '2' : ret=9600; break;
		case '3' : ret=19200; break;
		case '4' : ret=38400; break;
		case '5' : ret=57600; break;
		case '6' : ret=115200; break;
		case '7' : ret=230400; break;
		case '8' : ret=460800; break;
		case '9' : ret=921600; break;
		default : ret = -1; break;
	}
	
	return ret;
}

void WiFly_UartBaudSetting()
{
	int k;
	int len;
	U8 temp[75];
	
	while(1) {
		UsbUartPuts("\r\n");
		UsbUartPuts("         < Select WiFly(RN-171) Current UART BaudRate >\r\n");
		UsbUartPuts("\r\n");
		UsbUartPuts(" 0:2400, 1:4800, 2:9600, 3:19200, 4:38400, 5:57600, 6:115200\r\n");
		UsbUartPuts("\r\n");
		strcpy(temp, "Select Number - ");

		len = readline(temp);
				
		if(len<=0) {
			UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
		}
		else {
			if(strlen(console_buffer)>1) {
				UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
			}
			else {
				k = String_UartBaud(console_buffer[0]);
				if(k<0 || k>115200) {
					UsbUartPuts("Invalid Number is selected! Please Re-select!\r\n");
				}
				else {
					UartConfig(UART_CH6,k,DATABITS_8,STOPBITS_1,UART_PARNONE);
					UsbUartPuts("\r\n ==> WiFly(RN-171)'s uart baudrate is set to ");
					sprintf(temp, "%d\r\n", k);
					UsbUartPuts(temp);
					break;
				}
			}
		}
	}
}

int cmd_in = 0;

int EnterCmdMode(void)
{
	int i=0, j;
	U8 rdat;
	U8 temp[65];
	int cnt=0xfffff;

	UsbUartPuts("\r\n\t => Enter WiFly Command Mode!\r\n");
	for(j=0; j<5; j++) {
		UartRxBufFlush(UART_CH6);
		UartPuts(UART_CH6, "$$$");
		delayms(500);
		while(cnt>0) {
			if(UartGetCh(UART_CH6, &rdat)) {
				UartPutCh(7, rdat);
				if(rdat == 'C') {
					temp[i]=rdat;
					i++;
					goto Nnext;
				}
				else if(rdat == '$') {
					UsbUartPuts("Already came into CMD mode!\r\n");
					cmd_in = 1;
					return 1;
				}
			}
			cnt--;		
		}
		cnt = 0xfffff;
		UsbUartPuts("\r\n\t => reEnter WiFly Command Mode!\r\n");
	}
	if(j==5) {
		UsbUartPuts("\t => Fail!\r\n");
		cmd_in = 0;
		return 0;
	}

Nnext:
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7, rdat);
			if(rdat == 0x0d) break;
			temp[i]=rdat;
			i++;
		}
	}

	// UartPuts(7, usb_data_out);
	if(strncmp(temp, "CMD", 3) == 0) {
		UsbUartPuts("\t => OK!\r\n");
		cmd_in = 1;
	}
	else {
		UsbUartPuts("\t => Fail!\r\n");
		cmd_in = 0;
	}

	UartRxBufFlush(UART_CH6);
	
	return cmd_in;
}


int GetAP_ConnectStatus(U8 *ssid)
{
	int i=0;
	char rdat;
	int ret;
	U8 tmp[256];
	
	if(!cmd_in) {
		ret = EnterCmdMode();
		if(!ret) {
			UsbUartPuts("Fail!, Enter CMD Mode!\r\n");
			return 0;
		}
	}
	delayms(3000);
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show connection\r");

	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7,rdat);
			if(rdat == 0x0a) break;
		}
	}
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			UartPutCh(7,rdat);
			if(rdat == 0x0d) {
				tmp[i] = 0;
				break;
			}
			else tmp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	if(tmp[2]=='3' && tmp[3]!='3') ret=1;
	else ret=0;

	i=0;
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "show net\r");
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0a) break;
		}
	}
	while(1) {
		if(UartGetCh(UART_CH6, &rdat)) {
			if(rdat == 0x0d) {
				tmp[i] = 0;
				break;
			}
			else tmp[i] = rdat;
			i++;
		}
	}
	UartRxBufFlush(UART_CH6);

	strcpy(ssid, &tmp[5]);	
	
	return ret;
}

int main()
{	
	int i;
	int len;
	U8 usb_data_out[64];
	U8 u_buf[64];
	USB_RING_BUF* pubuf;
	
	pubuf = &usbbuf;
	
	InitRingBuf_u(pubuf);
	
	CANfly_init();
	
	*(volatile U16*)0x80000408 = 0xFFF0;//BANK2 8Bit
	*R_PAF0 = 0xAAAA; // SRAM interface address, data 
	*R_PAF1 = 0x00FF; // P1.0, P1.1, P1.2, P1.3, UART6, UART7
	*R_PAF2 = 0xEFAF; // nCS2, nWE, nRE, P2.7, P2.5, P2.4, P2.1, P2.0
	*R_PAF3 = 0xFFF3; // EIRQ1, P3.0, P3.2~7
	*R_PAF4 = 0xAFFF; // TWI, P4.0~5
	// *R_PAF5 = 0xBFFF; // A18, P5.0~6
	*R_PAF5 = 0xFFFF; // P5.0~7
	*R_PAF6 = 0x0300;//PIO6.4
	
	*R_P5oDIR = (1<<7); //P5.7 Output
	*R_P5oHIGH = (1<<7); // LED2 OFF
	
	*R_P6oDIR = (1<<4);//PIO6.4 output
	*R_P6oLOW = (1<<4);//PIO6.4 high , off
	
	*R_P1oDIR = 0x3; //PIO1.0,1.1 output // for LCD_nReset(PIO1.0) and LCD BackLight(PIO1.1)
	*R_P1oLOW= (1<<1);// LCD BackLight OFF

	// *R_P1oHIGH = 1; // LCD_nRESET High
	// delayms(1);
	
	
	
	InitInterrupt();
	UartConfig(7,115200,DATABITS_8,STOPBITS_1,UART_PARNONE);

	setinterrupt(INTNUM_USB,usb_isr);
	EnableInterrupt(INTNUM_USB,TRUE);
	
	//USB On, not USB ctrl
	*R_P6oHIGH = (1<<4);//PIO6.4 high , on
	delayms(10);
	
	usb_init();
	
	delayms(1000);
	delayms(1000);
	
	*R_PAF5 |= (1<<15);
	*R_PAF5 &= ~(1<<14); // A18
	
	delayms(1000);
	WiFly_UartBaudSetting();
	
	*R_P1oLOW = 1; // LCD_nRESET Low
	delayms(500);
	*R_P1oHIGH = 1; // LCD_nRESET High
	
	LCDInit();
	
	LCDcls(0,0,0);	// clear white
	delayms(500);
	
	LCDcls(255,255,0);	// back ground yellow
	delayms(500);
	
	SetFontColor(0, 0, 255, 135, 255, 60);
	SetFontSize(Font16, Font16);

	xp=0;
	yp=0;

	*R_P1oHIGH = (1<<3);// RN-171 Reset Release
	delayms(1000);
	if(GetAP_ConnectStatus(usb_data_out)) {
		UsbUartPuts(" ==> Connected to AP(SSID - ");
		UsbUartPuts(usb_data_out);
		UsbUartPuts(")\r\n\r\n");
	}
	else {
		UsbUartPuts(" ==> Not connected to AP\r\n\r\n");
	}
	
	UartRxBufFlush(UART_CH6);
	UartPuts(UART_CH6, "exit\r");
	
	while(1)
	{
		if((len = UsbGetData(u_buf, 64))>0) {
			for(i=0; i<len; i++)
				UartPutCh(6,u_buf[i]);
		}
		
		len=UartGetData(6,usb_data_out,64);
		
		if(len > 0)
		{
			write_usb(usb_data_out, len);
			FontDisplayLCD(usb_data_out, len);
		}
		
	}
	return 0;
}
