#include "cantus.h"
#include "driver/lcd_g2.h"

#define LCD_SUPPORT_FILESYSTEM
 

#ifdef LCD_SUPPORT_FILESYSTEM
 typedef struct tag_BITMAPFILEHEADER {
	U32   bfSize;
	U32    bfReserved; 
	U32   bfOffBits; 
	U32  biSize; 
	S32   biWidth; 
	S32   biHeight; 
	U16   biPlanes; 
	U16   biBitCount;
	U32  biCompression; 
	U32  biSizeImage; 
	S32   biXPelsPerMeter; 
	S32   biYPelsPerMeter; 
	U32  biClrUsed; 
	U32  biClrImportant; 
}BITMAPFILEHEADER;

   
typedef struct tag_RGBQUAD {
	U8    rgbBlue;
	U8    rgbGreen; 
    U8    rgbRed;
	U8    rgbReserved;
} RGBQUAD; 


BOOL drawBMPRect(char* fname,int startx, int starty,int dx,int dy)
{
	FIL fsrc;
	FIL* fp;
	FRESULT res;
	U32 br;
	ALIGN4 BITMAPFILEHEADER bmpfh;
	U32 w,h;
	U16 ID;
	
	fp = &fsrc;
	
	res = f_open(fp,fname,FA_OPEN_EXISTING | FA_READ);
	if(res != FR_OK)
	{
		debugprintf("file(%s) not found\r\n",fname);
		return FALSE;
	}
	f_read(fp,&ID,2,&br);
	f_read(fp,&bmpfh,sizeof(BITMAPFILEHEADER),&br);

	if(ID != 0x4d42) // "BM"
	{
		debugprintf("Invalid BMP-ID(%x)\r\n",ID);
		f_close(fp);
		return FALSE;
	}
	/*
	if (bmpfh.biCompression != 0 ) 
	{
		f_close(fp);
		return FALSE;
	}
	*/
	if( bmpfh.biBitCount == 24 || bmpfh.biBitCount == 8 || bmpfh.biBitCount == 4 )
	{
		debugstring("support only 16 bit bmp\r\n");
		f_close(fp);		
		return FALSE;
	}

	w = bmpfh.biWidth;
	h = bmpfh.biHeight;
	U32 ibpl; //line pitch
	int i,x, y;
	ALIGN4 U8 linebuf[256*3];
	ibpl = ((w*2) + 3) & 0xfffffffc;
	
	if(ibpl>512*3)
	{
		debugprintf("Image width is too big \r\n",w);
		f_close(fp);
		return FALSE;
	}
	
	LCDRamSet(startx,starty,startx+dx-1,starty+dy-1);
	volatile U8* frame=(volatile U8*)(LCD_BASE_ADDR|LCD_SEL_CTL_REG);
	
	for(y=dy;y>0;y--)
	{
		f_lseek(fp,bmpfh.bfOffBits+(y-1)*ibpl);		
		res = f_read(fp,linebuf,ibpl,&br);
		if(res || br==0)
			break;
		for(i=0,x=0;x<dx;x++)
		{
			*frame = linebuf[i+1];
			*frame = linebuf[i];
			i+=2;
		}
	}

	f_close(fp);
	return TRUE;
}

BOOL drawBMP(char* fname)
{
	FIL fsrc;
	FIL* fp;
	FRESULT res;
	
	U32 w,h;
	U16 ID;
	int end_x,end_y;
	U32 ibpl; //line pitch
	int i,x, y;
	U32 br;
	U16 temp;
	
	ALIGN4 BITMAPFILEHEADER bmpfh;
	ALIGN4 U8 linebuf[240*3];	
	
	RGBQUAD color_palette[256];
	
	fp = &fsrc;
	res = f_open(fp,fname,FA_OPEN_EXISTING | FA_READ);
	
	if(res != FR_OK)
	{
		debugprintf("file(%s) not found\r\n",fname);
		return FALSE;
	}
	
	f_read(fp,&ID,2,&br);
	f_read(fp,&bmpfh,sizeof(BITMAPFILEHEADER),&br);
	
	if(ID != 0x4d42) // "BM"
	{
		debugprintf("Invalid BMP-ID(%x)\r\n",ID);
		f_close(fp);
		return FALSE;
	}
	
	if( bmpfh.biBitCount == 4 )
	{
		debugstring("Not support 4bit bmp\r\n");
		f_close(fp);		
		return FALSE;
	}
	
	w = bmpfh.biWidth;
	h = bmpfh.biHeight;
	end_x = w>240?240:w;
	end_y = h>320?320:h;	
	
	if(bmpfh.biBitCount==8 && bmpfh.biClrUsed!=0)
	{
		f_read(fp, color_palette,256,&br);
		
		for(i=0,x=0;i<256;i++)
		{	
			color_palette[i].rgbRed=((color_palette[i].rgbRed&0xf8)|((color_palette[i].rgbGreen&0xfc)>>2&0x07));
			color_palette[i].rgbGreen=((color_palette[i].rgbGreen&0xfc)>>5|(color_palette[i].rgbBlue&0xf8));
		}
		
		ibpl = ((w*1) + 3) & 0xfffffffc;		
		
		if(ibpl>240*3)
		{
			debugprintf("Image width is too big \r\n",w);
			f_close(fp);
			return FALSE;
		}		
			
		LCDRamSet(0,0,end_x-1,end_y-1);		
		volatile U8* frame=(volatile U8*)(LCD_BASE_ADDR|LCD_SEL_CTL_REG);
		for(y=end_y;y>0;y--)
		{
			f_lseek(fp,bmpfh.bfOffBits+(y-1)*ibpl);
			res = f_read(fp,linebuf,ibpl,&br);
			if(res || br==0)
				break;
			
			for(i=0,x=0;x<end_x;x++)
			{
				*frame=(U8)color_palette[linebuf[i]].rgbRed;
				*frame=(U8)color_palette[linebuf[i]].rgbGreen;
				i++;
			}
		}
	}
	else if(bmpfh.biBitCount==16)
	{			
		ibpl = ((w*2) + 3) & 0xfffffffc;
		if(ibpl>240*3)
		{
			debugprintf("Image width is too big \r\n",w);
			f_close(fp);
			return FALSE;
		}
		
		LCDRamSet(0,0,end_x-1,end_y-1);
		volatile U8* frame=(volatile U8*)(LCD_BASE_ADDR|LCD_SEL_CTL_REG);
		
		if(bmpfh.biCompression==0)	// 555
		{
			for(y=end_y;y>0;y--)
			{
				f_lseek(fp,bmpfh.bfOffBits+(y-1)*ibpl);
				res = f_read(fp,linebuf,ibpl,&br);
				if(res || br==0)
					break;
				for(i=0,x=0;x<end_x;x++)
				{
					temp=(linebuf[i+1]<<8|linebuf[i])<<1;
					
					*frame=temp>>8;
					*frame=(linebuf[i]&0x1f)|(temp&0xc0);

					i+=2;
				}
			}
		}
		else
		{	
			for(y=end_y;y>0;y--)
			{
				f_lseek(fp,bmpfh.bfOffBits+(y-1)*ibpl);
				res = f_read(fp,linebuf,ibpl,&br);
				if(res || br==0)
					break;
				for(i=0,x=0;x<end_x;x++)
				{
					*frame = linebuf[i+1];
					*frame = linebuf[i];
					i+=2;
				}
			}
		}
	}
	else if(bmpfh.biBitCount==24) // 24bit color
	{
		ibpl = ((w*3) + 3) & 0xfffffffc;
		if(ibpl>240*3)
		{
			debugprintf("Image width is too big \r\n",w);
			f_close(fp);
			return FALSE;
		}
		
		LCDRamSet(0,0,end_x-1,end_y-1);
		volatile U8* frame=(volatile U8*)(LCD_BASE_ADDR|LCD_SEL_CTL_REG);
		
		for(y=end_y;y>0;y--)
		{
			f_lseek(fp,bmpfh.bfOffBits+(y-1)*ibpl);		
			res = f_read(fp,linebuf,ibpl,&br);
			if(res || br==0)
				break;
			for(i=0,x=0;x<end_x;x++)
			{
				*frame=(linebuf[i+2]&0xf8)|(linebuf[i+1]&0xfc)>>5;
				*frame=((linebuf[i+1]&0x1c)<<3) | linebuf[i]>>3;
				i+=3;
			}
		}
	}
	
	f_close(fp);
	return TRUE;
}
#endif

void drawVline(int x, int y,int dy,U8 r, U8 g, U8 b)
{
	LCDRamSet(x,y,x,y+dy);
    for (y=0; y<dy; y++)
    {
		*(volatile unsigned char *)(LCD_BASE_ADDR|LCD_SEL_CTL_REG)=(unsigned char)r;//red
		*(volatile unsigned char *)(LCD_BASE_ADDR|LCD_SEL_CTL_REG)=(unsigned char)(g);//green
		*(volatile unsigned char *)(LCD_BASE_ADDR|LCD_SEL_CTL_REG)=(unsigned char)(b);//blue
    }
}
void drawHline(int x, int y,int dx,U8 r, U8 g, U8 b)
{
	LCDRamSet(x,y,x+dx,y);
	for (x=0; x<dx; x++)
	{
		*(volatile unsigned char *)(LCD_BASE_ADDR|LCD_SEL_CTL_REG)=(unsigned char)r;//red
		*(volatile unsigned char *)(LCD_BASE_ADDR|LCD_SEL_CTL_REG)=(unsigned char)(g);//green
		*(volatile unsigned char *)(LCD_BASE_ADDR|LCD_SEL_CTL_REG)=(unsigned char)(b);//blue
	}
}

void drawboxline(int x, int y, int dx, int dy,U8 r, U8 g, U8 b)
{
	drawHline(x,y,dx,r,g,b);
	drawVline(x,y,dy,r,g,b);
	drawVline(x+dx,y,dy,r,g,b);
	drawHline(x,y+dx,dx,r,g,b);
}
