
#pragma once

/***********************************************
* NAND FLASH CONTROLLER
***********************************************/
#define R_NFMCON	(volatile unsigned int*)0x80000c00
#define R_NFMCMD	(volatile unsigned int*)0x80000c04
#define R_NFMADDR	(volatile unsigned int*)0x80000c08
#define R_NFMDATA	(volatile unsigned int*)0x80000c0c
#define R_NFMDMA	(volatile unsigned int*)0x80000c10
#define R_NFMSTAT	(volatile unsigned int*)0x80000c14
#define R_NFMECC	(volatile unsigned int*)0x80000c18
#define R_NFMCFG	(volatile unsigned int*)0x80000c1c
#define R_NFMECCLSN	(volatile unsigned int*)0x80000c20
#define R_NFMLSM	(volatile unsigned int*)0x80000c24
#define R_NFMHSM	(volatile unsigned int*)0x80000c2c
#define R_NFMLCECC0	(volatile unsigned int*)0x80000c34
#define R_NFMLCECC1	(volatile unsigned int*)0x80000c38
#define R_NFMERRLOC0	(volatile unsigned int*)0x80000c40
#define R_NFMERRLOC1	(volatile unsigned int*)0x80000c44
#define R_NFMERRLOC2	(volatile unsigned int*)0x80000c48
#define R_NFMERRLOC3	(volatile unsigned int*)0x80000c4c


// NAND Command Define
#define NAND_COMM_READ1	0x00	// NAND Read 1st Cycle
#define NAND_COMM_READ1_2	0x01	// NAND Read 2nd harf register
#define NAND_COMM_READ2	0x30	// NAND Read 2st Cycle
#define NAND_COMM_SPAREREAD	0x50 // NAND Read spare area
#define NAND_COMM_RID	0x90	// NAND Read ID
#define NAND_COMM_RST	0xff	// NAND Reset
#define NAND_COMM_PPRO1	0x80	// NAND Page Program 1st Cycle
#define NAND_COMM_PPRO2 0x10	// NAND Page Program 2st Cycle
#define NAND_COMM_BERS1	0x60	// NAND Block Erase 1st Cycle
#define NAND_COMM_BERS2	0xd0	// NAND Block Erase 2st Cycle
#define NAND_COMM_IRD	0x85	// NAND Random Data Input
#define NAND_COMM_OWD1	0x05	// NAND Random Data Output 1st Cycle
#define NAND_COMM_OWD2	0xe0	// NAND Random Data Output 2st Cycle
#define NAND_STATUS		0x70	// NAND Status Read


//sector means 512 byte memory
BOOL nand_phy_readpage(U32 pagenum, void* buf);
BOOL nand_phy_writepage(U32 pagenum, void* buf);
BOOL nand_phy_eraseblock(U32 blocknum);
BOOL nand_phy_readspare(U32 pagenum,void *buffer,U32 len);
BOOL nand_phy_writespare(U32 pagenum,void* buffer,U32 len);

BOOL nand_phy_readsect(U32 sector,void* buf);//read only 512 byte
BOOL nand_set_sparedata(U32 lognum);
void nand_set_cfg(U32 cfg);
BOOL nand_init();
U32 nand_get_pagesize();
U32 nand_get_blocksize();
U32 nand_get_pageperblock();
U32 nand_get_sectorcount();//sector is 512
U32 nand_get_blockcnt();
U32 nand_get_memsize_kbyte();
U32 nand_get_phy_block(U32 lognum);

U32 nand_eraseblock(U32 logblocknum);
BOOL nand_readpage(U32 pagenum,void* buf);
BOOL nand_writepage(U32 pagenum,void* buf);
void nand_flushdata();

// only 512byte r/w for FAT
BOOL nand_readsect(U32 sector,void* buf);
BOOL nand_readsects(U32 sector,void* buf,U32 cnt);
BOOL nand_updatesects(U32 sector,void* buf,U32 cnt);
